% adaptiveHypsing provides the implementation of an adaptive mesh-refining 
% algorithm for the hypersingular integral equation. 
%
% Usage: adaptiveHypsing
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the hypersingular integral
%              equation
%
%                W*u = (K'-1/2)*phiN
%
%              which is equivalent to the Neumann problem
% 
%                -Laplace(u) = 0 in Omega with du/dn = phiN on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape1 with an
%              L-shaped domain Omega and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) refine given mesh Tcoarse uniformly and obtain Tfine
%                (2) build Galerkin data w.r.t. Tfine
%                (3) compute Galerkin solution w.r.t. Tfine
%                (4) compute Tcoarse-elementwise contributions of mu-tilde
%                    and Dirichlet data oscillations
%                (5) use Doerfler strategy to mark certain Tcoarse-elements
%                (6) refine marked elements to obtain new mesh Tcoarse
%                (7) iterate as long as #Tcoarse is lower than given bound
%
%              After the computation, the Galerkin solution uh as well as the
%              exact solution u are plotted.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter 
theta = 0.25;
rho = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d\r',size(elements,1))

    %*** build uniformly refined mesh
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    %*** discretize Neumann data and compute data oscillations
    [osc_fine,phih_fine] ...
        = computeOscNeumann(coordinates_fine,elements_fine,@phi);
    osc = osc_fine(father2son(:,1)) + osc_fine(father2son(:,2));

    %*** compute fine-mesh solution
    W_fine = buildW(coordinates_fine,elements_fine) ...
             + buildHypsingStabilization(coordinates_fine,elements_fine);
    b_fine = buildHypsingRHS(coordinates_fine,elements_fine,phih_fine);
    x_fine = W_fine\b_fine;
    
    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde = computeEstHypMuTilde(elements_fine,elements,father2son,...
                                         x_fine);
    
    %*** mark elements for refinement
    marked = markElements(theta,rho,mu_tilde + osc);

    %*** generate new mesh
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements,marked);

    if size(elements,1) > nEmax
      break;
    end
end

%*** visualize exact and adaptively computed solution
plotArclengthS1(coordinates_fine,elements_fine,x_fine,@g);
