% adaptiveSymmHierarchical provides the implementation of an adaptive 
% mesh-refining algorithm for the hypersingular integral equation. 
%
% Usage: adaptiveSymmHierarchical
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the hypersingular integral
%              equation
%
%                W*u = (K'-1/2)*phiN
%
%              which is equivalent to the Neumann problem
% 
%                -Laplace(u) = 0 in Omega with du/dn = phiN on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape1 with an
%              L-shaped domain Omega and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) initialize mesh Tcoarse
%                (2) build Galerkin data w.r.t. Tcoarse
%                (3) compute Galerkin solution w.r.t. Tcoarse
%                (4) compute E_ell-elementwise contributions of two-level
%                    error estimator tau and Neumann data oscillations
%                    (To that end build integral operator and RHS w.r.t.
%                    the uniformly refined mesh Tfine)
%                (5) use Doerfler strategy to mark certain Tcoarse-elements
%                (6) refine marked elements to obtain new mesh Tcoarse
%                (7) iterate as long as #Tcoarse is lower than given bound
%
%              After the computation, the Galerkin solution uh as well as the
%              exact solution u are plotted.
%
% (C) 2010-2012 HILBERT-Team '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');

coordinates = load('coordinates.dat')/4;
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
[coordinates_fine,elements_fine,father2son_fine] ...
    = refineBoundaryMesh(coordinates,elements);

%*** discretize data and compute corresponding data oscillations for coarse 
%*** and fine mesh
[osc,phiNh] = computeOscNeumann(coordinates,elements,@phi);
[osc_fine,phiNh_fine] = computeOscNeumann(coordinates_fine,elements_fine,@phi);
osc = sum(osc_fine(father2son_fine),2);

err=[];
est=[];
nEll=[];
%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d\r', ...
        size(elements,1));

    %*** compute integral operators
    %fine mesh
    W_fine = buildW(coordinates_fine,elements_fine) ...
             + buildHypsingStabilization(coordinates_fine,elements_fine);
    %coarse mesh
    W = buildW(coordinates,elements) ...
             + buildHypsingStabilization(coordinates,elements);
    
    %*** compute right-hand side
    %fine mesh
    b_fine = buildHypsingRHS(coordinates_fine,elements_fine,phiNh_fine);
    
    %*** build index field k = coarse2fine (j) such that j-th node of coarse
    %*** mesh coincides with k-th node of fine mesh
    nC = size(coordinates,1);
    coarse2fine = zeros(nC,1);
    coarse2fine(elements) = [ elements_fine(father2son_fine(:,1),1) ...
                              elements_fine(father2son_fine(:,2),2) ];
                          
    %*** build index field k = element2midpoint(j) such that midpoint of j-th
    %*** element of coarse mesh is k-th node of fine mesh
    element2midpoint = elements_fine(father2son_fine(:,1),2);
    
    %*** build index field [i,j] = node2element(k) such that
    %*** intersection(Ei,Ej) = zk
    nE_ = size(elements,1);
    node2element = zeros(nC,2);
    node2element(elements(:,1),2) = (1:nE_)';
    node2element(elements(:,2),1) = (1:nE_)';
    
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b = b_fine(coarse2fine) ...
        + 0.5*sum(b_fine(element2midpoint(node2element)),2);

    %*** compute Galerkin solution for coarse mesh
    x = W\b;

    %*** stopping criterion
    if (size(elements,1)  > nEmax )
      break;
    end

    %*** compute error estimates
    tau = computeEstHypTau(elements_fine,elements,father2son_fine,...
        W_fine,b_fine,x);
       
    %*** mark elements for refinement
    [marked] = markElements(theta,tau + osc);
    
    %*** generate new mesh
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements,marked);

    %*** generate fine mesh
    [coordinates_fine,elements_fine,father2son_fine] = ...
        refineBoundaryMesh(coordinates,elements);

    %*** discretize data and compute corresponding data oscillations for coarse
    %*** and fine mesh
    [osc,phiNh] = computeOscNeumann(coordinates,elements,@phi);
    [osc_fine,phiNh_fine] ...
        = computeOscNeumann(coordinates_fine,elements_fine,@phi);
    osc = sum(osc_fine(father2son_fine),2); 
end

%*** visualize exact and adaptively computed solution
plotArclengthS1(coordinates,elements,x,@g,1)
