% adaptiveHypsingResidualIndirect provides the implementation of an adaptive
% mesh-refining algorithm for the hypersingular integral equation by use of the
% weighted residual error estimator
%
% Usage: adaptiveHypsingResidualIndirect
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the hypersingular integral equation
%
%                Wu = phi
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves the so-called slit-problem,
%              where Gamma = (-1,1) x {0} and phi(x,y) = 1. The exact solution
%              phi is then known explicitely,
%
%                phi(x,y) = 2*(1-x^2)^{1/2}.
%
%              which exhibits generic singularities at the endpoints of Gamma.
%              Given an initial mesh, the adaptive algorithm implemented below
%              reads as follows:
%
%                (1) compute Galerkin solution w.r.t. T
%                (2) compute T-elementwise contributions of rho and
%                    Dirichlet data oscillations
%                (3) use Doerfler strategy to mark certain T-elements
%                (4) refine marked elements to obtain new mesh T
%                (5) iterate as long as #T is lower than given bound
%
%              After the computation, the Galerkin solution uh as well as the
%              exact solution u are plotted.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** define computational domain
coordinates = [-1 0; 1 0];
elements = [1 2];
[coordinates,elements] = refineBoundaryMesh(coordinates,elements);

%*** define function handle for phi = 1
phi = @(x,a,b) ones(size(x,1),1);

%*** define exact solution u
u = @(x) 2*sqrt(1-x(:,1).^2);

%*** maximal number of elements
nEmax = 100;

%*** adaptivity parameter 
theta = 0.25;
theta_min = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d\r',size(elements,1))

    %*** discretize Dirichlet data and compute data oscillations
    [osc_,phih] = computeOscNeumann(coordinates,elements,phi);

    %*** compute solution
    W = buildW(coordinates,elements);
    M = buildM(coordinates,elements)';
    b = M*phih;
    free = [3:size(coordinates,1)];
    x = W(free,free)\b(free);
    x=[0;0;x];

    [co_fine,el_fine,f2s] = refineBoundaryMesh(coordinates,elements);
    [osc_fine,phih_fine] = computeOscNeumann(co_fine,el_fine,phi);
    W_fine = buildW(co_fine,el_fine);
    M_fine = buildM(co_fine,el_fine)';
    b_fine = M_fine*phih_fine;
    free_fine = [3:size(co_fine,1)];
    x_fine = W_fine(free_fine,free_fine)\b_fine(free_fine);
    x_fine = [0;0;x_fine];
    
    %*** compute weighted residual-error estimator rho
    rho_ = computeEstHypResidual(coordinates,elements,x,[],phih);
    muTilde_ = computeEstHypMuTilde(el_fine,elements,f2s,x_fine);

    %*** compute error
    err_ = computeErrDirichlet(coordinates,elements,x,u);
    
    %*** mark elements for refinement
    marked = markElements(theta,theta_min,muTilde_ + osc_);

    %*** generate new mesh
    [coordinates_new,elements_new] = refineBoundaryMesh( ...
      coordinates,elements,marked);
    
    if size(elements,1) > nEmax
      break;
    end
    coordinates = coordinates_new;
    elements = elements_new;
end

%*** visualize exact and adaptively computed solution
plotArclengthS1(coordinates,elements,x,u,1);
