% adaptiveMixed provides the implementation of an adaptive mesh-refining
% algorithm for the symmetric integral formulation of a mixed boundary value
% problem.
%
% Usage: adaptiveMixed
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the symmetric integral formulation 
%              of a mixed boundary value problem
%
%                -Laplace(u)  = 0    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              using the functionality provided by the HILBERT program package.
%              As example serves problem lshape1 with an L-shaped domain Omega
%              and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) refine given mesh Tcoarse uniformly and obtain Tfine
%                (2) build Galerkin data w.r.t. Tfine
%                (3) compute Galerkin solution w.r.t. Tfine
%                (4) compute Tcoarse-elementwise contributions of mu-tilde and
%                    Dirichlet data oscillations
%                (5) use Doerfler strategy to mark certain Tcoarse-elements
%                (6) refine marked elements to obtain new mesh Tcoarse
%                (7) iterate as long as #Tcoarse is lower than given bound
%
%              After the computation, the discrete Cauchy data (uDh,phih) are 
%              plotted over the arclength. Here, the discrete function uDh is
%              an S1(Gamma)-approximation of the trace uD := u|_Gamma, whereas
%              phih is a P0(Gamma)-approximation of the normal derivative
%              phiN := du/dn on Gamma.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = elements([1,2,3,4],:);
neumann = elements([5,6,7,8],:);

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** rearrange indices such that Neumann nodes are first
[coordinates,neumann,dirichlet] = ...
    buildSortedMesh(coordinates,neumann,dirichlet);

%*** initialize Dirichlet data
uDh = zeros(size(coordinates,1),1);
uDh(unique(dirichlet)) = g(coordinates(unique(dirichlet),:));

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
    [coordinates_fine,dirichlet_fine,neumann_fine,...
        father2dirichlet_fine,father2neumann_fine] ...
        = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** rearrange indices such that Neumann nodes are first
[coordinates_fine,neumann_fine,dirichlet_fine] = ...
    buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

%*** discretize data and compute corresponding data oscillations for fine mesh
    [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          father2neumann_fine,neumann,uDh,@g,@phi);    

oscD = sum(oscD_fine(father2dirichlet_fine),2);
oscN = sum(oscN_fine(father2neumann_fine),2);

%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d\r',size(neumann,1)+size(dirichlet,1))

    %*** compute integral operators for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);  
    W_fine = buildW(coordinates_fine,elements_fine);

    %*** compute right-hand side for fine mesh
    [b_nodes_fine,b_elements_fine] ... 
        = buildMixedRHS(coordinates_fine,dirichlet_fine, ...
        neumann_fine,V_fine,K_fine,W_fine,uDh_fine,phiNh_fine);
                                  
    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nN_fine = length(freeNeumann_fine);

    %*** shrink integral operators and right-hand side
    W_fine = W_fine(freeNeumann_fine,freeNeumann_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    b_nodes_fine = b_nodes_fine(freeNeumann_fine);
    b_elements_fine = b_elements_fine(freeDirichlet_fine);

    %*** compute Galerkin solution for fine mesh
    x = [W_fine K_fine' ; -K_fine V_fine] \ [b_nodes_fine ; b_elements_fine];

    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    xN_fine = zeros(nC_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine);      %** dof on Neumann boundary
    xD_fine = x((1:nD_fine) + nN_fine);            %** dof on Dirichlet boundary
   
    %*** stopping criterion
    if (size(neumann,1) + size(dirichlet,1) > nEmax )
      break;
    end

    %*** compute (h-h/2)-error estimator tilde-mu on the associated boundaries
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet, ...
        father2dirichlet_fine,xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann, ...
        father2neumann_fine,xN_fine);

    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann] ...
        = markElements(theta,rho,muD_tilde + oscD,muN_tilde + oscN);
    
    %*** generate new mesh
    [coordinates,dirichlet,neumann,father2dirichlet_adap,father2neumann_adap]...
        = refineBoundaryMesh(coordinates,dirichlet,neumann,...
                             marked_dirichlet,marked_neumann);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates,neumann,dirichlet] = ...
        buildSortedMesh(coordinates,neumann,dirichlet);

    neumann_coarse = neumann_fine;
    father2neumann_coarse = father2neumann_fine;
    [coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine,...
        father2neumann_fine]= refineBoundaryMesh(coordinates,dirichlet,neumann);

    %*** build coarse2fine array
    coarse2fine = generateFather2Son(father2neumann_adap, ...
                                     father2neumann_coarse, ...
                                     father2neumann_fine);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates_fine,neumann_fine,dirichlet_fine] = ...
        buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

    %*** discretize data and compute corresponding data oscillations for
    %*** fine mesh
    [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          coarse2fine,neumann_coarse,uDh_fine,@g,@phi);    
    oscD = sum(oscD_fine(father2dirichlet_fine),2);
    oscN = sum(oscN_fine(father2neumann_fine),2);
end

%*** xN_fine = dof on Neumann boundary, i.e. update for Dirichlet data
%*** to obtain approximation uh of trace of PDE solution u
uh =  xN_fine+uDh_fine;

%*** plot approximate vs exact trace
plotArclengthS1(coordinates_fine,[dirichlet_fine;neumann_fine],uh,@g,2)

%*** xD_fine = dof on Dirichlet boundary, i.e. update for Neumann data
%*** to obtain approximation phih of normal derivative of PDE solution u
phih = [xD_fine ; phiNh_fine(nD_fine+1:end)];

%*** plot approximate vs exact normal derivative
plotArclengthP0(coordinates_fine,[dirichlet_fine;neumann_fine],phih,@phi,1)

