% adaptiveMixedHierarchical provides the implementation of an adaptive 
% mesh-refining algorithm for the symmetric integral formulation of a mixed 
% boundary value problem.
%
% Usage: adaptiveMixedHierarchical
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the symmetric integral formulation 
%              of a mixed boundary value problem
%
%                -Laplace(u)  = 0    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              using the functionality provided by the HILBERT program package.
%              As example serves problem lshape1 with an L-shaped domain Omega
%              and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) initialize mesh Tcoarse
%                (2) build Galerkin data w.r.t. Tcoarse
%                (3) compute Galerkin solution w.r.t. Tcoarse
%                (4) compute Tcoarse-elementwise contributions of
%                    tau_ell (to that end, build integral operators
%                    with respect to uniformly refined mesh Tfine)
%                (5) use Doerfler strategy to mark certain
%                    Tcoarse-elements
%                (6) refine marked elements to obtain new mesh
%                    Tcoarse
%                (7) iterate as long as #Tcoarse is lower than
%                    given bound
%               
%              After the computation, the discrete Cauchy data (uh,phih) are 
%              plotted over the arclength. Here, the discrete function uh is
%              an S1(Gamma)-approximation of the trace uD := u|_Gamma, whereas
%              phih is a P0(Gamma)-approximation of the normal derivative
%              phiN := du/dn on Gamma.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = [1 2;2 3;3 6;6 5];
neumann = [5 8;8 7;7 4;4 1];

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
[coordinates_fine,dirichlet_fine,neumann_fine,...
    father2dirichlet_fine,father2neumann_fine] ...
    = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** rearrange indices such that Neumann nodes are first
[coordinates_fine,neumann_fine,dirichlet_fine] = ...
    buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

%*** discretize data and compute corresponding data oscillations for coarse 
%*** and fine mesh
[oscD,oscN,uDh,phiNh] ...
        = computeOscMixed(coordinates,dirichlet,neumann,@g,@phi);
[oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          father2neumann_fine,neumann,uDh,@g,@phi);
oscD = sum(oscD_fine(father2dirichlet_fine),2);
oscN = sum(oscN_fine(father2neumann_fine),2);
err=[];
est=[];
nEll=[];

%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d\r', ...
        size(neumann,1)+size(dirichlet,1));

    %*** compute integral operators
    %*** fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);  
    W_fine = buildW(coordinates_fine,elements_fine);
    
    %*** coarse mesh
    elements = [dirichlet;neumann];
    V = buildV(coordinates,elements);
    K = buildK(coordinates,elements);  
    W = buildW(coordinates,elements);
    
    %*** compute degrees of freedom
    %*** fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nN_fine = length(freeNeumann_fine);
    
    %*** coarse mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;
    nN = length(freeNeumann);
    
    %*** compute right-hand side
    %*** fine mesh
    [b_nodes_fine,b_elements_fine] = buildMixedRHS(coordinates_fine,...
        dirichlet_fine,neumann_fine,V_fine,K_fine,W_fine,uDh_fine,phiNh_fine);
    
    father2son = [father2dirichlet_fine ; father2neumann_fine + nD_fine];
    
    %*** build index field k = coarse2fine (j) such that j-th node of coarse
    %*** mesh coincides with k-th node of fine mesh
    nC = size(coordinates,1);
    coarse2fine = zeros(nC,1);
    coarse2fine(elements) = [ elements_fine(father2son(:,1),1) ...
                              elements_fine(father2son(:,2),2) ];
                          
    %*** build index field k = element2midpoint(j) such that midpoint of j-th
    %*** element of coarse mesh is k-th node of fine mesh
    element2midpoint = elements_fine(father2son(:,1),2);
    
    %*** build index field [i,j] = node2element(k) such that
    %*** intersection(Ei,Ej) = zk
    nE_ = size(elements,1);
    node2element = zeros(nC,2);
    node2element(elements(:,1),2) = (1:nE_)';
    node2element(elements(:,2),1) = (1:nE_)';
    
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b_nodes = b_nodes_fine(coarse2fine) ...
        + 0.5*sum(b_nodes_fine(element2midpoint(node2element)),2);
    b_elements = ...
        b_elements_fine(father2son(:,1))+b_elements_fine(father2son(:,2));
    
    %*** shrink integral operators and right-hand side
    %*** fine mesh
    W_fine = W_fine(freeNeumann_fine,freeNeumann_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    b_nodes_fine = b_nodes_fine(freeNeumann_fine);
    b_elements_fine = b_elements_fine(freeDirichlet_fine);
    
    %*** coarse mesh
    W = W(freeNeumann,freeNeumann);
    K = K(freeDirichlet,freeNeumann);
    V = V(freeDirichlet,freeDirichlet);
    b_nodes = b_nodes(freeNeumann);
    b_elements = b_elements(freeDirichlet);
   
    %*** compute Galerkin solution for coarse mesh
    x = [ W K' ; -K V ] \ [ b_nodes ; b_elements ];

    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    xN = zeros(nC,1);
    xN(freeNeumann) = x(1:nN);      %** dof on Neumann boundary
    xD = x((1:nD) + nN);            %** dof on Dirichlet boundary
   
    %*** stopping criterion + size(triangles,1)
    if (size(neumann,1) + size(dirichlet,1)  > nEmax )
      break;
    end

    %*** compute error estimates
    tau = computeEstMixTau(father2dirichlet_fine,father2neumann_fine, ...
        neumann,neumann_fine,V_fine,K_fine,W_fine,b_elements_fine, ...
        b_nodes_fine,x,freeNeumann,freeNeumann_fine);
	tauN=tau(1:size(neumann,1));
    tauD=tau(size(neumann,1)+1:end);
    
    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann] = markElements(theta,tauD + oscD, ...
        tauN + oscN );
    
    %*** generate new mesh
    neumann_old=neumann;
    [coordinates,dirichlet,neumann,father2dirichlet,father2neumann_adap] ...
	= refineBoundaryMesh(coordinates,dirichlet,neumann, ...
	             marked_dirichlet,marked_neumann);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates,neumann,dirichlet] = ...
        buildSortedMesh(coordinates,neumann,dirichlet);

    %*** generate fine mesh
    neumann_coarse = neumann_fine;
    father2neumann_coarse = father2neumann_fine;
    [coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine, ...
        father2neumann_fine] = ...
        refineBoundaryMesh(coordinates,dirichlet,neumann);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates_fine,neumann_fine,dirichlet_fine] = ...
        buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

    %*** build coarse2fine array
    coarse2fine = generateFather2Son(father2neumann_adap, ...
                                     father2neumann_coarse, ...
                                     father2neumann_fine);

    %*** discretize data and compute corresponding data oscillations
    %*** fine mesh
    [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          coarse2fine,neumann_coarse,uDh_fine,@g,@phi);
    
    %*** coarse mesh         
    [oscD,oscN,uDh,phiNh]=computeOscMixed(coordinates,dirichlet,neumann, ...
        father2neumann_adap,neumann_old,uDh,@g,@phi);	  
    oscD = sum(oscD_fine(father2dirichlet_fine),2);
    oscN = sum(oscN_fine(father2neumann_fine),2);
end

%*** obtain approximation uh of trace of PDE solution u
uh =  xN+uDh;

%*** plot approximate vs exact trace
plotArclengthS1(coordinates,[dirichlet;neumann],uh,@g,2)

%*** obtain approximation phih of normal derivative of PDE solution u
phih = [xD ; phiNh(nD+1:end)];

%*** plot approximate vs exact normal derivative
plotArclengthP0(coordinates,[dirichlet;neumann],phih,@phi,1)
