% adaptiveMixedResidual provides the implementation of an adaptive 
% mesh-refining algorithm for the symmetric integral formulation of a mixed 
% boundary value problem.
%
% Usage: adaptiveMixedResidual
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the symmetric integral formulation 
%              of a mixed boundary value problem
%
%                -Laplace(u)  = 0    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              using the functionality provided by the HILBERT program package.
%              As example serves problem lshape1 with an L-shaped domain Omega
%              and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) build Galerkin data w.r.t. T
%                (3) compute Galerkin solution w.r.t. T
%                (4) compute T-elementwise contributions of
%                    rho as well as data oscillations
%                (5) use Doerfler strategy to mark certain
%                    T-elements
%                (6) refine marked elements to obtain new mesh T
%                (7) iterate as long as #T is lower than
%                    given bound
%
%              After the computation, the discrete Cauchy data (uh,phih) are 
%              plotted over the arclength. Here, the discrete function uh is
%              an S1(Gamma)-approximation of the trace uD := u|_Gamma, whereas
%              phih is a P0(Gamma)-approximation of the normal derivative
%              phiN := du/dn on Gamma.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0
addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = elements([1,2,3,4],:);
neumann = elements([5,6,7,8],:);

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** extract boundary mesh
[coordinates,neumann,dirichlet] = ...
    buildSortedMesh(coordinates,neumann,dirichlet);

%*** discretize data and compute corresponding data oscillations for fine mesh
[oscD,oscN,uDh,phiNh] = computeOscMixed(coordinates,dirichlet,neumann,@g,@phi);

%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d\r', ...
        size(neumann,1)+size(dirichlet,1));

    %*** compute integral operators for fine mesh
    elements = [dirichlet;neumann];
    V = buildV(coordinates,elements);
    K= buildK(coordinates,elements);  
    W = buildW(coordinates,elements);

    %*** compute right-hand side for fine mesh
    [b_nodes,b_elements] = buildMixedRHS(coordinates,dirichlet,neumann, ...
        V,K,W,uDh,phiNh);
                                  
    %*** compute degrees of freedom for fine mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;
    nN = length(freeNeumann);

    %*** shrink integral operators and right-hand side
    W = W(freeNeumann,freeNeumann);
    K = K(freeDirichlet,freeNeumann);
    V = V(freeDirichlet,freeDirichlet);
    b_nodes = b_nodes(freeNeumann);
    b_elements = b_elements(freeDirichlet);

    %*** compute Galerkin solution for fine mesh
    x = [ W K' ; -K V ] \ [ b_nodes ; b_elements];

    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    xN = zeros(nC,1);
    xN(freeNeumann) = x(1:nN);      %** dof on Neumann boundary
    xD = x((1:nD) + nN);            %** dof on Dirichlet boundary
    uNh =  xN+uDh;
    phiDh = [xD ; phiNh(nD+1:end)];
   
    %*** stopping criterion + size(triangles,1)
    if (size(neumann,1) + size(dirichlet,1) > nEmax )
      break;
    end

    %*** compute error estimates
    [indD,indN]=computeEstMixResidual(coordinates,dirichlet,neumann, ...
        xN,xD,uDh,phiNh);
    indD = indD(1:nD);
    indN = indN(nD+1:end);
	     
    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann] = markElements(theta,indD + oscD, ...
        indN + oscN);
    
    %*** generate new mesh
    [coordinates,dirichlet,neumann,father2dirichlet,father2neumann_adap] = ...
        refineBoundaryMesh(coordinates,dirichlet,neumann, ...
        marked_dirichlet,marked_neumann);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates,neumann,dirichlet] = ...
        buildSortedMesh(coordinates,neumann,dirichlet);

    %*** discretize data and compute corresponding data oscillations for
    %*** fine mesh
    [oscD,oscN,uDh,phiNh] = computeOscMixed(coordinates,dirichlet,neumann, ...
                                            @g,@phi);	  
end

%*** obtain approximation uh of trace of PDE solution u
uh =  xN+uDh;

%*** plot approximate vs exact trace
plotArclengthS1(coordinates,[dirichlet;neumann],uh,@g,2)

%*** obtain approximation phih of normal derivative of PDE solution u
phih = [xD ; phiNh(nD+1:end)];

%*** plot approximate vs exact normal derivative
plotArclengthP0(coordinates,[dirichlet;neumann],phih,@phi,1)

% Author: Michael Feischl
% 
% History: MF 6.12.2011 initial version
%          TF 5.06.2011 -> fixed some comments, removed convergence plot,
%                          fixed some notational issues
