% adaptiveSymm provides the implementation of an adaptive mesh-refining 
% algorithm for Symm's integral equation. 
%
% Usage: adaptiveSymm
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for Symm's integral equation
%
%                V*phi = (K+1/2)*uD
%
%              which is equivalent to the Dirichlet problem
% 
%                -Laplace(u) = 0 in Omega with u = uD on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape1 with an
%              L-shaped domain Omega and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) refine given mesh Tcoarse uniformly and obtain Tfine
%                (2) build Galerkin data w.r.t. Tfine
%                (3) compute Galerkin solution w.r.t. Tfine
%                (4) compute Tcoarse-elementwise contributions of mu-tilde and
%                    Dirichlet data oscillations
%                (5) use Doerfler strategy to mark certain Tcoarse-elements
%                (6) refine marked elements to obtain new mesh Tcoarse
%                (7) iterate as long as #Tcoarse is lower than given bound
%
%              After the computation, the Galerkin solution phih as well as the
%              exact solution phi are plotted.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter 
theta = 0.25;
rho = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d\r',size(elements,1))

    %*** build uniformly refined mesh
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    %*** discretize Dirichlet data and compute data oscillations
    [osc_fine,uDh_fine] = computeOscDirichlet(coordinates_fine,elements_fine,@g);
    osc = osc_fine(father2son(:,1)) + osc_fine(father2son(:,2));

    %*** compute fine-mesh solution
    V_fine = buildV(coordinates_fine,elements_fine);
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
    x_fine = V_fine\b_fine;
    
    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde = computeEstSlpMuTilde(coordinates,elements,father2son, ...
                                           x_fine);
    
    %*** mark elements for refinement
    marked = markElements(theta,rho,mu_tilde + osc);

    %*** generate new mesh
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements,marked);
    
    if size(elements,1) > nEmax
      break;
    end
end

%*** visualize exact and adaptively computed solution
plotArclengthP0(coordinates_fine,elements_fine,x_fine,@phi,1);
