% adaptiveSymmHierarchical provides the implementation of an adaptive 
% mesh-refining algorithm for Symm's integral equation. 
%
% Usage: adaptiveSymmHierarchical
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for Symm's integral equation
%
%                V*phi = (K+1/2)*uD
%
%              which is equivalent to the Dirichlet problem
% 
%                -Laplace(u) = 0 in Omega with u = uD on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape1 with an
%              L-shaped domain Omega and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) initialize mesh Tcoarse
%                (2) build Galerkin data w.r.t. Tcoarse
%                (3) compute Galerkin solution w.r.t. Tcoarse
%                (4) compute E_ell-elementwise contributions of two-level
%                    error estimator tau and Dirichlet data oscillations
%                    (To that end build integral operator and RHS w.r.t.
%                    the uniformly refined mesh Tfine)
%                (5) use Doerfler strategy to mark certain Tcoarse-elements
%                (6) refine marked elements to obtain new mesh Tcoarse
%                (7) iterate as long as #Tcoarse is lower than given bound
%
%              After the computation, the Galerkin solution phih as well as the
%              exact solution phi are plotted.
%
% (C) 2010-2012 HILBERT-Team '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');

coordinates = load('coordinates.dat')/4;
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
[coordinates_fine,elements_fine,father2son_fine] ...
    = refineBoundaryMesh(coordinates,elements);

%*** discretize data and compute corresponding data oscillations for coarse 
%*** and fine mesh
[osc,uDh] = computeOscDirichlet(coordinates,elements,@g);
[osc_fine,uDh_fine] = computeOscDirichlet(coordinates_fine,elements_fine,@g);
osc = sum(osc_fine(father2son_fine),2);

err=[];
est=[];
nEll=[];
%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d\r', ...
        size(elements,1));

    %*** compute integral operators
    %fine mesh
    V_fine = buildV(coordinates_fine,elements_fine);
    %coarse mesh
    V = buildV(coordinates,elements);
    
    %*** compute right-hand side
    %fine mesh
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
    
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b = b_fine(father2son_fine(:,1))+b_fine(father2son_fine(:,2));

    %*** compute Galerkin solution for coarse mesh
    x = V\b;
    
    %*** stopping criterion + size(triangles,1)
    if (size(elements,1)  > nEmax )
      break;
    end

    %*** compute error estimates
    tau = computeEstSlpTau(father2son_fine,V_fine,b_fine,x);
    
    %*** mark elements for refinement
    [marked] = markElements(theta,tau + osc);
    
    %*** generate new mesh
    [coordinates,elements] ...
        = refineBoundaryMesh(coordinates,elements,marked);

    %*** generate fine mesh
    [coordinates_fine,elements_fine,father2son_fine] = ...
        refineBoundaryMesh(coordinates,elements);

    %*** discretize data and compute corresponding data oscillations 
    %*** for coarse and fine mesh
    [osc,uDh] = computeOscDirichlet(coordinates,elements,@g);
    [osc_fine,uDh_fine] ...
        = computeOscDirichlet(coordinates_fine,elements_fine,@g);
    osc = sum(osc_fine(father2son_fine),2);  
end

%*** visualize exact and adaptively computed solution
plotArclengthP0(coordinates,elements,x,@phi,1)
