% adaptiveSymmResidual provides the implementation of an adaptive mesh-refining
% algorithm for Symm's integral equation by use of the weighted residual error
% estimator
%
% Usage: adaptiveSymmResidual
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for Symm's integral equation
%
%                V*phi = (K+1/2)*uD
%
%              which is equivalent to the Dirichlet problem
% 
%                -Laplace(u) = 0 in Omega with u = uD on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape1 with an
%              L-shaped domain Omega and known exact solution
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) compute Galerkin solution w.r.t. T
%                (2) compute T-elementwise contributions of rho and
%                    Dirichlet data oscillations
%                (3) use Doerfler strategy to mark certain T-elements
%                (4) refine marked elements to obtain new mesh T
%                (5) iterate as long as #T is lower than given bound
%
%              After the computation, the Galerkin solution phih as well as the
%              exact solution phi are plotted.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter 
theta = 0.25;
theta_min = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d\r',size(elements,1))

    %*** discretize Dirichlet data and compute data oscillations
    [osc,uDh] = computeOscDirichlet(coordinates,elements,@g);

    %*** compute fine-mesh solution
    V = buildV(coordinates,elements);
    b = buildSymmRHS(coordinates,elements,uDh);
    x = V\b;
    
    %*** compute weighted-residual-error estimator rho
    rho = computeEstSlpResidual(coordinates,elements,x,uDh);
    
    %*** mark elements for refinement
    marked = markElements(theta,theta_min,rho + osc);

    %*** generate new mesh
    [coordinates_new,elements_new] ...
        = refineBoundaryMesh(coordinates,elements,marked);
    
    if size(elements,1) > nEmax
      break;
    end
    coordinates = coordinates_new;
    elements = elements_new;
end

%*** visualize exact and adaptively computed solution
plotArclengthP0(coordinates,elements,x,@phi,1);
