function adaptiveGreensRepresentation(nEmax)

% adaptiveGreensRepresentation provides an experimental comparison between
% uniform and adaptive mesh-refinement for Symm's integral equation on a rotated
% L-shaped boundary, where the solution has a generic singularity, whereas the
% data are smooth. The focus lies on the nodal error for the PDE solution if the
% given Dirichlet data are discretized by either nodal interpolation or
% L2-projection onto S1. 
%
% Usage: adaptiveGreensRepresentation(NMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              We consider Symm's integral equation 
%
%                V phi = (K+1/2) uD
%
%              on an L-shaped boundary Gamma, which is equivalent to the
%              Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = uD on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              Then, phi is the normal derivative of u on Gamma.
%
%              For a given initial partition T0 with 8 uniform elements, we 
%              compute discrete solutions for both, uniform and adaptive 
%              mesh-refinement, as long as the generated meshes T have less
%              than NMAX elements. We plot the weighted-residual error
%              estimator, the data oscillations for the approximation of uD by 
%              uDh for nodal projection and L2-projection, as well as the nodal
%              error  
%
%                 | u(z) - uh(z) |  at the point z = [0.2, 0.13] in Omega
%
%              The approximate value is computed by means of the third Green's 
%              indentity
%
%                 uh(z) = V*phih(z) - K*uDh(z) 
%
%              For smooth exact solution, it is known that the nodal error in
%              Omega decays with order h^2 for nodal interpolation and h^3 for
%              uDh obtained by L2-projection.
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');
figure(1)
clf

%*** use lshape1 for demonstration purposes
addpath('lshape1/');
coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** adaptivity parameter 
theta = 0.25;
rho = 0.25;

%*** evaluation point for Green's representation formula
z = [0.2 0.13];

%***
%*** uniform mesh-refinement
%***

coor = coordinates;
elem = elements;
counter = 1;

while 1

    %*** compute data oscillations for nodal interpolation
    [osc_nodal,uDh_nodal] = computeOscDirichlet(coor,elem,@g);

    %*** compute data oscillations for L2 projection
    [osc_L2,uDh_L2] = computeOscDirichletL2(coor,elem,@g);
    
    %*** compute discrete solution and error estimator for nodal interpolation
    V = buildV(coor,elem);
    b_nodal = buildSymmRHS(coor,elem,uDh_nodal);
    x_nodal = V\b_nodal;
    rho_nodal = computeEstSlpResidual(coor,elem,x_nodal,uDh_nodal);
    
    %*** compute discrete solution for L2 projection
    b_L2 = buildSymmRHS(coor,elem,uDh_L2);
    x_L2 = V\b_L2;
    rho_L2 = computeEstSlpResidual(coor,elem,x_L2,uDh_L2);
    
    %*** compute nodal errors
    errz_nodal = abs( g(z) - evaluateV(coor,elem,x_nodal,z) ...
                           + evaluateK(coor,elem,uDh_nodal,z) );
    errz_L2 = abs( g(z) - evaluateV(coor,elem,x_L2,z) ...
                        + evaluateK(coor,elem,uDh_L2,z) );
                 
    %*** store computed data
    unif_dof(counter) = size(elem,1);
    unif_osc_nodal(counter) = sqrt(sum(osc_nodal));
    unif_osc_L2(counter ) = sqrt(sum(osc_L2));
    unif_rho_nodal(counter) = sqrt(sum(rho_nodal));
    unif_rho_L2(counter) = sqrt(sum(rho_L2));
    unif_errz_nodal(counter) = errz_nodal;
    unif_errz_L2(counter) = errz_L2;
    
    %*** plot computed data
    loglog(unif_dof,unif_rho_nodal,'rx-'); hold on;
    loglog(unif_dof,unif_errz_nodal,'ro-');
    loglog(unif_dof,unif_osc_nodal,'rs-');
    loglog(unif_dof,unif_rho_L2,'bx-');
    loglog(unif_dof,unif_errz_L2,'bo-');
    loglog(unif_dof,unif_osc_L2,'bs-');
    drawnow;
    
    %*** increase counter, refine mesh and interate
    counter = counter + 1;
    [coor,elem] = refineBoundaryMesh(coor,elem);
    if size(elem,1) > nEmax
        break
    end
    
end

%***
%*** adaptive mesh-refinement with nodal interpolation
%***

coor = coordinates;
elem = elements;
counter = 1;

while 1
    
    %*** compute data oscillations for nodal interpolation
    [osc_nodal,uDh_nodal] = computeOscDirichlet(coor,elem,@g);

    %*** compute discrete solution and error estimator for nodal interpolation
    V = buildV(coor,elem);
    b_nodal = buildSymmRHS(coor,elem,uDh_nodal);
    x_nodal = V\b_nodal;
    rho_nodal = computeEstSlpResidual(coor,elem,x_nodal,uDh_nodal);

    %*** compute nodal error
    errz_nodal = abs( g(z) - evaluateV(coor,elem,x_nodal,z) ...
                           + evaluateK(coor,elem,uDh_nodal,z) );
                 
    %*** store computed data
    nodal_dof(counter) = size(elem,1);
    nodal_osc(counter) = sqrt(sum(osc_nodal));
    nodal_rho(counter) = sqrt(sum(rho_nodal));
    nodal_errz(counter) = errz_nodal;
    
    %*** plot computed data
    loglog(nodal_dof,nodal_rho,'mx-');
    loglog(nodal_dof,nodal_errz,'mo-');
    loglog(nodal_dof,nodal_osc,'ms-');
    drawnow;
    
    %*** mark elements for refinement
    marked = markElements(theta,rho,rho_nodal + osc_nodal);
    
    %*** increase counter, refine mesh and interate
    counter = counter + 1;
    [coor,elem] = refineBoundaryMesh(coor,elem,marked);
    
    if size(elem,1) > nEmax
      break;
    end
end

%***
%*** adaptive mesh-refinement with L2 projection
%***

coor = coordinates;
elem = elements;
counter = 1;

while 1
    
    %*** compute data oscillations for nodal interpolation
    [osc_L2,uDh_L2] = computeOscDirichletL2(coor,elem,@g);

    %*** compute discrete solution and error estimator for nodal interpolation
    V = buildV(coor,elem);
    b_L2 = buildSymmRHS(coor,elem,uDh_L2);
    x_L2 = V\b_L2;
    rho_L2 = computeEstSlpResidual(coor,elem,x_L2,uDh_L2);

    %*** compute nodal error
    errz_L2 = abs( g(z) - evaluateV(coor,elem,x_L2,z) ...
                        + evaluateK(coor,elem,uDh_L2,z) );
                 
    %*** store computed data
    l2_dof(counter) = size(elem,1);
    l2_osc(counter) = sqrt(sum(osc_L2));
    l2_rho(counter) = sqrt(sum(rho_L2));
    l2_errz(counter) = errz_L2;
    
    %*** plot computed data
    loglog(l2_dof,l2_rho,'cx-');
    loglog(l2_dof,l2_errz,'co-');
    loglog(l2_dof,l2_osc,'cs-');
    drawnow;
    
    %*** mark elements for refinement
    marked = markElements(theta,rho,rho_L2 + osc_L2);
    
    %*** increase counter, refine mesh and interate
    counter = counter + 1;
    [coor,elem] = refineBoundaryMesh(coor,elem,marked);
    
    if size(elem,1) > nEmax
      break;
    end
end

%*** draw everything again
clf
loglog(unif_dof,unif_rho_nodal,'rx-'); hold on;
loglog(unif_dof,unif_errz_nodal,'ro-');
loglog(unif_dof,unif_osc_nodal,'rs-');
loglog(unif_dof,unif_rho_L2,'bx-');
loglog(unif_dof,unif_errz_L2,'bo-');
loglog(unif_dof,unif_osc_L2,'bs-');
loglog(nodal_dof,nodal_rho,'mx-');
loglog(nodal_dof,nodal_errz,'mo-');
loglog(nodal_dof,nodal_osc,'ms-');
loglog(l2_dof,l2_rho,'cx-');
loglog(l2_dof,l2_errz,'co-');
loglog(l2_dof,l2_osc,'cs-');

%*** draw convergence orders
dof = [min([unif_dof,nodal_dof,l2_dof]),3*max([unif_dof,nodal_dof,l2_dof])];
loglog(dof,dof.^(-2/3),'k--');
loglog(dof,dof.^(-3/2),'k-');
loglog(dof,dof.^(-2),'k-.');
loglog(dof,dof.^(-3),'k:');

legend('rho (unif, nodal)','err (unif, nodal)','osc (unif, nodal)', ...
       'rho (unif, L2)','err (unif, L2)','osc (unif, L2)', ...
       'rho (adap, nodal)','err (adap, nodal)','osc (adap, nodal)', ...
       'rho (adap, L2)','err (adap, L2)','osc (adap, L2)', ...
       'order 2/3','order 3/2','order 2','order 3', -1);
hold off;