function [nE,time,eta,eta_tilde,mu,mu_tilde,osc,err] ...
          = exampleAdapMixed(coordinates,dirichlet,neumann, ...
                             uD,phiN,nEmax,theta,varargin)

% exampleAdapMixed provides the experimental data for the numerical solution of 
% the equivalent integral formulation of a mixed boundary value problem with
% adaptive mesh-refinement. 
%
% Usage: [N,TIME,ETA,ETA_TILDE,MU,MU_TILDE,OSC,ERR] ...
%          = exampleAdapMixed(COORDINATES,DIRICHLET,NEUMANN, ...
%                             UD,PHIN,NMAX,THETA [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              Given an initial mesh T0 and data uD and phiN on GammaD and
%              GammaN, respectively, we numerically solve the mixed boundary
%              value problem
%
%                -Laplace(u)  = 0    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              via an equivalent integral formulation. To that end, we realize
%              the following adaptive algorithm: 
%
%              (1) compute uniformly refined mesh T_fine
%              (2) compute Galerkin matrix and right-hand side for T_fine
%              (3) compute Galerkin solution u_fine w.r.t T_fine
%              (4) compute refinement indicators mu-tilde and data-oscillations
%              (5) use Doerfler strategy for marking
%              (6) refine marked elements and obtain a new mesh
%              (7) iterate until coarse mesh has sufficiently many elements
%
%              Furthermore, the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, and mu-tilde are computed. Assuming that the data uD and phiN 
%              are, in fact, given over the whole boundary Gamma, we 
%              additionally compute a reliable error bound for the overall 
%              approximation error on GammaD and GammaN. If a figure number is 
%              specified, all quantities are plotted on-the-fly during the 
%              computation over the number of degrees of freedom and the total 
%              computational time. The latter considers precisely the time 
%              cumulatively needed to perform the adaptive loop, i.e. the 
%              computational overhead caused by computation of eta, 
%              eta-tilde, mu, mu-tilde, and the reliable error bound is not 
%              included in the time measurement.
%
%              COORDINATES gives the coordinates for all nodes on the boundary.
%              DIRICHLET and NEUMANN specify the boundary elements on the
%              Dirichlet boundary GammaD and the Neumann boundary GammaN,
%              respectively. The function handles UD and PHIN allow to evaluate
%              the given Dirichlet and Neumann data. For an (M x 2)-matrix of
%              nodes X, UD returns the (M x 1)-vector Y of corresponding
%              function values, i.e. Y(j) := uD(X(j,:)). For an (M x 2)-matrix
%              of nodes X and the respective boundary elements given by the
%              (M x 2)-matrices A and B, PHIN returns the (M x 1)-vector Y of
%              corresponding function values, i.e. Y(j) := PHI(X(j,:)). The
%              algorithm terminates, if the coarse mesh has more than NMAX
%              elements. THETA is the adaptivity parameter used for Doerfler
%              marking. The optional parameter FIG gives the figure number,
%              where a convergence graph is plotted on-the-fly.
%
%              All output parameters are vectors containing the respective
%              values corresponding to each step of the adaptive loop. N
%              contains the number of coarse mesh elements. TIME contains the
%              cumulatively measured computational time necessary to perform the
%              adaptive loop up to each step. ETA, ETA_TILDE, MU, and MU_TILDE
%              contain the global error estimator quantities computed. OSC
%              contains the global data oscillations. ERR contains a reliable
%              upper bound for the error, where we assume that UD and PHIN are 
%              known on the entire boundary Gamma. 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** check whether optional figure number is given
if nargin > 6 && isnumeric(varargin{1})
    figure_number = varargin{1};
else
    figure_number = false;
end

%*** rearrange indices such that Neumann nodes are first
[coordinates,neumann,dirichlet] = buildSortedMesh(coordinates, ...
    neumann,dirichlet);

%*** initialize Dirichlet data
g = zeros(size(coordinates,1),1);
g(unique(dirichlet)) = uD(coordinates(unique(dirichlet),:));

%*** adaptive mesh-refining algorithm
counter = 1;

%
%--- start time-measurement for adaptive mesh-refinement
%
time_ = cputime;

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
    [coordinates_fine,dirichlet_fine,neumann_fine,...
        father2dirichlet_fine,father2neumann_fine] ...
        = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** rearrange nodes such that neumann nodes are first
[coordinates_fine,neumann_fine,dirichlet_fine] = ...
    buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

%*** discretize data and compute corresponding data oscillations for fine mesh
    [oscD_fine,oscN_fine,g_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          father2neumann_fine,neumann,g,uD,phiN);

while size(dirichlet,1) + size(neumann,1) < nEmax
    
    %*** if not first step, refine mesh uniformly and prolongate uDh
    if(counter>1)
        %
        %--- start time-measurement for adaptive mesh-refinement
        %
        time_ = cputime;
        
        neumann_coarse=neumann_fine;
        father2neumann_coarse=father2neumann_fine;

        [coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine,father2neumann_fine]...
            =refineBoundaryMesh(coordinates,dirichlet,neumann);

        %*** rearrange nodes such that neumann nodes are first
        [coordinates_fine,neumann_fine,dirichlet_fine] = ...
            buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine); 

        %*** build coarse2fine array
	coarse2fine = generateFather2Son(father2neumann_adap, ...
	                                 father2neumann_coarse, ...
					 father2neumann_fine);

        %*** discretize data and compute corresponding data oscillations for
        %*** fine mesh
        [oscD_fine,oscN_fine,g_fine,phiNh_fine] ...
            = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
            coarse2fine,neumann_coarse,g_fine,uD,phiN);
    end
     
    oscD = sum(oscD_fine(father2dirichlet_fine),2);
    oscN = sum(oscN_fine(father2neumann_fine),2);
    
    %*** compute integral operators for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);  
    W_fine = buildW(coordinates_fine,elements_fine);

    %*** compute right-hand side for fine mesh
    [b1_fine,b2_fine] = buildMixedRHS(coordinates_fine,dirichlet_fine, ...
                                      neumann_fine,V_fine,K_fine, ...
                                      W_fine,g_fine,phiNh_fine);
    
    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nN_fine = length(freeNeumann_fine);

    %*** shrink integral operators for fine mesh
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    nodesNeumann = unique(neumann_fine);
    W_fine = W_fine(nodesNeumann,nodesNeumann);    
    
    %*** compute fine-mesh Galerkin solution
    x = [ W_fine(freeNeumann_fine,freeNeumann_fine) K_fine' ;  ...
         -K_fine V_fine ] ...
         \ [ b1_fine(freeNeumann_fine) ; b2_fine(freeDirichlet_fine) ];
    
    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD) 
    dim_fine = length(unique(neumann_fine));
    xN_fine = zeros(dim_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine); %** dof on Neumann boundary    
    xD_fine = x((1:nD_fine) + nN_fine);       %** dof on Dirichlet boundary
    
    %*** compute tilde-mu estimators for GammaD and GammaN
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet,father2dirichlet_fine, ...
                                     xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann,father2neumann_fine, ...
                                     xN_fine);
    
    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann] ...
        = markElements(theta,0.25,muD_tilde + oscD,muN_tilde + oscN);
    
    %*** generate new mesh
    [coordinates_new,dirichlet_new,neumann_new,...
        father2dirichlet_adap,father2neumann_adap] ...
        = refineBoundaryMesh(coordinates,dirichlet,neumann,...
                             marked_dirichlet,marked_neumann);
                         
    %*** rearrange indices such that Neumann nodes are first
    [coordinates_new,neumann_new,dirichlet_new] ...
        = buildSortedMesh(coordinates_new,neumann_new,dirichlet_new);

    %
    %--- stop time-measurement for adaptive mesh-refinement
    %
    time_ = cputime - time_;
    if counter>1
        time_ = time(counter-1)+time_;
    end
    
    %*** free obsolete memory
    clear K_fine x_fine coarse2fine

    %*** compute degrees of freedom for coarse mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;

    %*** compute and shrink integral operators for coarse mesh
    elements = [dirichlet;neumann];
    father2son = [father2dirichlet_fine ; father2neumann_fine + nD_fine];
    
    V = buildV(coordinates,elements);
    V = V(freeDirichlet,freeDirichlet);
    K = buildK(coordinates,elements);  
    K = K(freeDirichlet,freeNeumann);
    W = buildW(coordinates,elements);
    W = W(freeNeumann,freeNeumann);
    
    %*** build index field k = coarse2fine (j) such that j-th node of coarse
    %*** mesh coincides with k-th node of fine mesh
    nC = size(coordinates,1);
    coarse2fine = zeros(nC,1);
    coarse2fine(elements) = [ elements_fine(father2son(:,1),1) ...
                              elements_fine(father2son(:,2),2) ];
                          
    %*** build index field k = element2midpoint(j) such that midpoint of j-th
    %*** element of coarse mesh is k-th node of fine mesh
    element2midpoint = elements_fine(father2son(:,1),2);

    %*** build index field [i,j] = node2element(k) such that
    %*** intersection(Ei,Ej) = zk
    nE_ = size(elements,1);
    node2element = zeros(nC,2);
    node2element(elements(:,1),2) = (1:nE_)';
    node2element(elements(:,2),1) = (1:nE_)';
 
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b1 = b1_fine(coarse2fine) ...
        + 0.5*sum(b1_fine(element2midpoint(node2element)),2);
    b2 = b2_fine(father2son(:,1))+b2_fine(father2son(:,2));

    %*** compute Galerkin solution w.r.t. coarse mesh for error estimation
    x = [ W K' ; -K V ] \ [ b1(freeNeumann) ; b2(freeDirichlet) ];
    
    %*** free obsolete memory
    clear V K W b1 b2 b1_fine b2_fine
   
    %*** compute solution vectors w.r.t. P0(GammaD) und S1(GammaN)
    nN = length(freeNeumann);
    dim = length(unique(neumann));
    xN = zeros(dim,1);
    xN(freeNeumann) = x(1:nN);     %** dof on Neumann boundary
    xD = x((1:nD) + nN);           %** dof on Dirichlet boundary

    %*** compute mu estimators for GammaD and GammaN
    muD = computeEstSlpMu(coordinates,dirichlet,father2dirichlet_fine,xD_fine,xD);
    muN = computeEstHypMu(neumann_fine,neumann,father2neumann_fine,xN_fine,xN);

    %*** compute tilde-eta estimators for GammaD and GammaN
    etaD_tilde = computeEstSlpEtaTilde(father2dirichlet_fine,V_fine,xD_fine);
    etaN_tilde = computeEstHypEtaTilde(neumann_fine,neumann,father2neumann_fine, ...
                                       W_fine,xN_fine);
    
    %*** compute eta estimators for GammaD and GammaN
    etaD = computeEstSlpEta(father2dirichlet_fine,V_fine,xD_fine,xD);
    etaN = computeEstHypEta(neumann_fine,neumann,father2neumann_fine,W_fine, ...
                            xN_fine,xN);

    %*** compute reliable error bound for GammaD and GammaN
    errD = computeErrNeumann(coordinates,dirichlet,xD,phiN);
    xNg = g_fine(coarse2fine);
    
    xNg(freeNeumann) = xNg(freeNeumann) + x(1:nN);
    errN = computeErrDirichlet(coordinates,neumann,xNg,uD);
    
    %*** free obsolete memory
    clear coordinates_fine elements_fine dirichlet_fine
    clear father2son father2dirichlet father2neumann
    clear V_fine W_fine b_fine x_fine x
    
    %*** store computed data
    nE(counter) = size(elements,1);
    time(counter) = time_;
    eta(counter) = sqrt(etaD + etaN);
    eta_tilde(counter) = sqrt(etaD_tilde + etaN_tilde);
    mu(counter) = sqrt(sum([muD ; muN]));
    mu_tilde(counter) = sqrt(sum([muD_tilde ; muN_tilde]));
    osc(counter) = sqrt(sum([oscD ; oscN]));
    err(counter) = sqrt(sum([errD ; errN]));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over number of coarse-mesh elements
        subplot(1,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,osc,'<k-')
        loglog(nE,err,'sm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
               'osc (adap)','error (adap)')
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(1,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,osc,'<k-')
        loglog(time,err,'sm-')
       
        title('estimator vs computational time')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
               'osc (adap)','error (adap)')
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
    coordinates = coordinates_new;
    dirichlet = dirichlet_new;
    neumann = neumann_new;    
end

