function [nE,time,eta,eta_tilde,mu,mu_tilde,osc,err] ...
                 = exampleUnifHypsing(coordinates0,elements0,phi,nEmax,varargin)

% exampleUnifHypsing provides the experimental data for the numerical solution
% of the equivalent integral formulation of a Neumann boundary value problem
% with uniform mesh-refinement. 
%
% Usage: [N,TIME,ETA,ETA_TILDE,MU,MU_TILDE,OSC [,ERR]] ...
%               = exampleUnifHypsing(COORDINATES,ELEMENTS,PHI,NMAX [,UD] [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Given an initial mesh T0 and data phi on Gamma, we numerically
%              solve the hypersingular integral equation
%
%                W*u = (1/2-K')*phi on Gamma = boundary(Omega)
%
%              for some Lipschitz domain Omega. To that end, we realize
%              the following algorithm:
%
%              (1) refine the initial mesh T0 k-times uniformly
%              (2) compute Galerkin matrix and right-hand side for this mesh
%              (3) compute Galerkin solution
%
%              In addition, we compute further a posteriori error
%              estimators, namely
%              - the (h-h/2)-based error estimators eta, eta-tilde,
%                mu, mu-tilde.
%              Assuming the exact solution solution uD is given, we
%              additionally compute a reliable error bound for the
%              approximation error on Gamma. If a figure number is
%              specified, all error quantities are plotted on-the-fly during the 
%              computation over the number of degrees of freedom and the
%              computational time. The latter considers precisely the time
%              necessary to set up the mesh and solve the coarse system, i.e.
%              the computational overhead caused by calculation of eta,
%              eta-tilde, mu, mu-tilde, and the reliable error bound is not
%              included in the time measurement. Moreover the time measurement
%              is not cumulative.
%
%              COORDINATES gives the coordinates for all nodes on the boundary.
%              ELEMENTS specifies the boundary elements on Gamma. The function
%              handle PHI allows to evaluate the given Neumann data. For an
%              (M x 2)-matrix of nodes X and the respective boundary elements
%              given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values, i.e.
%              Y(j) := PHI(X(j,:)). NMAX gives an upper bound for the number of
%              boundary elements. Starting with the initial mesh T0, all
%              solutions corresponding to uniformly refined meshes Tell with
%              #Tell < NMAX are computed. If the exact solution of the integral
%              equation is known, it can be provided by means of the optional
%              function handle UD. For an (M x 2)-matrix of nodes X, UD returns
%              the (M x 1)-vector Y of corresponding function values, i.e.
%              Y(j) := uD(X(j,:)). The optional parameter FIG gives the figure
%              number, where a convergence graph is plotted on-the-fly. 
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial mesh. N contains the number of
%              coarse mesh elements. TIME contains the computational time
%              necessary to compute the coarse mesh solution. ETA, ETA_TILDE,
%              MU, and MU_TILDE contain the global error estimator quantities
%              computed. OSC contains the global data oscillations. The
%              optional output parameter ERR contains a reliable upper bound
%              for the error, provided that the solution UD was given as input
%              parameter.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** check whether optional figure number is given
if nargin > 4 && isnumeric(varargin{1})
    figure_number = varargin{1};
elseif nargin > 5 && isnumeric(varargin{2})
    figure_number = varargin{2};
else
    figure_number = false;
end

%*** check whether optional function phi is given
if nargin > 4 && isa(varargin{1},'function_handle')
    u_D = varargin{1};
elseif nargin > 5 && isa(varargin{2},'function_handle')
    u_D = varargin{2};
else
    u_D = false;
end

counter = 1;
elements = elements0;

while size(elements,1) < nEmax
    %
    %--- start time-measurement for uniform mesh-refinement
    %
    time_ = cputime;
    
    %*** build mesh by successive uniform refinements of initial mesh
    coordinates = coordinates0;
    elements = elements0;

    for j = 1:counter-1
        [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    end
    
    %*** discretize Neumann data and compute data oscillations
    [osc_,phih] = computeOscNeumann(coordinates,elements,phi);

    %*** compute coarse-mesh solution
    W = buildW(coordinates,elements) ...
        + buildHypsingStabilization(coordinates,elements);
    
    b = buildHypsingRHS(coordinates,elements,phih);
    x = W\b;
    
    %
    %--- stop time-measurement for uniform mesh-refinement
    %
    time_ = cputime - time_;
    
    %*** free obsolete memory
    clear W b
    
    %*** generate uniformly refined mesh
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    %*** discretize Dirichlet data and compute data oscillations
    [foo,phih_fine] = computeOscNeumann(coordinates_fine,elements_fine,phi);

    %*** compute fine-mesh solution to estimate error
    W_fine = buildW(coordinates_fine,elements_fine) ...
             + buildHypsingStabilization(coordinates_fine,elements_fine);
    b_fine = buildHypsingRHS(coordinates_fine,elements_fine,phih_fine);
    x_fine = W_fine\b_fine;
    
    %*** compute (h-h/2)-type estimators
    eta_ = computeEstHypEta(elements_fine,elements,father2son,W_fine,x_fine,x);
    eta_tilde_ = computeEstHypEtaTilde(elements_fine,elements,father2son, ...
                                       W_fine,x_fine);
    mu_ = computeEstHypMu(elements_fine,elements,father2son,x_fine,x);
    mu_tilde_ = computeEstHypMuTilde(elements_fine,elements,father2son,x_fine);

    %*** compute error bound
    if isa(u_D,'function_handle')
        err_ = computeErrDirichlet(coordinates_fine,elements_fine,x_fine,u_D);
        err(counter) = sqrt(sum(err_));
    else
        err(counter) = 0;
    end
    
    %*** free obsolete memory
    clear coordinates_fine elements_fine father2son
    clear W_fine b_fine x_fine x
    
    %*** compute Neumann data oscillation
    osc_ = computeOscNeumann(coordinates,elements,phi);
    
    %*** store computed data
    nE(counter) = size(elements,1);
    time(counter) = time_;
    eta(counter) = sqrt(eta_);
    eta_tilde(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_));
    mu_tilde(counter) = sqrt(sum(mu_tilde_));
    osc(counter) = sqrt(sum(osc_));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(1,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,osc,'<k-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(1,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,osc,'<k-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
end
