function [nE,time,eta,eta_tilde,mu,mu_tilde,osc,err] ...
          = exampleUnifMixed(coordinates0,dirichlet0,neumann0, ...
                             uD,phiN,nEmax,varargin)

% exampleUnifMixed provides the experimental data for the numerical solution of
% the equivalent integral formulation of a mixed boundary value problem with
% uniform mesh-refinement. 
%
% Usage: [N,TIME,ETA,ETA_TILDE,MU,MU_TILDE,OSC,ERR] ...
%         = exampleUnifMixed(COORDINATES,DIRICHLET,NEUMANN,UD,PHIN,NMAX [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              Given an initial mesh T0 and data uD and phiN on GammaD and
%              GammaN, respectively, we numerically solve the mixed boundary
%              value problem
%
%                -Laplace(u)  = 0    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              via an equivalent integral formulation. To that end, we realize
%              the following algorithm:
%
%              (1) refine the initial mesh T0 k-times uniformly
%              (2) compute Galerkin matrix and right-hand side for this mesh
%              (3) compute Galerkin solution
%
%              Furthermore, the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, and mu-tilde are computed. Assuming the data uD and phiN are,
%              in fact, given over the whole boundary Gamma, we additionally
%              compute a reliable error bound for the overall approximation
%              error on GammaD and GammaN. If a figure number is specified, all
%              error quantities are plotted on-the-fly during the computation
%              over the number of degrees of freedom and the computational time.
%              The latter considers precisely the time necessary to set up the
%              mesh and solve the coarse system, i.e. the computational overhead
%              caused by calculation of eta, eta-tilde, mu, mu-tilde, and the
%              reliable error bound is not included in the time measurement.
%              Moreover, the time measurement is not cumulative.
%
%              COORDINATES gives the coordinates for all nodes on the boundary.
%              DIRICHLET and NEUMANN specify the boundary elements on the
%              Dirichlet boundary GammaD and the Neumann boundary GammaN,
%              respectively. The function handles UD and PHIN allow to evaluate
%              the given Dirichlet and Neumann data. For an (M x 2)-matrix of 
%              nodes X, UD returns the (M x 1)-vector Y of corresponding
%              function values, i.e. Y(j) := uD(X(j,:)). For an (M x 2)-matrix
%              of nodes X and the respective boundary elements given by the
%              (M x 2) matrices A and B, PHIN returns the (M x 1)-vector Y of
%              corresponding function values, i.e. Y(j) := PHI(X(j,:)). NMAX
%              gives an upper bound for the number of boundary elements.
%              Starting with the initial mesh T0 all solutions corresponding to
%              uniformly refined meshes Tell with #Tell < NMAX are computed. The
%              optional parameter FIG gives the figure number where a
%              convergence graph is plotted on-the-fly.
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial mesh. N contains the number of coarse
%              mesh elements. TIME contains the computational time needed to
%              compute the coarse mesh solution. ETA, ETA_TILDE, MU, and
%              MU_TILDE contain the global error estimator quantities computed.
%              OSC contains the global data oscillations. ERR contains a
%              reliable upper bound for the error, where we assume that UD and
%              PHIN are known on the entire boundary Gamma.                
%
% (C) 2009-2012 HILBERT-Team '09, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** check whether optional figure number is given
if nargin > 6 && isnumeric(varargin{1})
    figure_number = varargin{1};
else
    figure_number = false;
end

%*** rearrange indices such that Neumann nodes are first
[coordinates0,neumann0,dirichlet0] ...
    = buildSortedMesh(coordinates0,neumann0,dirichlet0);

%*** initialize Dirichlet data
g0 = zeros(size(coordinates0,1),1);
g0(unique(dirichlet0)) = uD(coordinates0(unique(dirichlet0),:));

%*** start uniform computation
counter = 1;
dirichlet = dirichlet0;
neumann = neumann0;

while size(dirichlet,1) + size(neumann,1) < nEmax
    %
    %--- start time-measurement for uniform mesh-refinement
    %
    time_ = cputime;
    
    %*** build mesh by successive uniform refinements of initial mesh
    coordinates = coordinates0;
    dirichlet = dirichlet0;
    neumann = neumann0;
    g = g0;
    for j = 1:counter-1
        
        %*** refine mesh uniformly
        [coordinates,dirichlet,neumann_new,father2dirichlet,father2neumann] ...
            = refineBoundaryMesh(coordinates,dirichlet,neumann);
        
        %*** rearrange indices such that Neumann nodes are first
        [coordinates,neumann_new,dirichlet] ...
            = buildSortedMesh(coordinates,neumann_new,dirichlet);
        
        %*** prolongate Neumann data to the fine mesh
        g(neumann_new(father2neumann(:,1),2)) = 0.5*sum(g(neumann),2);
        g(neumann_new(father2neumann(:,1),1)) = g(neumann(:,1));
        g(neumann_new(father2neumann(:,2),2)) = g(neumann(:,2));
        g(unique(dirichlet)) = uD(coordinates(unique(dirichlet),:));

        neumann = neumann_new;            
    end
    
    %*** discretize data and compute corresponding data oscillations for fine mesh
    [oscD,oscN,foo,phiNh] ...
        = computeOscMixed(coordinates,dirichlet,neumann, uD,phiN);
    
   %*** compute integral operators for coarse mesh
    elements = [dirichlet;neumann];
    V = buildV(coordinates,elements);
    K = buildK(coordinates,elements);  
    W = buildW(coordinates,elements);
    
    %*** compute degrees of freedom for coarse mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;

    %*** compute right-hand side and shrink it
    [b1,b2] = buildMixedRHS(coordinates,dirichlet,neumann,V,K,W,g,phiNh);
    b1 = b1(freeNeumann);
    b2 = b2(freeDirichlet);

    %*** shrink integral operators
    V = V(freeDirichlet,freeDirichlet);
    K = K(freeDirichlet,freeNeumann);
    W = W(freeNeumann,freeNeumann);

    %*** compute Galerkin solution for coarse mesh
    x = [ W K' ; -K V ] \ [ b1 ; b2 ];

    %*** compute solution vectors w.r.t. P0(GammaD) and S1(GammaN)
    nN = length(freeNeumann);
    dim = length(unique(neumann));
    xN = zeros(dim,1);
    xN(freeNeumann) = x(1:nN);
    xD = x((1:nD) + nN);    
    %
    %--- stop time-measurement for uniform mesh-refinement
    %
    time_ = cputime - time_;
    
    %*** free obsolete memory
    clear V K W b1 b2
    
    %*** refine mesh uniformly
    [coordinates_fine,dirichlet_fine,neumann_fine,...
        father2dirichlet,father2neumann] ...
        = refineBoundaryMesh(coordinates,dirichlet,neumann);
    
    %*** rearrange indices such that Neumann nodes are first
    [coordinates_fine,neumann_fine,dirichlet_fine] ...
        = buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

    %*** discretize data and compute corresponding data oscillations for fine mesh
    [oscD_fine,oscN_fine,g_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          father2neumann,neumann,g,uD,phiN);    
    
    %*** compute integral operators for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);  
    W_fine = buildW(coordinates_fine,elements_fine);

    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nodesNeumann_fine = unique(neumann_fine);
    nN_fine = length(freeNeumann_fine);

    %*** compute right-hand side for fine mesh and shrink it
    [b1_fine,b2_fine] = buildMixedRHS(coordinates_fine,dirichlet_fine, ...
                                      neumann_fine,V_fine,K_fine,W_fine, ...
                                      g_fine,phiNh_fine);
    b1_fine = b1_fine(freeNeumann_fine);
    b2_fine = b2_fine(freeDirichlet_fine);

    %*** shrink integral operators
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    W_fine = W_fine(nodesNeumann_fine,nodesNeumann_fine);
    xold=x;
    %*** compute Galerkin solution for fine mesh
    x = [ W_fine(freeNeumann_fine,freeNeumann_fine) K_fine' ; ...
         -K_fine V_fine ] ...
        \ [ b1_fine ; b2_fine ];
    
    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    dim_fine = length(unique(neumann_fine));
    xN_fine = zeros(dim_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine);
    xD_fine = x((1:nD_fine) + nN_fine);

    %*** compute eta estimators for GammaD and GammaN
    etaD = computeEstSlpEta(father2dirichlet,V_fine,xD_fine,xD);
    etaN = computeEstHypEta(neumann_fine,neumann,father2neumann,W_fine, ...
        xN_fine,xN);

    %*** compute eta tilde estimators for GammaD and GammaN
    etaD_tilde = computeEstSlpEtaTilde(father2dirichlet,V_fine,xD_fine);
    etaN_tilde = computeEstHypEtaTilde(neumann_fine,neumann,father2neumann, ...
        W_fine,xN_fine);

    %*** compute mu estimators for GammaD and GammaN
    muD = computeEstSlpMu(coordinates,dirichlet,father2dirichlet,xD_fine,xD);
    muN = computeEstHypMu(neumann_fine,neumann,father2neumann,xN_fine,xN);

    %*** compute mu tilde estimators for GammaD and GammaN
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet,father2dirichlet,...
        xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann,father2neumann, ...
        xN_fine);

    %*** compute reliable error bound for GammaD and GammaN
    errD = computeErrNeumann(coordinates,dirichlet,xD,phiN);
    xNg = g;
    xNg(freeNeumann) = xNg(freeNeumann) + xold(1:nN);
    errN = computeErrDirichlet(coordinates,neumann,xNg,uD);   

    %*** free obsolete memory
    clear coordinates_fine elements_fine dirichlet_fine neumann_fine
    clear father2son father2dirichlet father2neumann
    clear V_fine K_fine W_fine b1_fine b2_fine x_fine x
    
    %*** compute data oscillation for given data
    oscD = computeOscDirichlet(coordinates,dirichlet,uD);
    oscN = computeOscNeumann(coordinates,neumann,phiN);
    
    %*** store computed data
    nE(counter) = size(elements,1);
    time(counter) = time_;
    eta(counter) = sqrt(etaD + etaN);
    eta_tilde(counter) = sqrt(etaD_tilde + etaN_tilde);
    mu(counter) = sqrt(sum([muD; muN]));
    mu_tilde(counter) = sqrt(sum([muD_tilde; muN_tilde]));
    osc(counter) = sqrt(sum([oscD; oscN]));
    err(counter) = sqrt(sum([errD; errN]));
      
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over number of coarse-mesh elements
        subplot(1,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,osc,'<k-')
        loglog(nE,err,'sm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(1,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,osc,'<k-')
        loglog(time,err,'sm-')
        
        title('estimator vs computational time')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
end

