function [nE,time,eta,eta_tilde,mu,mu_tilde,osc,err] ...
    = exampleUnifSymm(coordinates0,elements0,uD,nEmax,varargin)

% exampleUnifSymm provides the experimental data for the numerical solution of
% the equivalent integral formulation of a Dirichlet boundary value problem with
% uniform mesh-refinement. 
%
% Usage: [N,TIME,ETA,ETA_TILDE,MU,MU_TILDE,OSC [,ERR]] ...
%            = exampleUnifSymm(COORDINATES,ELEMENTS,UD,NMAX [,PHI] [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Given an initial mesh T0 and data uD on Gamma, we numerically
%              solve Symm's integral equation
%
%                V*phi = (K+1/2)*uD on Gamma = boundary(Omega)
%
%              for some Lipschitz domain Omega. To that end, we realize
%              the following algorithm:
%
%              (1) refine the initial mesh T0 k-times uniformly
%              (2) compute Galerkin matrix and right-hand side for this mesh
%              (3) compute Galerkin solution
%
%              In addition, we compute further a posteriori error
%              estimators, namely
%              - the (h-h/2)-based error estimators eta, eta-tilde,
%                mu, mu-tilde.
%              If the exact solution phi is provided by the user, we
%              additionally compute a reliable error bound for the
%              approximation error. If a figure number is specified, all
%              error quantities are plotted on-the-fly during the
%              computation over both, the number of degrees of freedom and
%              the total computational time. The latter considers precisely
%              the time necessary to set up the mesh and solve the coarse
%              system, i.e. the computational overhead caused by
%              calculation of eta, eta-tilde, mu, mu-tilde, and the
%              reliable error bound is not included in the time
%              measurement. Moreover the time measurement is not cumulative.
%
%              COORDINATES gives the coordinates for all nodes on the boundary.
%              ELEMENTS specifies the boundary elements on Gamma. The function
%              handle UD allows to evaluate the given Dirichlet data. For an
%              (M x 2)-matrix of nodes X, UD returns the (M x 1)-vector Y of
%              corresponding function values, i.e. Y(j) := uD(X(j,:)). NMAX
%              gives an upper bound for the number of boundary elements.
%              Starting with the initial mesh T0, all solutions corresponding to
%              uniformly refined meshes Tell with #Tell < NMAX are computed. 
%              If the exact solution of Symm's integral equations is known, it
%              can be provided by means of the optional function handle PHI.
%              For an (M x 2)-matrix of nodes X and the respective boundary
%              elements given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := PHI(X(j,:),A(j,:),B(j,:)). The optional parameter
%              FIG gives the figure number, where a convergence graph is plotted
%              on-the-fly. 
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial mesh. N contains the number of coarse
%              mesh elements. TIME contains the computation time necessary to
%              compute the coarse mesh solution. ETA, ETA_TILDE, MU, and
%              MU_TILDE contain the global error estimator quantities computed.
%              OSC contains the global data oscillations. The optional output
%              parameter ERR contains a reliable upper bound for the error,
%              provided that the solution PHI was given as input parameter.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** check whether optional figure number is given
if nargin > 4 && isnumeric(varargin{1})
    figure_number = varargin{1};
elseif nargin > 5 && isnumeric(varargin{2})
    figure_number = varargin{2};
else
    figure_number = false;
end

%*** check whether optional function phi is given
if nargin > 4 && isa(varargin{1},'function_handle')
    phi_N = varargin{1};
elseif nargin > 5 && isa(varargin{2},'function_handle')
    phi_N = varargin{2};
else
    phi_N = false;
end

counter = 1;
elements = elements0;

while size(elements,1) < nEmax
    
    %
    %--- start time-measurement for uniform mesh-refinement
    %
    time_ = cputime;
    
    %*** build mesh by successive uniform refinements of initial mesh
    coordinates = coordinates0;
    elements = elements0;
    for j = 1:counter-1
        [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    end
    
    %*** discretize Dirichlet data and compute data oscillations
    [osc_,uDh] = computeOscDirichlet(coordinates,elements,uD);
    
    %*** compute coarse-mesh solution
    V = buildV(coordinates,elements);
    b = buildSymmRHS(coordinates,elements,uDh);
    x = V\b;
    
    %
    %--- stop time-measurement for uniform mesh-refinement
    %
    time_ = cputime - time_;
    
    %*** free obsolete memory
    clear V b
    
    %*** generate uniformly refined mesh
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    %*** discretize Dirichlet data and compute data oscillations
    [foo,uDh_fine] = computeOscDirichlet(coordinates_fine,elements_fine,uD);

    %*** compute fine-mesh solution to estimate error
    V_fine = buildV(coordinates_fine,elements_fine);
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
    x_fine = V_fine\b_fine;
    
    %*** compute (h-h/2)-type estimators
    eta_ = computeEstSlpEta(father2son,V_fine,x_fine,x);
    eta_tilde_ = computeEstSlpEtaTilde(father2son,V_fine,x_fine);
    mu_ = computeEstSlpMu(coordinates,elements,father2son,x_fine,x);
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);
    
    %*** compute error bound
    if isa(phi_N,'function_handle')
	err_ = computeErrNeumann(coordinates_fine,elements_fine,x_fine,phi_N);
	err(counter) = sqrt(sum(err_));
    else
        err(counter) = 0;
    end

    %*** free obsolete memory
    clear coordinates_fine elements_fine father2son
    clear V_fine b_fine x_fine x
        
    %*** store computed data
    nE(counter) = size(elements,1);
    time(counter) = time_;
    eta(counter) = sqrt(eta_);
    eta_tilde(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_));
    mu_tilde(counter) = sqrt(sum(mu_tilde_));
    osc(counter) = sqrt(sum(osc_));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(1,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,osc,'<k-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(1,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,osc,'<k-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'osc (unif)','error (unif)')
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
end

