function exampleMixedc(nEmax)

% exampleMixedc provides an experimental comparison between uniform and
% adaptive mesh-refinement for the integral formulation of a mixed boundary
% value problem on an L-shaped boundary. The normal derivative has a generic
% singularity, whereas the trace is smooth. The boundary conditions change in 
% the reentrant corner so that given Neumann data and missing Neumann data are
% both singular.
%
% Usage: exampleMixedc(NMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              We consider the equivalent integral formulation of the Laplace
%              problem
% 
%                -Laplace(u) = 0 in Omega
%      
%              with mixed boundary conditions
% 
%                u = g on Gamma_D   and   du/dn = phi on Gamma_N
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              Then, phi is the normal derivative of u on Gamma.
%
%              For a given initial partition T0 with 8 uniform elements, we 
%              compute discrete solutions for both, uniform and adaptive 
%              mesh-refinement, as long as the generated meshes T have less
%              than NMAX elements. We plot the (h-h/2)-error estimators
%              eta, eta-tilde, mu, and mu-tilde as well as a reliable upper 
%              bound for the error. All quantities are plotted over the number
%              of elements and over the computational time. 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../general/')
addpath('../../../lib/')

coordinates = load('coordinates.dat');
elements = load('elements.dat');
nE = size(elements,1);

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = elements([1,2,3,4],:);
neumann = elements([5,6,7,8],:);

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** visualize Gamma and initial partition
figure(1)
clf
for j = 1:size(dirichlet,1)
    a = coordinates(dirichlet(j,1),:);
    b = coordinates(dirichlet(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-b','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
for j = 1:size(neumann,1)
    a = coordinates(neumann(j,1),:);
    b = coordinates(neumann(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-r','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
axis equal
title('initial mesh with N=8 elements (GammaD = blue, GammaN = red)')

%*** compute results for uniform mesh-refinement
[nE_unif,time_unif, ...
    eta_unif,eta_tilde_unif,mu_unif,mu_tilde_unif,osc_unif,err_unif] ...
    = exampleUnifMixed(coordinates,dirichlet,neumann,@g,@phi,nEmax, 2);

%*** compute results for adaptive mesh-refinement
[nE_adap,time_adap, ...
    eta_adap,eta_tilde_adap,mu_adap,mu_tilde_adap,osc_adap,err_adap] ...
    = exampleAdapMixed(coordinates,dirichlet,neumann,@g,@phi,nEmax,0.25, 2);

%*** plot results
figure(2)
clf

%*** plot estimators over coarse-mesh elements
subplot(1,2,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,osc_unif,'r<-')
loglog(nE_unif,err_unif,'mp-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bx-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,osc_adap,'b<-')
loglog(nE_adap,err_adap,'cp-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,0.5*N.^(-3/2),'k--')
N = [nE_unif,2*max(nE_unif)];
loglog(N,0.75*N.^(-2/3),'k--')

title('Mixed BVP on symmetrically rotated L-shape (exampleMixedc)')
xlabel('number of coarse-mesh elements')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)', ...
       'osc (unif)','error (unif)', ...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)', ...
       'osc (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,2,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,osc_unif,'r<-')
loglog(time_unif,err_unif,'mp-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,osc_adap,'b<-')
loglog(time_adap,err_adap,'cp-')

title('Mixed BVP on symmetrically rotated L-shape (exampleMixedc)')
xlabel('computational time [s]')
ylabel('estimators')      
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)', ...
       'osc (unif)','error (unif)', ...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)', ...
       'osc (adap)','error (adap)')

