function exampleSymm(nEmax)

% exampleSymm provides an experimental comparison between uniform and adaptive
% mesh-refinement for Symm's integral equation on a rotated L-shaped boundary,
% where the solution has a generic singularity, whereas the data are smooth.
%
% Usage: exampleSymm(NMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              We consider Symm's integral equation 
%
%                V phi = (K+1/2) uD
%
%              on an L-shaped boundary Gamma, which is equivalent to the
%              Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = uD on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              Then, phi is the normal derivative of u on Gamma.
%
%              For a given initial partition T0 with 8 uniform elements, we 
%              compute discrete solutions for both, uniform and adaptive 
%              mesh-refinement, as long as the generated meshes T have less
%              than NMAX elements. We plot the (h-h/2)-error estimators
%              eta, eta-tilde, mu, and mu-tilde as well as a reliable upper 
%              bound for the error. All quantities are plotted over the number
%              of elements and over the computational time. 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../general/')
addpath('../../../lib/')

coordinates = load('coordinates.dat');
elements = load('elements.dat');
nE = size(elements,1);

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** shrink domain to ensure ellipticity of V by diam(Omega)<1
coordinates = coordinates/4;

%*** visualize Gamma and initial partition
figure(1)
clf
for j = 1:nE
    a = coordinates(elements(j,1),:);
    b = coordinates(elements(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-k','MarkerFaceColor','r')
    hold on
end
axis equal
title('initial mesh with N=8 elements')

%*** compute results for adaptive mesh-refinement
[nE_adap,time_adap,...
    eta_adap,eta_tilde_adap,mu_adap,mu_tilde_adap, ...
    osc_adap,error_adap] ...
    = exampleAdapSymm(coordinates,elements,@g,nEmax,0.25, @phi,2);

%*** compute results for uniform mesh-refinement
[nE_unif,time_unif, ...
    eta_unif,eta_tilde_unif,mu_unif,mu_tilde_unif, ...
    osc_unif,error_unif] ...
    = exampleUnifSymm(coordinates,elements,@g,nEmax, @phi,2);

%*** plot results
figure(2)
clf

%*** plot estimators over number of coarse-mesh elements
subplot(1,2,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,osc_unif,'r<-')
loglog(nE_unif,error_unif,'m^-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bx-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,osc_adap,'b<-')
loglog(nE_adap,error_adap,'c^-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,0.5*N.^(-3/2),'k--')
N = [nE_unif,2*max(nE_unif)];
loglog(N,N.^(-2/3),'k--')

title('Dirichlet problem on symmetrically rotated L-shape (exampleSymm)')
xlabel('number of coarse-mesh elements')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
       'osc (unif)','error (unif)',...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
       'osc (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,2,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,osc_unif,'r<-')
loglog(time_unif,error_unif,'m^-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,osc_adap,'b<-')
loglog(time_adap,error_adap,'c^-')

title('Dirichlet problem on symmetrically rotated L-shape (exampleSymm)')
xlabel('computational time [s]')
ylabel('estimators')      
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
       'osc (unif)','error (unif)',...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
       'osc (adap)','error (adap)')
