function exampleMixeda(nEmax)

% exampleMixeda provides an experimental comparison between uniform and
% adaptive mesh-refinement for the integral formulation of a mixed boundary
% value problem on the square, where given data and sought solution are
% arbitrarily smooth. Here, the boundary parts GammaD and GammaN are connected
% and symmetric.
%
% Usage: exampleMixeda(NMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              We consider the equivalent integral formulation of the Laplace
%              problem
% 
%                -Laplace(u) = 0 in Omega
%      
%              on a square [0,0.5] x [0,0.5] with mixed boundary conditions
% 
%                u = g on GammaD   and   du/dn = phi on GammaN
%
%              We prescribe a smooth exact solution
%
%                u(x,y) = sinh(2*pi*x) * cos(2*pi*y)
%
%              For a given initial partition T0 with 8 elements, we 
%              compute discrete solutions for both, uniform and adaptive 
%              mesh-refinement, as long as the generated meshes T have less
%              than NMAX elements. We plot the (h-h/2)-error estimators
%              eta, eta-tilde, mu, and mu-tilde as well as a reliable upper 
%              bound for the error. All quantities are plotted over the number
%              of elements and over the computational time. 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../general/')
addpath('../../../lib/')

coordinates = load('coordinates.dat');
elements = load('elements.dat');

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = elements([1,2],:);
neumann = elements([3,4],:);

%*** refine mesh uniformly
[coordinates,dirichlet,neumann] ...
    = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** visualize Gamma and initial partition
figure(1)
clf
for j = 1:size(dirichlet,1)
    a = coordinates(dirichlet(j,1),:);
    b = coordinates(dirichlet(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-b','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
for j = 1:size(neumann,1)
    a = coordinates(neumann(j,1),:);
    b = coordinates(neumann(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-r','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
axis equal
title('initial mesh with N=8 elements (GammaD = blue, GammaN = red)')

%*** compute results for adaptive mesh-refinement
[nE_adap,time_adap, ...
    eta_adap,eta_tilde_adap,mu_adap,mu_tilde_adap,osc_adap,err_adap] ...
    = exampleAdapMixed(coordinates,dirichlet,neumann,@g,@phi,nEmax,0.25, 2);

%*** compute results for uniform mesh-refinement
[nE_unif,time_unif, ...
    eta_unif,eta_tilde_unif,mu_unif,mu_tilde_unif,osc_unif,err_unif] ...
    = exampleUnifMixed(coordinates,dirichlet,neumann,@g,@phi,nEmax, 2);

%*** plot results
figure(2)
clf

%*** plot estimators over number of coarse-mesh elements
subplot(1,2,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,osc_unif,'r<-')
loglog(nE_unif,err_unif,'mp-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bx-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,osc_adap,'b<-')
loglog(nE_adap,err_adap,'cp-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,15*N.^(-3/2),'k--')

title('Mixed BVP with smooth solution on square (exampleMixeda)')
xlabel('number of coarse-mesh elements')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)', ...
       'osc (unif)','error (unif)', ...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)', ...
       'osc (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,2,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,osc_unif,'r<-')
loglog(time_unif,err_unif,'mp-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,osc_adap,'b<-')
loglog(time_adap,err_adap,'cp-')

title('Mixed BVP with smooth solution on square (exampleMixeda)')
xlabel('computational time [s]')
ylabel('estimators')      
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)', ...
       'osc (unif)','error (unif)', ...
       'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)', ...
       'osc (adap)','error (adap)')
  
