function phix = phi(x,a,b)

% phi provides the Neumann data for some homogeneous Laplace problem realized
% in examples/laplace/zshape/*
%
% Usage: Y = phi(X,A,B)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/laplace/zshape/*, we treat the Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = g on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = r^{4/7} * cos(4theta/7)
%
%              which also be written in the complex plane by
%
%                u(a+bi) = Real( (a+bi)^(4/7) )
%
%              The evaluation points are given by the (N x 2)-matrix X. 
%              The (N x 2)-matrices A and B give the nodes of the corresponding
%              boundary elements, i.e., X(j,:) belongs to the line segment 
%              Ej = conv{ A(j,:) , B(j,:) }. This information is used to 
%              compute the outer unit normal vector on Ej. The            
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = phi(X(j,:)). 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** obtain parameter alpha = 4/7
alpha = parameter();

%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** compute gradient gradu(x)
[t,r]=cart2pol(x(:,1),x(:,2));
grad(:,1)=alpha*r.^(alpha-1) .* ( cos(t).*cos(alpha*t)+sin(t).*sin(alpha*t) );
grad(:,2)=alpha*r.^(alpha-1) .* ( sin(t).*cos(alpha*t)-cos(t).*sin(alpha*t) );

%*** compute normal derivative d_n u(x) = gradu(x)*n(x)
phix = sum(grad.*n,2);

