function data = threading(coordinates, elements, vertices, triangles, threads)

% threading builds all operators with different numbers of threads for given
% boundary/volume meshs and measures the time for each configuration.
%
% Usage: data = threading(COORDINATES, ELEMENTS, VERTICES, TRIANGLES, THREADS)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              COORDINATES and ELEMENTS give a boundary mesh. VERTICES and
%              VOLUMES gives a (possibly different) volume mesh. THREADS is
%              an array of integers.
%
%              For each element of THREADS, this function builds all operators
%              with the given number of threads. Then it build K, V, W for
%              the boundary mesh and measures the time it takes to assemble the
%              respective matrices. The results are written to the first, third
%              and forth row of DATA. Then it builds N for the given volume mesh
%              and measures the time it takes to assemble the matrix. The result
%              is written to the second row of DATA. The individual columns of
%              DATA correspond to the number of threads used for assembling the
%              matrices (the first column was built with threads(1) threads, the
%              second with threads(2) threads and so on ...)
%
% (C) 2009-2013 HILBERT-Team '09, '10, '12, '13
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

    addpath lib
    addpath source/make/config
    addpath source/make

    generateConfigure( defaultConfigure() );
    c = Configure();
    data = zeros( 4, length(threads) );

    for i = 1:length(threads)
        t = threads(i);
        make clean;
	if t == 1
		c.Threading = 'no';
	else
        	c.Threading = 'yes';
	end
        c.NumberOfCores = num2str(t);
        c.CompFlags = '-O2 -fPIC';
        generateConfigure( c );
        clear Configure;

        make;

	clear buildK buildN buildV buildW

        tic;
        K = buildK( coordinates, elements );
        data( 1, i ) = toc
        clear K;

	[vertices, triangles, coordinates_coarse, elements_coarse] = ...
		buildSortedMesh( vertices, triangles );
        tic;
        N = buildN( coordinates_coarse, elements_coarse, vertices, triangles );
        data( 2, i ) = toc
        clear N;

        tic;
        V = buildV( coordinates, elements );
        data( 3, i ) = toc
        clear V;

        tic;
        W = buildW( coordinates, elements );
        data( 4, i ) = toc
        clear W;
    end
end

