% adaptiveHypsingResidualVol provides the implementation of an adaptive 
% mesh-refining algorithm for the hypersingular integral equation with
% volume force by use of the weighted-residual error estimator
%
% Usage: adaptiveHypsingResidualVol
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the hypersingular integral
%              equation
%
%                W*u = (K'-1/2)*phiN - N_1 f
%
%              which is equivalent to the Neumann problem
% 
%                -Laplace(u) = f in Omega with du/dn = phiN on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape2 with an
%              L-shaped domain Omega and known exact solution
%
%                u(x) = |x-z|^(1.8) + r^(2/3)*cos(2 \theta / 3)
%
%              with z = [0.14 0.14].
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) build Galerkin data w.r.t. (E,T)
%                (2) compute Galerkin solution w.r.t. (E,T)
%                (3) compute E-elementwise contributions of
%                    rho and Dirichlet data oscillations,
%                    compute T-elementwise contributions of
%                    volume data oscillations
%                (4) use Doerfler strategy to mark certain
%                    (E,T)-elements
%                (5) refine marked elements to obtain new mesh
%                    (E,T)
%                (6) iterate as long as #(E,T) is lower than
%                    given bound
%
%              After the computation, the Galerkin solution uh and the
%              discrete Neumann data phih is plotted over the arclength.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

clear all;

addpath('../../lib/');

%*** use lshape2 for demonstration purposes
addpath('lshape2/');

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** extract boundary mesh
[vertices,triangles,coordinates,elements] = buildSortedMesh(vertices,triangles);

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter 
theta = 0.25;
theta_min = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d (Gamma), %d (Omega)\r', ...
        size(elements,1),size(triangles,1));

    %*** discretize Neumann and volume data and compute data oscillations
    [oscN_,phih] ...
        = computeOscNeumann(coordinates,elements,@phi);
    [oscV_,fh] = computeOscVolume(vertices,triangles,@f);

    %*** compute Galerkin solution
    W = buildW(coordinates,elements) ...
             + buildHypsingStabilization(coordinates,elements);
    [b,lambdah] = buildHypsingVolRHS(coordinates,elements, ...
        phih,vertices,triangles,fh);
    x = W\b;

    %*** stopping criterion
    if ( size(elements,1) + size(triangles,1) ) > nEmax
      break;
    end
    
    %*** compute weighted-residual estimator rho for hypersingular term
    rho_ = computeEstHypResidual(coordinates,elements,x,phih,lambdah);

    %*** compute weighted-residual estimator rho_V for Symm's equation to
    %*** measure error induced within buildHypsingVolRHS
    rho_V_ = computeEstSlpResidual(vertices,triangles,elements,lambdah,[],-fh);

    %*** mark elements for refinement
    [marked_elements,marked_triangles] ...
        = markElements(theta,rho_+rho_V_+oscN_,oscV_);

    %*** generate new mesh
    [vertices,triangles,elements] ...
        = refineMesh(vertices,triangles,elements, ...
        marked_triangles,marked_elements);
    coordinates = vertices(unique(elements),:);
end

%*** visualize exact and adaptively computed solution
plotArclengthS1(coordinates,elements,x,@g,1);

%*** visualize exact and approximated Neumann data
plotArclengthP0(coordinates,elements,phih,@phi,2);
