% adaptiveHypsingVol provides the implementation of an adaptive mesh-refining 
% algorithm for the hypersingular integral equation with volume force.
%
% Usage: adaptiveHypsingVol
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the hypersingular integral
%              equation
%
%                W*u = (K'-1/2)*phiN - N_1 f
%
%              which is equivalent to the Neumann problem
% 
%                -Laplace(u) = f in Omega with du/dn = phiN on Gamma
%
%              To this end, we use functionality provided by the HILBERT
%              program package. As example serves problem lshape2 with an
%              L-shaped domain Omega and known exact solution
%
%                u(x) = |x-z|^(1.8) + r^(2/3)*cos(2 \theta / 3)
%
%              with z = [0.14 0.14].
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) refine given boundary mesh Ecoarse uniformly and obtain
%                    Efine
%                (2) build Galerkin data w.r.t. (Efine,Tcoarse)
%                (3) compute Galerkin solution w.r.t. (Efine,Tcoarse)
%                (4) compute Ecoarse-elementwise contributions of
%                    mu-tilde Dirichlet data oscillations,
%                    compute Tcoarse-elementwise contributions of
%                    volume data oscillations
%                (5) use Doerfler strategy to mark certain
%                    (Ecoarse,Tcoarse)-elements
%                (6) refine marked elements to obtain new mesh
%                    (Ecoarse,Tcoarse)
%                (7) iterate as long as #(Ecoarse,Tcoarse) is lower than
%                    given bound
%
%              After the computation, the Galerkin solution uh and the
%              discrete Neumann data phih is plotted over the arclength.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape2 for demonstration purposes
addpath('lshape2/');

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** extract boundary mesh
[vertices,triangles,coordinates,elements] = buildSortedMesh(vertices,triangles);

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter 
theta = 0.25;
rho = 0.25;

%*** adaptive mesh-refining algorithm
while 1
    
    fprintf('number of elements: N = %d (Gamma), %d (Omega)\r', ...
        size(elements,1),size(triangles,1));

    %*** build uniformly refined mesh
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    %*** discretize Neumann and volume data and compute data oscillations
    [osc_fine,phih_fine] ...
        = computeOscNeumann(coordinates_fine,elements_fine,@phi);
    [oscV,fh] = computeOscVolume(vertices,triangles,@f);
    oscN = osc_fine(father2son(:,1)) + osc_fine(father2son(:,2));

    %*** compute fine-mesh solution
    W_fine = buildW(coordinates_fine,elements_fine) ...
             + buildHypsingStabilization(coordinates_fine,elements_fine);
    [b_fine,y_fine] = buildHypsingVolRHS(coordinates_fine,elements_fine, ...
        phih_fine,vertices,triangles,fh);
    x_fine = W_fine\b_fine;

    %*** stopping criterion
    if ( size(elements,1) + size(triangles,1) ) > nEmax
      break;
    end
    
    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde_ = computeEstHypMuTilde(elements_fine,elements,father2son,...
                                         x_fine);

    %*** compute (h-h/2)-error estimator tilde-mu for Symm's equation to
    %*** measure error induced within buildHypsingVolRHS
    mu_V_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,y_fine);

    %*** mark elements for refinement
    [marked_elements,marked_triangles] ...
        = markElements(theta,mu_tilde_+mu_V_tilde_+oscN,oscV);

    %*** generate new mesh
    [vertices,triangles,elements] ...
        = refineMesh(vertices,triangles,elements,marked_triangles,marked_elements);
    coordinates = vertices(unique(elements),:);
end

%*** visualize exact and adaptively computed solution
plotArclengthS1(coordinates_fine,elements_fine,x_fine,@g,1);

%*** visualize exact and approximated Neumann data
plotArclengthP0(coordinates_fine,elements_fine,phih_fine,@phi,2);
