% adaptiveMixedVol provides the implementation of an adaptive mesh-refining
% algorithm for the symmetric integral formulation of a mixed boundary value
% problem with volume force.
%
% Usage: adaptiveMixedVol
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              This example file provides an implementation of an adaptive
%              mesh-refining algorithm for the symmetric integral formulation 
%              of a mixed boundary value problem
%
%                -Laplace(u)  = f    in Omega
%                         u   = uD   on GammaD
%                       du/dn = phiN on GammaN
%
%              using the functionality provided by the HILBERT program package.
%              As example serves problem lshape2 with an L-shaped domain Omega
%              and known exact solution
%
%                u(x) = |x-z|^(1.8) + r^(2/3)*cos(2 \theta / 3)
%
%              with z = [0.14 0.14].
%              which exhibits a generic singularity. Given an initial mesh,
%              the adaptive algorithm implemented below reads as follows:
%
%                (1) refine given boundary mesh Ecoarse uniformly and obtain
%                    Efine
%                (2) build Galerkin data w.r.t. (Efine,Tcoarse)
%                (3) compute Galerkin solution w.r.t. (Efine,Tcoarse)
%                (4) compute Ecoarse-elementwise contributions of
%                    mu-tilde Dirichlet data oscillations,
%                    compute Tcoarse-elementwise contributions of
%                    volume data oscillations
%                (5) use Doerfler strategy to mark certain
%                    (Ecoarse,Tcoarse)-elements
%                (6) refine marked elements to obtain new mesh
%                    (Ecoarse,Tcoarse)
%                (7) iterate as long as #(Ecoarse,Tcoarse) is lower than
%                    given bound
%
%              After the computation, the discrete Cauchy data (uh,phih) are 
%              plotted over the arclength. Here, the discrete function uh is
%              an S1(Gamma)-approximation of the trace uD := u|_Gamma, whereas
%              phih is a P0(Gamma)-approximation of the normal derivative
%              phiN := du/dn on Gamma.
%
% (C) 2010-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../../lib/');

%*** use lshape2 for demonstration purposes
addpath('lshape2/');

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = [1 2;2 3;3 6;6 5];
neumann = [5 8;8 7;7 4;4 1];

%*** maximal number of elements
nEmax = 200;

%*** adaptivity parameter
theta = 0.25;
rho = 0.25;

%*** extract boundary mesh
[vertices,triangles,coordinates,neumann,dirichlet] = ...
    buildSortedMesh(vertices,triangles,neumann,dirichlet);

%*** initialize Dirichlet data
uDh = zeros(size(coordinates,1),1);
uDh(unique(dirichlet)) = g(coordinates(unique(dirichlet),:));

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
[coordinates_fine,dirichlet_fine,neumann_fine,...
    father2dirichlet_fine,father2neumann_fine] ...
    = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** rearrange indices such that Neumann nodes are first
[coordinates_fine,neumann_fine,dirichlet_fine] = ...
    buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

%*** discretize data and compute corresponding data oscillations for fine mesh
[oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          father2neumann_fine,neumann,uDh,@g,@phi);
[oscV,fh] = computeOscVolume(vertices,triangles,@f);
oscD = sum(oscD_fine(father2dirichlet_fine),2);
oscN = sum(oscN_fine(father2neumann_fine),2);

%*** adaptive mesh-refining algorithm
while 1

    fprintf('number of elements: N = %d (Gamma), %d (Omega)\r', ...
        size(neumann,1)+size(dirichlet,1),size(triangles,1));

    %*** compute integral operators for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);  
    W_fine = buildW(coordinates_fine,elements_fine);

    %*** compute right-hand side for fine mesh
    [b_nodes_fine,b_elements_fine,Nell_fine] = buildMixedVolRHS( ...
        coordinates_fine,dirichlet_fine,neumann_fine,uDh_fine,phiNh_fine, ...
	vertices,triangles,fh,V_fine,K_fine,W_fine);
                                  
    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nN_fine = length(freeNeumann_fine);

    %*** shrink integral operators and right-hand side
    W_fine = W_fine(freeNeumann_fine,freeNeumann_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    b_nodes_fine = b_nodes_fine(freeNeumann_fine);
    b_elements_fine = b_elements_fine(freeDirichlet_fine);

    %*** compute Galerkin solution for fine mesh
    x = [ W_fine K_fine' ; -K_fine V_fine ] \ [ b_nodes_fine ; b_elements_fine ];

    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    xN_fine = zeros(nC_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine);      %** dof on Neumann boundary
    xD_fine = x((1:nD_fine) + nN_fine);            %** dof on Dirichlet boundary
   
    %*** stopping criterion + size(triangles,1)
    if (size(neumann,1) + size(dirichlet,1) + size(triangles,1) > nEmax )
      break;
    end

    %*** compute error estimates
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet,father2dirichlet_fine, ...
                                     xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann,father2neumann_fine, ...
                                     xN_fine);
    mu1_tilde_D = computeEstSlpMuTilde(coordinates,dirichlet, ...
        father2dirichlet_fine,Nell_fine(1:nD_fine));
    mu1_tilde_N = computeEstSlpMuTilde(coordinates,neumann, ...
        father2neumann_fine,Nell_fine(nD_fine+1:end));
				     
    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann,marked_triangles] ...
        = markElements(theta,muD_tilde + oscD + mu1_tilde_D, ...
        muN_tilde + oscN + mu1_tilde_N,oscV);
    
    %*** generate new mesh
    [vertices,triangles,dirichlet,neumann,father2triangles, ...
        father2dirichlet,father2neumann_adap] ...
        = refineMesh(vertices,triangles,dirichlet,neumann,marked_triangles, ...
	             marked_dirichlet,marked_neumann);

    %*** rearrange indices such that Neumann nodes are first
    [vertices,triangles,coordinates,neumann,dirichlet] = ...
        buildSortedMesh(vertices,triangles,neumann,dirichlet);

    %*** generate fine mesh
    neumann_coarse = neumann_fine;
    father2neumann_coarse = father2neumann_fine;
    [coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine, ...
        father2neumann_fine] = refineBoundaryMesh(coordinates, ...
                                                    dirichlet,neumann);

    %*** rearrange indices such that Neumann nodes are first
    [coordinates_fine,neumann_fine,dirichlet_fine] = ...
        buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

    %*** build coarse2fine array
    coarse2fine = generateFather2Son(father2neumann_adap, ...
                                     father2neumann_coarse, ...
                                     father2neumann_fine);

    %*** discretize data and compute corresponding data oscillations for
    %*** fine mesh
    [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
                          coarse2fine,neumann_coarse,uDh_fine,@g,@phi);
    [oscV,fh] = computeOscVolume(vertices,triangles,@f);	  
    oscD = sum(oscD_fine(father2dirichlet_fine),2);
    oscN = sum(oscN_fine(father2neumann_fine),2);
end

%*** xN_fine = dof on Neumann boundary, i.e. update for Dirichlet data
%*** to obtain approximation uh of trace of PDE solution u
uh =  xN_fine+uDh_fine;

%*** plot approximate vs exact trace
plotArclengthS1(coordinates_fine,[dirichlet_fine;neumann_fine],uh,@g,2)

%*** xD_fine = dof on Dirichlet boundary, i.e. update for Neumann data
%*** to obtain approximation phih of normal derivative of PDE solution u
phih = [xD_fine ; phiNh_fine(nD_fine+1:end)];

%*** plot approximate vs exact normal derivative
plotArclengthP0(coordinates_fine,[dirichlet_fine;neumann_fine],phih,@phi,1)
