function [nE,nV,time,memory,eta,eta_tilde,mu,mu_tilde,mu_V_tilde,oscGamma, ...
          oscOmega,err] ...
    = exampleAdapHypsingVol(vertices,triangles,phi,f,nVmax,theta,varargin)

% exampleAdapHypsingVol provides the experimental data for an adaptive
% mesh-refining strategy for the Hypersingular Integral Equation
% with non-homogeneous volume force
%
% Usage: [N,NV,TIME,MEMORY,ETA,ETA_TILDE,MU,MU_TILDE,OSCN,OSCV,ESTIMATOR] ...
%            = exampleAdapHypsingVol(VERTICES,VOLUMES,PHI,F,MMAX, THETA, ...
%                                    [,UD] [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let an initial triangulation of the domain T0 be given.
%              For the data f on Omega and phiN on
%              Gamma we numerically solve the hypersingular integral equation
%
%                W*u = (1/2 - K')phiN - N_1*f on Gamma=boundary(Omega)
%
%              for some  Lipschitz domain Omega. To that end, we realize
%              the following adaptive algorithm:
%              (1) compute uniformly refined mesh E_fine
%              (2) compute Galerkin matrix and right-hand side
%              (3) compute Galerkin solution u_fine
%              (4) compute refinement indicators mu-tilde and data-oscillations
%              (5) use Doerfler strategy for marking
%              (6) refine marked elements and obtain a new mesh
%              (7) iterate until coarse mesh has sufficiently many elements
%
%              Furthermore, the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, mu-tilde, and mu_V-tilde are computed.
%              If Dirichlet data uD are provided by the user, we additionally
%              compute reliable error bounds for the approximation error.
%              If a figure number is specified, all error quantities are
%              plotted on the fly during the computation over the number
%              of degrees of freedom and the total computation time.
%              The latter considers precisely the time
%              cumulatively necessary to perform the adaptive loop, i.e. the
%              computational overhead caused by computation of eta, eta-tilde,
%              mu, mu-tilde, mu_V-tilde and the reliable error bound is not 
%              included in the time measurement.
%
%              VERTICES gives the coordinates for all
%              nodes of the triangulation of the domain. VOLUMES specifies the
%              triangular volume elements. 
%              The function handle PHI allows to evaluate the given Neumann
%              data. For an
%              (M x 2)-matrix of nodes X and the respective boundary elements
%              given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values, i.e.
%              Y(j) := PHI(X(j,:)). The function handle F allows to
%              evaluate the volume forces in Omega. For an (M x 2)-matrix of
%              nodes X, F returns the (M x 1)-vector of functiona values, i.e.
%              Y(j) := f(X(j,:)). The algorithm terminates, when the coarse
%              volume mesh has more than MMAX elements.
%              THETA is the adaptivity parameter used for Doerfler marking.
%              If the exact solution of the Hypersingular Integral Equation is
%              known, it can be provided by means of the optional function
%              handle UD.
%              The function handle UD allows to evaluate the given Dirichlet
%              data. For an (M x 2)-matrix of nodes X, UD returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := uD(X(j,:)). The optional parameter FIG gives the
%              figure number where a convergence graph is plotted on the fly.
%
%              All output parameters are vectors containing the respective
%              values corresponding to each step of the adaptive loop. N
%              containins the number of coarse boundary mesh elements.
%              NV contains the number of coarse volume mesh elements. TIME
%              contains the cumulatively measured computation time necessary to
%              perform the adaptive loop up to each step. MEMORY contains
%              the memory consumption. ETA, ETA_TILDE, MU,
%              MU_TILDE and MU_V_TILDE contain the global error estimator
%              quantities computed.
%              OSCN contains the global data oscillations w.r.t the
%              Neumann data. OSCV contains the global data oscillations w.r.t
%              the volume forces. The optional output parameter ERR contains a
%              reliabel upper bound for the error, provided that the solution
%              UD was given as input parameter.
%
% (C) 2010-2012 HILBERT-Team '10 , '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at

% Version: 3.0

if nargin > 6 && isnumeric(varargin{1})
    figure_number = varargin{1};
elseif nargin > 7 && isnumeric(varargin{2})
    figure_number = varargin{2};
else
    figure_number = false;
end

%*** Check whether optional function uD is given
if nargin > 6 && isa(varargin{1},'function_handle')
    u_D = varargin{1};
elseif nargin > 7 && isa(varargin{2},'function_handle')
    u_D = varargin{2};
else
    u_D = false;
end

%*** extract boundary mesh
[vertices,triangles,coordinates,elements] = ...
    buildSortedMesh(vertices,triangles);

%*** adaptive mesh-refining algorithm
counter = 1;

while size(triangles,1) < nVmax
    fprintf('*** problem size (adaptive) = %d (Gamma), %d (Omega)\n', ...
        size(elements,1),size(triangles,1));
    
    %
    %--- start time measurement for adaptive mesh-refinment
    %
    time_ = cputime;
    
    %*** compute fine-mesh solution
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    W_fine = buildW(coordinates_fine,elements_fine) ...
        + buildHypsingStabilization(coordinates_fine,elements_fine);
    
    % compute Neumann- and volume data and data oscillation
    [oscN_fine,phih_fine] = computeOscNeumann(coordinates_fine,...
        elements_fine,phi);
    [oscV,fh] = computeOscVolume(vertices,triangles,f);
    
    [b_fine,y_fine] = buildHypsingVolRHS(coordinates_fine,elements_fine, ...
        phih_fine,vertices,triangles,fh);
    x_fine = W_fine\b_fine;
    
    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde_ = computeEstHypMuTilde(elements_fine,elements,father2son,x_fine);
    
    %*** compute (h-h/2)-error estimator tilde-mu for Symm's equation to
    %*** measure error induced within buildHypsingVolRHS
    mu_V_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,y_fine);
    
    %*** compute data oscillations
    oscN = oscN_fine(father2son(:,1))+oscN_fine(father2son(:,2));
    
    %*** mark elements for refinement
    [marked_elements,marked_triangles] ...
        = markElements(theta,mu_tilde_+mu_V_tilde_+oscN,oscV);
    
    %*** generate new mesh
    [vertices_new,triangles_new,elements_new] ...
        = refineMesh(vertices,triangles,elements,marked_triangles,...
                     marked_elements);
    coordinates_new = vertices_new(unique(elements_new),:);
    
    %
    %--- stop time measurement for adaptive mesh-refinement
    %
    time_ = cputime - time_;
    if counter>1
        time_ = time(counter-1)+time_;
    end
    
    %
    %--- memory consumption
    %
    % memory of N_fine, V_fine, K_fine, M_fine, in byte
    memory_ = 8*(size(triangles,1)^2 + size(elements_fine,1)^2 + ...
        2*(size(elements_fine,1)*size(coordinates_fine,1)) + ...
        size(coordinates_fine,1)^2);
    
    %*** Build index field k = coarse2fine (j) such that j-th node of coarse
    %*** mesh coincides with k-th nodes of fine mesh
    nC = size(coordinates,1);
    coarse2fine = zeros(nC,1);
    coarse2fine(elements) = [ elements_fine(father2son(:,1),1) ...
        elements_fine(father2son(:,2),2) ];
    
    %*** Build index field k = element2midpoint(j) such that midpoint of j-th
    %*** element of coarse mesh is k-th node of fine mesh
    element2midpoint = elements_fine(father2son(:,1),2);
    
    %*** Build index field [i,j] = node2element(k) such that
    %*** intersection(Ti,Tj) = zk
    nE_ = size(elements,1);
    node2element = zeros(nC,2);
    node2element(elements(:,1),2) = (1:nE_)';
    node2element(elements(:,2),1) = (1:nE_)';
    
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b = b_fine(coarse2fine) + 0.5*sum(b_fine(element2midpoint(node2element)),2);
    
    %*** compute coarse-mesh solution for error estimation
    W = buildW(coordinates,elements) ...
        + buildHypsingStabilization(coordinates,elements);
    x= W\b;
    
    %*** free obsolete memory
    clear W b
    clear idx coarse2fine element2midpoint node2element
    
    %*** compute (h-h/2)-estimators eta, tilde-eta, and mu
    eta_ = computeEstHypEta(elements_fine,elements,father2son,W_fine,x_fine,x);
    eta_tilde_ = computeEstHypEtaTilde(elements_fine,elements,father2son,...
        W_fine,x_fine);
    mu_ = computeEstHypMu(elements_fine,elements,father2son,x_fine,x);
    
    %*** compute Dirichlet data oscillation as reliable error bound
    if isa(u_D,'function_handle')
        err_ = computeErrDirichlet(coordinates_fine,elements_fine,x_fine,u_D);
        err(counter) = sqrt(sum(err_));
    else
        err(counter) = 0;
    end
    
    %*** free obsolete memory
    clear coordinates_fine elements_fine father2son
    clear W_fine b_fine x_fine
    
    %*** store computed data
    nE(counter) = size(elements,1);
    nV(counter) = size(triangles,1);
    time(counter) = time_;
    memory(counter) = memory_;
    eta(counter) = sqrt(eta_);
    eta_tilde(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_));
    mu_tilde(counter) = sqrt(sum(mu_tilde_));
    mu_V_tilde(counter) = sqrt(sum(mu_V_tilde_));
    oscGamma(counter) = sqrt(sum(oscN));
    oscOmega(counter) = sqrt(sum(oscV));
    
    %*** plot results
    if figure_number
        figure(figure_number);
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(2,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,mu_V_tilde,'bs-');
        loglog(nE,oscGamma,'<k-')
        loglog(nE,oscOmega,'sk-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'tilde-V-mu (adap)','oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('number of coarse-mesh boundary elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(2,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,mu_V_tilde,'bs-');
        loglog(time,oscGamma,'<k-')
        loglog(time,oscOmega,'sk-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'tilde-V-mu (adap)','oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
        
        subplot(2,2,3)
        plotArclengthS1(coordinates,elements,x,u_D,-1);
        drawnow
        
        subplot(2,2,4)
        trisurf(triangles,vertices(:,1),vertices(:,2),f(vertices),...
            'facecolor','interp')
        axis equal
        view(2)
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
    coordinates = coordinates_new;
    elements = elements_new;
    vertices = vertices_new;
    triangles = triangles_new;
end

