function [nE,nV,time,memory,eta,eta_tilde,mu,mu_tilde,mu_V_tilde,oscGamma, ...
          oscOmega,err] ...
    = exampleAdapMixedVol(vertices,triangles,dirichlet,neumann,uD,phiN,f, ...
                          nVmax,theta,varargin)

% exampleAdapMixedVol provides the experimental data for the numerical solution
% of the equivalent integral formulation of a mixed boundary value problem
% with non-homogeneous volume forces and adaptive mesh refinement.
%
% Usage: [N,NV,TIME,MEMORY,ETA,ETA_TILDE,MU,MU_TILDE,MU_V_TILDE, ...
%         OSCG,OSCV [,ERR]] ...
%                = exampleAdapMixedVol(COORDINATES,DIRICHLET,NEUMANN,UD, ...
%                                      PHIN,VOLUMES,F,NMAX,MMAX [,FIG] )
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let an initial triangulation of the domain T0 be given.
%              For the data f on Omega, uD on GammaD and phiN on GammaN,
%              we numericall solve the mixed problem
%
%                -K*uN + V*phiD  = 1/2 + K*uD - V*phiN - N0*f
%                 W*uN + K'*phiD = 1/2 - W*uD - K'*phiN - N1*f
%
%              for some  Lipschitz domain Omega. To that end, we realize
%              the following adaptive algorithm:
%              (1) compute uniformly refined mesh E_fine
%              (2) compute Galerkin matrix and right-hand side
%              (3) compute Galerkin solution u_fine
%              (4) compute refinement indicators mu-tilde and data-oscillations
%              (5) use Doerfler strategy for marking
%              (6) refine marked elements and obtain a new mesh
%              (7) iterate until coarse mesh has sufficiently many elements
%
%              Furthermore the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, mu_tilde, mu_V-tilde as well as data oscillations are
%              computed. Assuming the data uD and phiN are,
%              in fact, given over the whole boundary Gamma, we additionally
%              compute a reliable error bound for the overall approximation
%              error on GammaD and GammaN. If a figure number is specified, all
%              error quantities are plotted on-the-fly during the computation
%              over the number of degrees of freedom and the computational time.
%              The latter considers precisely the time necessary to set up the
%              mesh and solve the coarse system, i.e. the computational overhead
%              caused by calculation of eta, eta-tilde, mu, mu-tilde,
%              mu_V-tilde and the
%              reliable error bound is not included in the time measurement.
%              Moreover, the time measurement is not cumulative.
%              As a second important parameter we compute the memory
%              consumption of all discrete integral operators which are built
%              in this function or any functions called by it.
%
%              VERTICES gives the coordinates for all
%              nodes of the triangulation of the domain. VOLUMES specifies the
%              triangular volume elements. 
%              The function handle UD allows to evaluate the given Dirichlet
%              data. For an (M x 2)-matrix of nodes X, UD returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := uD(X(j,:)).
%              The function handle PHI allows to evaluate the given Neumann
%              data. For an
%              (M x 2)-matrix of nodes X and the respective boundary elements
%              given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values, i.e.
%              Y(j) := PHI(X(j,:)). The function handle F allows to
%              evaluate the volume forces in Omega. For an (M x 2)-matrix of
%              nodes X, F returns the (M x 1)-vector of functiona values, i.e.
%              Y(j) := f(X(j,:)). The algorithm terminates, when the coarse
%              volume mesh has more than MMAX elements.
%              THETA is the adaptivity parameter used for Doerfler marking.
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial meshes. N contains the number of
%              coarse boundary mesh elements. NV contains the number of 
%              coarse volume mesh elements. TIME contains the computation
%              time necessary to compute the coarse mesh solution. MEMORY
%              contains the memory consumption in [byte]. ETA,
%              ETA_TILDE, MU, MU_TILDE and MU_V_TILDE contain the global error
%              estimator quantities computed.
%              OSCD contains the global data oscillations
%              w.r.t the Dirichlet data. OSCV contains the global data
%              oscillations w.r.t the volume forces. The optional output
%              parameter ERR contains a reliabel upper bound for the error,
%              provided that UD and PHIN where given on the whole boundary.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** Check whether optional figure number is given
if nargin > 9 && isnumeric(varargin{1})
    figure_number = varargin{1};
else
    figure_number = false;
end

%*** rearrange indices such that Neumann nodes are first, extract coordinates
[vertices,triangles,coordinates,neumann,dirichlet] ...
    = buildSortedMesh(vertices,triangles,neumann,dirichlet);

%*** initialize Dirichlet data
uDh = zeros(size(coordinates,1),1);
uDh(unique(dirichlet)) = uD(coordinates(unique(dirichlet),:));

%*** adaptive mesh-refining algorithm
counter = 1;

%
%--- start time-measurement for adaptive mesh-refinement
%
time_ = cputime;

%*** Perform uniform refinement before starting the loop
%*** refine mesh uniformly
[coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine, ...
    father2neumann_fine] = refineBoundaryMesh(coordinates,dirichlet,neumann);

%*** rearrange indices such that Neumann nodes are first
[coordinates_fine,neumann_fine,dirichlet_fine] = ...
    buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);

%*** discretize data and compute corresponding data oscillations for fine mesh
[oscD_fine,oscN_fine,uDh_fine,phiNh_fine] = ...
    computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
    father2neumann_fine,neumann,uDh,uD,phiN);
[oscV,fh] = computeOscVolume(vertices,triangles,f);

while size(triangles,1) < nVmax
    
    if counter > 1
        %
        %--- start time-measurement for adaptive mesh-refinement
        %
        time_ = cputime;
        
        %***
        old_neumann_fine = neumann_fine;
        old_father2neumann_fine = father2neumann_fine;
        
        [coordinates_fine,dirichlet_fine,neumann_fine,father2dirichlet_fine,...
            father2neumann_fine] ...
            = refineBoundaryMesh(coordinates,dirichlet,neumann);
        
        %*** rearrange nodes such that neumann nodes are first
        [coordinates_fine,neumann_fine,dirichlet_fine] = ...
            buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);
        
        %*** build old2fine array which links old fine mesh to new fine mesh
        old2fine = generateFather2Son(father2neumann_new, ...
            old_father2neumann_fine, ...
            father2neumann_fine);
        
        %*** discretize data, compute oscillations for fine mesh
        [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] = ...
            computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
            old2fine,old_neumann_fine,uDh_fine,uD,phiN);
        [oscV,fh] = computeOscVolume(vertices,triangles,f);
    end
    
    oscD = sum(oscD_fine(father2dirichlet_fine),2);
    oscN = sum(oscN_fine(father2neumann_fine),2);
    
    %*** compute integral operators and right hand side for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);
    W_fine = buildW(coordinates_fine,elements_fine);
    [b1_fine,b2_fine,Nell_fine] = buildMixedVolRHS(coordinates_fine, ...
        dirichlet_fine,neumann_fine,uDh_fine,phiNh_fine,vertices,triangles, ...
        fh,V_fine,K_fine,W_fine);
    
    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nN_fine = length(freeNeumann_fine);
    
    %*** compute fine-mesh Galerkin solution
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    nodesNeumann = unique(neumann_fine);
    W_fine = W_fine(nodesNeumann,nodesNeumann);
    x = [ W_fine(freeNeumann_fine,freeNeumann_fine) K_fine' ;  ...
        -K_fine V_fine ] ...
        \ [ b1_fine(freeNeumann_fine) ; b2_fine(freeDirichlet_fine) ];
    
    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    dim_fine = length(unique(neumann_fine));
    xN_fine = zeros(dim_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine); %** dof on Neumann boundary
    xD_fine = x((1:nD_fine) + nN_fine);       %** dof on Dirichlet boundary
    
    %*** compute error estimates
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet, ...
        father2dirichlet_fine,xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann, ...
        father2neumann_fine,xN_fine);
    mu_V_tilde_D = computeEstSlpMuTilde(coordinates,dirichlet, ...
        father2dirichlet_fine,Nell_fine(1:nD_fine));
    mu_V_tilde_N = computeEstSlpMuTilde(coordinates,neumann, ...
        father2neumann_fine,Nell_fine(nD_fine+1:end));
    
    %*** mark elements for refinement
    [marked_dirichlet,marked_neumann,marked_triangles] ...
        = markElements(theta,muD_tilde + oscD + mu_V_tilde_D, ...
        muN_tilde + oscN + mu_V_tilde_N,oscV);
    
    %*** generate new mesh
    [vertices_new,triangles_new,dirichlet_new,neumann_new,...
        father2triangles_new,father2dirichlet_new,father2neumann_new] ...
        = refineMesh(vertices,triangles,dirichlet,neumann,marked_triangles, ...
        marked_dirichlet,marked_neumann);
    
    %*** rearrange indices such that neumann nodes are first
    [vertices_new,triangles_new,coordinates_new,neumann_new,dirichlet_new] = ...
        buildSortedMesh(vertices_new,triangles_new,neumann_new,dirichlet_new);
    
    %
    %--- stop time-measurement for adaptive mesh-refinement
    %
    time_ = cputime - time_;
    if counter>1
        time_ = time(counter-1)+time_;
    end
    
    %
    %--- memory consumption
    %
    % memory of N_fine + V_fine + K_fine + M_fine + W_fine
    memory_ = size(triangles,1)^2 + size(elements_fine,1)^2 + ...
        2*size(elements_fine,1)*size(coordinates_fine,1) + ...
        size(coordinates_fine,1)^2;
    
    %--------------------------------------------------------------------------
    % compute coarse mesh solution
    %--------------------------------------------------------------------------
    %*** compute degrees of freedom for coarse mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;
    
    %*** compute integral operators and right hand side for coarse mesh
    elements = [dirichlet;neumann];
    father2son = [father2dirichlet_fine ; father2neumann_fine + nD_fine];
    V = buildV(coordinates,elements);
    K = buildK(coordinates,elements);
    W = buildW(coordinates,elements);
    %[b1,b2] = buildMixedVolRHS(coordinates,dirichlet,neumann,vertices, ...
    %    triangles,V,K,W,@g,@phi,fh);
    
    %*** build index field k = coarse2fine (j) such that j-th node of coarse
    %*** mesh coincides with k-th node of fine mesh
    nC = size(coordinates,1);
    coarse2fine = zeros(nC,1);
    coarse2fine(elements) = [ elements_fine(father2son(:,1),1) ...
        elements_fine(father2son(:,2),2) ];
    
    %*** build index field k = element2midpoint(j) such that midpoint of j-th
    %*** element of coarse mesh is k-th node of fine mesh
    element2midpoint = elements_fine(father2son(:,1),2);
    
    %*** build index field [i,j] = node2element(k) such that
    %*** intersection(Ei,Ej) = zk
    nE_ = size(elements,1);
    node2element = zeros(nC,2);
    node2element(elements(:,1),2) = (1:nE_)';
    node2element(elements(:,2),1) = (1:nE_)';
    
    %*** obtain coarse-mesh RHS from fine-mesh RHS
    b1 = b1_fine(coarse2fine) ...
        + 0.5*sum(b1_fine(element2midpoint(node2element)),2);
    b2 = b2_fine(father2son(:,1))+b2_fine(father2son(:,2));
    
    %*** compute Galerkin solution w.r.t. coarse mesh for error estimation
    V = V(freeDirichlet,freeDirichlet);
    K = K(freeDirichlet,freeNeumann);
    W = W(freeNeumann,freeNeumann);
    x = [ W K' ; -K V ] \ [ b1(freeNeumann) ; b2(freeDirichlet) ];
    
    %*** compute solution vectors w.r.t. P0(GammaD) und S1(GammaN)
    nN = length(freeNeumann);
    dim = length(unique(neumann));
    xN = zeros(dim,1);
    xN(freeNeumann) = x(1:nN);     %** dof on Neumann boundary
    xD = x((1:nD) + nN);           %** dof on Dirichlet boundary
    
    %*** compute mu estimators for GammaD and GammaN
    muD = computeEstSlpMu(coordinates,dirichlet,father2dirichlet_fine,...
                          xD_fine,xD);
    muN = computeEstHypMu(neumann_fine,neumann,father2neumann_fine,xN_fine,xN);
    
    %*** compute tilde-eta estimators for GammaD and GammaN
    etaD_tilde = computeEstSlpEtaTilde(father2dirichlet_fine,V_fine,xD_fine);
    etaN_tilde = computeEstHypEtaTilde(neumann_fine,neumann,...
        father2neumann_fine,W_fine,xN_fine);
    
    %*** compute eta estimators for GammaD and GammaN
    etaD = computeEstSlpEta(father2dirichlet_fine,V_fine,xD_fine,xD);
    etaN = computeEstHypEta(neumann_fine,neumann,father2neumann_fine,W_fine, ...
                            xN_fine,xN);
    
    %*** compute reliable error bound for GammaD and GammaN
    errD = computeErrNeumann(coordinates,dirichlet,xD,phiN);
    xNg = uDh_fine(coarse2fine);
    
    xNg(freeNeumann) = xNg(freeNeumann) + x(1:nN);
    errN = computeErrDirichlet(coordinates,neumann,xNg,uD);
    
    %*** compute Neumann data on Gamma:
    %*** on Gamma_N, use L2-projection of phiNh_fine,
    %*** on GammaD, use solution xD on Gamma_D
    xDn = zeros(size(elements,1),1);
    xDn = 0.5*sum(phiNh_fine(father2son),2);
    xDn(1:nD) = xDn(1:nD) + xD;
    
    %*** store computed data
    fprintf('*** problem size (adaptive) = %d (Gamma), %d (Omega)\n', ...
        size(elements,1),size(triangles,1));
    nE(counter) = size(elements,1);
    nV(counter) = size(triangles,1);
    time(counter) = time_;
    memory(counter) = memory_;
    eta(counter) = sqrt(etaD + etaN);
    eta_tilde(counter) = sqrt(etaD_tilde + etaN_tilde);
    mu(counter) = sqrt(sum([muD ; muN]));
    mu_tilde(counter) = sqrt(sum([muD_tilde; muN_tilde]));
    mu_V_tilde(counter) = sqrt(sum([mu_V_tilde_D; mu_V_tilde_N]));
    oscGamma(counter) = sqrt(sum([oscD; oscN]));
    oscOmega(counter) = sqrt(sum(oscV));
    err(counter) = sqrt(sum([errD ; errN]));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(3,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,mu_V_tilde,'bv-')
        loglog(nE,oscGamma,'<k-')
        loglog(nE,oscOmega,'sk-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh boundary elements')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'tilde-mu_V (adap)','oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(3,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,mu_V_tilde,'bv-')
        loglog(time,oscGamma,'<k-')
        loglog(time,oscOmega,'sk-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'tilde-mu_V (adap)','oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
        
        subplot(3,2,3)
        plotArclengthS1(coordinates,[dirichlet;neumann],xNg,uD,-1);
        title('Dirichlet Data');
        subplot(3,2,5)
        
        plotArclengthP0(coordinates,[dirichlet;neumann],xDn,phiN,-1);
        title('Neumann Data');
        drawnow
        
        subplot(3,2,[4 6])
        trisurf(triangles,vertices(:,1),vertices(:,2),f(vertices),...
            'facecolor','interp')
        title('Volume Data');
        axis equal
        view(2)
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
    coordinates = coordinates_new;
    dirichlet = dirichlet_new;
    neumann = neumann_new;
    vertices = vertices_new;
    triangles = triangles_new;
end

