function [nE,nV,time,memory,eta,eta_tilde,mu,mu_tilde,oscGamma,oscOmega,err] ...
            = exampleAdapSymmVol(vertices,triangles,uD,f,nVmax,theta,varargin)

% exampleAdapSymmVol provides the experimental data for the numerical 
% solution of the equivalent integral formulation of a Dirichlet boundary
% value problem with non-homogeneous volume force by use of an
% adaptive mesh-refinement. 
%
% Usage: [N,NV,TIME,MEMORY,ETA,ETA_TILDE,MU,MU_TILDE,OSCD,OSCV, [,ERR]] ...
%           = exampleAdapSymmVol(VERTICES,VOLUMES,UD,F,MMAX,THETA [,PHI] [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              Let some initial triangulation of the domain T0 be given,
%              and let E0 be the induced boundary partition.
%              For the data f on Omega and uD on
%              Gamma we numerically solve Symm's integral equation
%
%                V*phi = (K+1/2)*uD - N*f on Gamma=boundary(Omega)
%
%              for some Lipschitz domain Omega. To that end, we realize
%              the following adaptive algorithm:
%              (1) compute uniformly refined mesh E_fine
%              (2) compute Galerkin matrix and right-hand side
%              (3) compute Galerkin solution u_fine
%              (4) compute refinement indicators mu-tilde and data-oscillations
%              (5) use Doerfler strategy for marking
%              (6) refine marked elements and obtain a new mesh
%              (7) iterate until coarse mesh has sufficiently many elements
%
%              Furthermore, the (h-h/2)-based error estimators eta, eta-tilde,
%              and mu are computed. If the Neumann data phi are
%              are provided by the user, we additionally compute reliable error
%              bounds for the approximation error. If a figure number is
%              specified, all error quantities are plotted on the fly during
%              the computation over the number of degrees of freedom and the
%              total computation time. The latter considers precisely the time
%              cumulatively necessary to perform the adaptive loop, i.e. the
%              computational overhead caused by computation of eta, eta-tilde,
%              mu, mu-tilde and the reliable error bound is not included in the 
%              time measurement.
%
%              VERTICES gives the coordinates for all
%              nodes of the triangulation of the domain. VOLUMES specifies the
%              triangular volume elements.
%              The function handle UD allows to evaluate the given Dirichlet
%              data. For an (M x 2)-matrix of nodes X, UD returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := uD(X(j,:)).  The function handle F allows to
%              evaluate the volume forces in Omega. For an (M x 2)-matrix of
%              nodes X, F returns the (M x 1)-vector of functiona values, i.e.
%              Y(j) := f(X(j,:)). The algorithm terminates, when the
%              volume mesh has more than MMAX elements.
%              THETA is the adaptivity parameter used for Doerfler marking. If
%              the exact solution of Symm's integral equations is known, it
%              can be provided by means of the optional function handle PHI.
%              For an (M x 2)-matrix of nodes X and the respective boundary
%              elements given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := PHI(X(j,:)). The optional parameter FIG gives the
%              figure number where a convergence graph is plotted on the fly.
%
%              All output parameters are vectors containing the respective
%              values corresponding to each step of the adaptive loop. N
%              containins the number of coarse boundary mesh elements.
%              NV contains the number of coarse volume mesh elements. TIME
%              contains the cumulatively measured computation time necessary to
%              perform the adaptive loop up to each step. MEMORY contains
%              the memory consumption. ETA, ETA_TILDE, MU,
%              and MU_TILDE contain the global error estimator quantities
%              computed. OSCD contains the global data oscillations w.r.t the
%              Dirichlet data. OSCV contains the global data oscillations w.r.t
%              the volume forces. The optional output parameter ERR contains a
%              reliabel upper bound for the error, provided that the solution
%              PHI was given as input parameter.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at

% Version: 3.0

%*** Check whether optional figure number is given
if nargin > 6 && isnumeric(varargin{1})
    figure_number = varargin{1};
elseif nargin > 7 && isnumeric(varargin{2})
    figure_number = varargin{2};
else
    figure_number = false;
end

%*** Check whether optional function phi is given
if nargin > 6 && isa(varargin{1},'function_handle')
    phi_N = varargin{1};
elseif nargin > 7 && isa(varargin{2},'function_handle')
    phi_N = varargin{2};
else
    phi_N = false;
end

%*** extract boundary mesh from volume mesh
[vertices,triangles,coordinates,elements]=buildSortedMesh(vertices,triangles);

%*** adaptive mesh-refining algorithm
counter = 1;

while size(triangles,1) < nVmax
    
    fprintf('*** problem size (adaptive) = %d (Gamma), %d (Omega)\n', ...
        size(elements,1),size(triangles,1));
    
    %
    %--- start time-measurement for adaptive mesh-refinement
    %
    time_ = cputime;
    
    %*** compute fine-mesh solution
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    
    V_fine = buildV(coordinates_fine,elements_fine);
    
    %*** discretize Dirichlet- and volume data and compute data oscillations
    [oscD_fine,uDh_fine] = ...
        computeOscDirichlet(coordinates_fine,elements_fine,uD);
    [oscV,fh] = computeOscVolume(vertices,triangles,f);
    b_fine = buildSymmVolRHS(coordinates_fine,elements_fine,uDh_fine,...
        vertices,triangles,fh);
    x_fine = V_fine\b_fine;
    
    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);
    
    %*** compute data oscillations
    oscD = oscD_fine(father2son(:,1))+oscD_fine(father2son(:,2));
    
    %*** mark elements for refinement
    [marked_elements,marked_triangles] = ...
        markElements(theta,mu_tilde_+oscD,oscV);
    
    %*** generate new mesh
    [vertices_new,triangles_new,elements_new] ...
        = refineMesh(vertices,triangles,elements,marked_triangles,...
                     marked_elements);
    coordinates_new = vertices_new(unique(elements_new),:);
    
    %
    %--- stop time-measurement for adaptive mesh-refinement
    %
    time_ = cputime - time_;
    if counter>1
        time_ = time(counter-1)+time_;
    end
    
    %
    %--- memory consumption
    %
    % memory of N_fine + V_fine + K_fine + M_fine in byte
    memory_ = 8*(size(triangles,1)^2 + size(elements_fine,1)^2 + ...
        2*size(elements_fine,1)*size(coordinates_fine,1));
    
    %*** compute coarse-mesh solution for error estimation
    b = b_fine(father2son(:,1))+b_fine(father2son(:,2));
    V = buildV(coordinates,elements);
    x = V\b;
    
    %*** free obsolete memory
    clear V b
    
    %*** compute (h-h/2)-estimators eta, tilde-eta, and mu
    eta_ = computeEstSlpEta(father2son,V_fine,x_fine,x);
    eta_tilde_ = computeEstSlpEtaTilde(father2son,V_fine,x_fine);
    mu_ = computeEstSlpMu(coordinates,elements,father2son,x_fine,x);
    
    %*** compute Neumann data oscillation as reliable error bound
    if isa(phi_N,'function_handle')
        err_ = computeErrNeumann(coordinates_fine,elements_fine,x_fine,phi_N);
        err(counter) = sqrt(sum(err_));
    else
        err(counter) = 0;
    end
    
    %*** free obsolete memory
    clear coordinates_fine elements_fine father2son
    clear V_fine b_fine x_fine
    
    %*** store computed data
    nE(counter) = size(elements,1);
    nV(counter) = size(triangles,1);
    time(counter) = time_;
    memory(counter) = memory_;
    eta(counter) = sqrt(eta_);
    eta_tilde(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_));
    mu_tilde(counter) = sqrt(sum(mu_tilde_));
    oscGamma(counter) = sqrt(sum(oscD));
    oscOmega(counter) = sqrt(sum(oscV));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(2,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,oscGamma,'<k-')
        loglog(nE,oscOmega,'sk-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh elements')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('number of coarse-mesh boundary elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(2,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,oscGamma,'<k-')
        loglog(time,oscOmega,'sk-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
            'oscD (adap)','oscV (adap)','error (adap)',3)
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
        
        %*** plot discrete solution
        subplot(2,2,3)
        plotArclengthP0(coordinates,elements,x,phi_N,-1);
        drawnow
        
        %*** plot volume mesh
        subplot(2,2,4)
        trisurf(triangles,vertices(:,1),vertices(:,2),f(vertices),...
            'facecolor','interp')
        axis equal
        view(2)
        drawnow
    end
    
    %*** increase counter and iterate
    counter = counter + 1;
    coordinates = coordinates_new;
    elements = elements_new;
    vertices = vertices_new;
    triangles = triangles_new;
end

