function [nE,nV,time,memory,eta,eta_tilde,mu,mu_tilde,mu_V_tilde,oscGamma, ...
          oscOmega,err] ...
    = exampleUnifMixedVol(vertices0,triangles0,dirichlet0,neumann0,uD,phiN, ...
                          f,nVmax,varargin)

% exampleUnifMixedVol provides the experimental data for the numerical solution
% of the equivalent integral formulation of a mixed boundary value problem
% with non-homogeneous volume forces and uniform mesh-refinement.
%
% Usage: [N,NV,TIME,MEMORY,ETA,ETA_TILDE,MU,MU_TILDE,MU_V_TILDE, ...
%         OSCG,OSCV [,ERR]] ...
%            = exampleUnifMixedVol(COORDINATES,DIRICHLET,NEUMANN,UD,PHIN, ...
%                                  VERTICES,VOLUMES,F,NMAX, ...
%                                  [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let an initial triangulation of the domain T0 be given.
%              For the data f on Omega, uD on GammaD and phiN on GammaN,
%              we numericall solve the mixed problem
%
%                -K*uN + V*phiD  = 1/2 + K*uD - V*phiN - N0*f
%                 W*uN + K'*phiD = 1/2 - W*uD - K'*phiN - N1*f
%
%              for some  Lipschitz domain Omega. To that end, we realize
%              the following algorithm:
%              (1) refine the initial mesh T0 k-times uniformly
%                  (1.1) let Ek be the boundary mesh induced by Tk
%              (2) compute Galerkin matrix and right-hand side
%              (3) compute Galerkin solution
%
%              Furthermore the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, mu_tilde, mu_V-tilde as well as data oscillations are
%              computed. Assuming the data uD and phiN are,
%              in fact, given over the whole boundary Gamma, we additionally
%              compute a reliable error bound for the overall approximation
%              error on GammaD and GammaN. If a figure number is specified, all
%              error quantities are plotted on-the-fly during the computation
%              over the number of degrees of freedom and the computational time.
%              The latter considers precisely the time necessary to set up the
%              mesh and solve the coarse system, i.e. the computational overhead
%              caused by calculation of eta, eta-tilde, mu, mu-tilde,
%              mu_V-tilde and the reliable error bound is not included in the
%              time measurement.
%              Moreover, the time measurement is not cumulative.
%              As a second important parameter we compute the memory
%              consumption of all discrete integral operators which are built
%              in this function or any functions called by it.
%
%              VERTICES gives the coordinates for all
%              nodes of the triangulation of the domain. VOLUMES specifies the
%              triangular volume elements. 
%              The function handle UD allows to evaluate the given Dirichlet
%              data. For an (M x 2)-matrix of nodes X, UD returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := uD(X(j,:)).
%              The function handle PHI allows to evaluate the given Neumann
%              data. For an
%              (M x 2)-matrix of nodes X and the respective boundary elements
%              given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values, i.e.
%              Y(j) := PHI(X(j,:)). The function handle F allows to
%              evaluate the volume forces in Omega. For an (M x 2)-matrix of
%              nodes X, F returns the (M x 1)-vector of functiona values, i.e.
%              Y(j) := f(X(j,:)). The algorithm terminates, when the coarse
%              volume mesh has more than MMAX elements.
%              THETA is the adaptivity parameter used for Doerfler marking.
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial meshes. N contains the number of
%              coarse boundary mesh elements. NV contains the number of 
%              coarse volume mesh elements. TIME contains the computation
%              time necessary to compute the coarse mesh solution. MEMORY
%              contains the memory consumption in [byte]. ETA,
%              ETA_TILDE, MU, MU_TILDE and MU_V_TILDE contain the global error
%              estimator quantities computed.
%              OSCD contains the global data oscillations
%              w.r.t the Dirichlet data. OSCV contains the global data
%              oscillations w.r.t the volume forces. The optional output
%              parameter ERR contains a reliabel upper bound for the error,
%              provided that UD and PHIN where given on the whole boundary.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** Check whether optional figure number is given
if nargin > 8 && isnumeric(varargin{1})
    figure_number = varargin{1};
else
    figure_number = false;
end

%*** rearrange indices such that Neumann nodes are first
[vertices0,triangles0,coordinates0,neumann0,dirichlet0] ...
    = buildSortedMesh(vertices0,triangles0,neumann0,dirichlet0);

%*** initialize Dirichlet data
uDh = zeros(size(coordinates0,1),1);
uDh(unique(dirichlet0)) = uD(coordinates0(unique(dirichlet0),:));

%*** start uniform computation
counter = 1;
coordinates = coordinates0;
dirichlet = dirichlet0;
neumann = neumann0;
vertices = vertices0;
triangles = triangles0;

while size(triangles,1) < nVmax
    %
    %--- start time-measurement for uniform mesh-refinement
    %
    time_ = cputime;
    
    %coordinates = coordinates0;
    dirichlet = dirichlet0;
    neumann = neumann0;
    vertices = vertices0;
    triangles = triangles0;
    for j=1:counter-1
        %*** refine mesh uniformly
        [vertices,triangles,neumann_new,dirichlet,father2vol, ...
            father2neumann,father2dirichlet] = refineMesh(vertices, ...
            triangles,neumann,dirichlet);
        
        %*** rearrange indices such that Neumann nodes are first
        [vertices,triangles,coordinates,neumann_new,dirichlet] ...
            = buildSortedMesh(vertices,triangles,neumann_new,dirichlet);
        
        %*** prolongate neumann data to the fine mesh
        uDh(neumann_new(father2neumann(:,1),2)) = 0.5*sum(uDh(neumann),2);
        uDh(neumann_new(father2neumann(:,1),1)) = uDh(neumann(:,1));
        uDh(neumann_new(father2neumann(:,2),2)) = uDh(neumann(:,2));
        uDh(unique(dirichlet)) = uD(coordinates(unique(dirichlet),:));
        
        neumann = neumann_new;
    end
    
    %*** compute coarse-mesh solution
    elements = [dirichlet;neumann];
    V = buildV(coordinates,elements);
    K = buildK(coordinates,elements);
    W = buildW(coordinates,elements);
    
    %*** compute degrees of freedom for coarse mesh
    nC = size(coordinates,1);
    nD = size(dirichlet,1);
    freeNeumann = setdiff(1:nC,unique(dirichlet));
    freeDirichlet = 1:nD;
    
    %*** discretize data and compute corresponding data oscillations
    [oscD,oscN,foo,phiNh] ...
        = computeOscMixed(coordinates,dirichlet,neumann,uD,phiN);
    [oscV,fh] = computeOscVolume(vertices,triangles,f);
    
    %*** compute right hand side and shrink it
    [b1,b2] = buildMixedVolRHS(coordinates,dirichlet,neumann,uDh,phiNh, ...
        vertices,triangles,fh,V,K,W);
    b1 = b1(freeNeumann);
    b2 = b2(freeDirichlet);
    
    % shrink integral operators
    V = V(freeDirichlet,freeDirichlet);
    K = K(freeDirichlet,freeNeumann);
    W = W(freeNeumann,freeNeumann);
    
    %*** compute Galerkin solution for coarse mesh
    x = [ W K' ; -K V ] \ [ b1 ; b2 ];
    
    %*** compute solution vectors w.r.t. P0(GammaD) and S1(GammaN)
    nN = length(freeNeumann);
    dim = length(unique(neumann));
    xN = zeros(dim,1);
    xN(freeNeumann) = x(1:nN);
    xD = x((1:nD) + nN);
    
    %
    %--- stop time-measurement for uniform mesh-refinement
    %
    time_ = cputime - time_;
    
    %
    %--- memory consumption
    %
    % memory of N_fine + V_fine + K_fine + M_fine in byte
    memory_ = 8*(size(triangles,1)^2 + size(elements,1)^2 + ...
        2*size(elements,1)*size(coordinates,1));
    
    %*** free obsolete memory
    clear V K W b1 b2
    
    %*** refine mesh uniformly
    [coordinates_fine,dirichlet_fine,neumann_fine,...
        father2dirichlet,father2neumann] ...
        = refineBoundaryMesh(coordinates,dirichlet,neumann);
    
    %*** rearrange indices such that Neumann nodes are first
    [coordinates_fine,neumann_fine,dirichlet_fine] ...
        = buildSortedMesh(coordinates_fine,neumann_fine,dirichlet_fine);
    
    %*** discretize data and compute corresponding data oscillations for
    %*** fine mesh
    [oscD_fine,oscN_fine,uDh_fine,phiNh_fine] ...
        = computeOscMixed(coordinates_fine,dirichlet_fine,neumann_fine, ...
        father2neumann,neumann,uDh,uD,phiN);
    
    %*** compute integral operators for fine mesh
    elements_fine = [dirichlet_fine;neumann_fine];
    V_fine = buildV(coordinates_fine,elements_fine);
    K_fine = buildK(coordinates_fine,elements_fine);
    W_fine = buildW(coordinates_fine,elements_fine);
    
    %*** compute degrees of freedom for fine mesh
    nC_fine = size(coordinates_fine,1);
    nD_fine = size(dirichlet_fine,1);
    freeNeumann_fine = setdiff(1:nC_fine,unique(dirichlet_fine));
    freeDirichlet_fine = 1:nD_fine;
    nodesNeumann_fine = unique(neumann_fine);
    nN_fine = length(freeNeumann_fine);
    
    % compute right-hand side for fine mesh and shrink it
    [b1_fine,b2_fine,Nell_fine] = buildMixedVolRHS(coordinates_fine, ...
        dirichlet_fine,neumann_fine,uDh_fine,phiNh_fine, ...
        vertices,triangles,fh,V_fine,K_fine,W_fine);
    b1_fine = b1_fine(freeNeumann_fine);
    b2_fine = b2_fine(freeDirichlet_fine);
    
    %*** shrink integral operators
    V_fine = V_fine(freeDirichlet_fine,freeDirichlet_fine);
    K_fine = K_fine(freeDirichlet_fine,freeNeumann_fine);
    W_fine = W_fine(nodesNeumann_fine,nodesNeumann_fine);
    
    %*** compute Galerkin solution for fine mesh
    x = [ W_fine(freeNeumann_fine,freeNeumann_fine) K_fine' ; ...
        -K_fine V_fine ] ...
        \ [ b1_fine ; b2_fine ];
    
    %*** compute coefficient vectors w.r.t. S1(GammaN) and P0(GammaD)
    dim_fine = length(unique(neumann_fine));
    xN_fine = zeros(dim_fine,1);
    xN_fine(freeNeumann_fine) = x(1:nN_fine);
    xD_fine = x((1:nD_fine) + nN_fine);
    
    %*** compute all available error estimators and error bounds
    etaD = computeEstSlpEta(father2dirichlet,V_fine,xD_fine,xD);
    etaN = computeEstHypEta(neumann_fine,neumann,father2neumann,W_fine, ...
        xN_fine,xN);
    etaD_tilde = computeEstSlpEtaTilde(father2dirichlet,V_fine,xD_fine);
    etaN_tilde = computeEstHypEtaTilde(neumann_fine,neumann,father2neumann, ...
        W_fine,xN_fine);
    muD = computeEstSlpMu(coordinates,dirichlet,father2dirichlet,xD_fine,xD);
    muN = computeEstHypMu(neumann_fine,neumann,father2neumann,xN_fine,xN);
    muD_tilde = computeEstSlpMuTilde(coordinates,dirichlet,father2dirichlet,...
        xD_fine);
    muN_tilde = computeEstHypMuTilde(neumann_fine,neumann,father2neumann, ...
        xN_fine);
    errD = computeErrNeumann(coordinates,dirichlet,xD,phiN);
    xNg = uDh;
    xNg(freeNeumann) = xNg(freeNeumann) + x(1:nN);
    errN = computeErrDirichlet(coordinates,neumann,xNg,uD);
    mu_V_tilde_D = computeEstSlpMuTilde(coordinates,dirichlet, ...
        father2dirichlet,Nell_fine(1:nD_fine));
    mu_V_tilde_N = computeEstSlpMuTilde(coordinates,neumann, ...
        father2neumann,Nell_fine(nD_fine+1:end));
    
    %*** compute Neumann data on Gamma:
    %*** on Gamma_N, use phiNh,
    %*** on GammaD, use solution xD on Gamma_D
    xDn = zeros(size(elements,1),1);
    xDn = phiNh;
    xDn(1:nD) = xDn(1:nD) + xD;
    
    %*** free obsolete memory
    clear coordinates_fine elements_fine dirichlet_fine neumann_fine
    clear father2son father2dirichlet father2neumann
    clear V_fine K_fine W_fine b1_fine b2_fine x_fine x
    
    %*** store computed data
    fprintf('*** problem size (uniform) = %d (Gamma, %d (Omega)\n', ...
        size(dirichlet,1)+size(neumann,1),size(triangles,1));
    nE(counter) = size(elements,1);
    nV(counter) = size(triangles,1);
    time(counter) = time_;
    memory(counter) = memory_;
    eta(counter) = sqrt(etaD + etaN);
    eta_tilde(counter) = sqrt(etaD_tilde + etaN_tilde);
    mu(counter) = sqrt(sum([muD; muN]));
    mu_tilde(counter) = sqrt(sum([muD_tilde; muN_tilde]));
    mu_V_tilde(counter) = sqrt(sum([mu_V_tilde_D; mu_V_tilde_N]));
    oscGamma(counter) = sqrt(sum([oscD; oscN]));
    oscOmega(counter) = sqrt(sum(oscV));
    err(counter) = sqrt(sum([errD; errN]));
    
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(3,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,mu_V_tilde,'bv-')
        loglog(nE,oscGamma,'<k-')
        loglog(nE,oscOmega,'sk-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh boundary elements')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
            'tilde-mu_V (unif)','oscD (unif)','oscV (unif)','error (unif)',3)
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(3,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,mu_V_tilde,'bv-')
        loglog(time,oscGamma,'<k-')
        loglog(time,oscOmega,'sk-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
            'tilde-mu_V (unif)','oscD (unif)','oscV (unif)','error (unif)',3)
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow
        
        subplot(3,2,3)
        plotArclengthS1(coordinates,[dirichlet;neumann],xNg,uD,-1);
        title('Dirichlet Data');
        subplot(3,2,5)
        plotArclengthP0(coordinates,[dirichlet;neumann],xDn,phiN,-1);
        title('Neumann Data');
        drawnow
        
        subplot(3,2,[4 6])
        trisurf(triangles,vertices(:,1),vertices(:,2),f(vertices),...
            'facecolor','interp')
        title('Volume Data');
        axis equal
        view(2)
        drawnow
    end
    
    counter = counter+1;
end

