function [nE,nV,time,memory,eta,eta_tilde,mu,mu_tilde,oscGamma,oscOmega,err] ...
    = exampleUnifSymmVol(vertices0,triangles0,uD,f,nVmax,varargin)

% exampleUnifSymmVol provides the experimental data for the numerical solution
% of the equivalent integral formulation of a Dirichlet boundary value problem
% with non-homogeneous volume forces and uniform mesh-refinement. 
%
% Usage: [N,NV,TIME,MEMORY,ETA,ETA_TILDE,MU,MU_TILDE,OSCD,OSCV [,ERR]] ...
%            = exampleUnifSymmVol(VERTICES,VOLUMES,UD,F,MMAX [,PHI] [,FIG])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let an initial triangulation of the domain T0 be given.
%              For the data f on Omega and uD on
%              Gamma we numerically solve Symm's integral equation
%
%                V*phi = (K+1/2)*uD - N*f on Gamma=boundary(Omega)
%
%              for some Lipschitz domain Omega. To that end, we realize
%              the following algorithm:
%              (1) refine the initial mesh T0 k-times uniformly
%                  (1.1) let Ek be the boundary mesh induced by Tk
%              (2) compute Galerkin matrix and right-hand side
%              (3) compute Galerkin solution
%
%              Furthermore the (h-h/2)-based error estimators eta, eta-tilde,
%              mu, mu_tilde as well as data oscillations are computed. Assuming
%              the solution phiN is given we additionally compute a reliable
%              error bound for the approximation error on Gamma.
%              If a figure number is specified, all error quantities are
%              plotted on the fly during the computation over the number of
%              degrees of freedom and the computation time.
%              The latter considers precisely the time
%              necessary to set up the mesh and solve the coarse system, i.e.
%              the computational overhead caused by calculation of eta,
%              eta-tilde, mu, mu-tilde, and the reliable error bounds is not
%              included in the time measurement. Moreover the time measurement
%              is not cumulative.
%              As a second important parameter we compute the memory
%              consumption of all discrete integral operators which are built
%              in this function or any functions called by it.
%
%              VERTICES gives the coordinates for all
%              nodes of the triangulation of the domain. VOLUMES specifies the
%              triangular volume elements. 
%              The function handle UD allows to evaluate the given Dirichlet
%              data. For an (M x 2)-matrix of nodes X, UD returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := uD(X(j,:)). The function handle F allows to
%              evaluate the volume forces in Omega. For an (M x 2)-matrix of
%              nodes X, F returns the (M x 1)-vector of functiona values, i.e.
%              Y(j) := f(X(j,:)). The algorithm terminates, when the coarse
%              volume mesh has more than MMAX elements.
%              THETA is the adaptivity parameter used for Doerfler marking. If
%              the exact solution of Symm's integral equations is known, it
%              can be provided by means of the optional function handle PHI.
%              For an (M x 2)-matrix of nodes X and the respective boundary
%              elements given by the (M x 2) matrices A and B, PHI returns the
%              (M x 1)-vector Y of corresponding function values,
%              i.e. Y(j) := PHI(X(j,:)). The optional parameter FIG gives the
%              figure number where a convergence graph is plotted on the fly.
%
%              All output parameters are vectors containing the respective
%              values corresponding to the number of uniform refinements
%              performed from the initial meshes. N contains the number of
%              coarse boundary mesh elements. NV contains the number of 
%              coarse volume mesh elements. TIME contains the computation
%              time necessary to compute the coarse mesh solution. MEMORY
%              contains the memory consumption in [byte]. ETA,
%              ETA_TILDE, MU, and MU_TILDE contain the global error estimator
%              quantities computed. OSCD contains the global data oscillations
%              w.r.t the Dirichlet data. OSCV contains the global data
%              oscillations w.r.t the volume forces. The optional output
%              parameter ERR contains a reliabel upper bound for the error,
%              provided that the solution PHI was given as input parameter.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** Check whether optional figure number is given
if nargin > 5 && isnumeric(varargin{1})
    figure_number = varargin{1};
elseif nargin > 6 && isnumeric(varargin{2})
    figure_number = varargin{2};
else
    figure_number = false;
end

%*** Check whether optional function phi is given
if nargin > 5 && isa(varargin{1},'function_handle')
    phi_N = varargin{1};
elseif nargin > 6 && isa(varargin{2},'function_handle')
    phi_N = varargin{2};
else
    phi_N = false;
end

%*** extract boundary mesh
[vertices0,triangles0,coordinates0,elements0] = ...
    buildSortedMesh(vertices0,triangles0);

counter = 1;
triangles = triangles0;

while size(triangles,1) < nVmax
    %
    %--- start time-measurement for uniform mesh-refinement
    %
    time_ = cputime;
    
    %*** build mesh by successive uniform refinements of initial mesh
    vertices = vertices0;
    triangles = triangles0;
    elements = elements0;
    for j = 1:counter-1
        [vertices,triangles,elements] = refineMesh(vertices,triangles,elements);
    end

    %*** sort boundary nodes first
    [vertices,triangles,coordinates,elements] = ...
        buildSortedMesh(vertices,triangles);
    
    fprintf('*** problem size (uniform) = %d (Gamma), %d (Omega)\n', ...
        size(elements,1),size(triangles,1));
    
    %*** compute coarse-mesh solution
    V = buildV(coordinates,elements);

    %*** compute Dirichlet- and volume data and data oscillation
    [oscD,uDh] = computeOscDirichlet(coordinates,elements,uD);
    [oscV,fh] = computeOscVolume(vertices,triangles,f);

    b = buildSymmVolRHS(coordinates,elements,uDh,vertices,triangles,fh);
    x = V\b;
    
    %
    %--- stop time-measurement for uniform mesh-refinement
    %
    time_ = cputime - time_;

    %
    %--- memory consumption
    %
    % memory of N_fine + V_fine + K_fine + M_fine in byte
    memory_ = 8*(size(triangles,1)^2 + size(elements,1)^2 + ...
        2*size(elements,1)*size(coordinates,1));

    %*** free obsolete memory
    clear V N b s
    
    %*** compute fine-mesh solution to estimate error
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    V_fine = buildV(coordinates_fine,elements_fine);

    %*** compute fine dirichlet data and oscillation
    [oscD_fine,uDh_fine] = computeOscDirichlet(coordinates_fine,elements_fine,uD);

    b_fine = buildSymmVolRHS(coordinates_fine,elements_fine,uDh_fine,...
        vertices,triangles,fh);
    x_fine = V_fine\b_fine;
    
    %*** compute (h-h/2)-type estimators
    eta_ = computeEstSlpEta(father2son,V_fine,x_fine,x);
    eta_tilde_ = computeEstSlpEtaTilde(father2son,V_fine,x_fine);
    mu_ = computeEstSlpMu(coordinates,elements,father2son,x_fine,x);
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);
    
    %*** compute error
    if isa(phi_N,'function_handle')
	err_ = computeErrNeumann(coordinates_fine,elements_fine,x_fine,phi_N);
	err(counter) = sqrt(sum(err_));
    else
        err(counter) = 0;
    end

    %*** free obsolete memory
    clear coordinates_fine elements_fine father2son
    clear V_fine b_fine x_fine

    %*** store computed data
    nE(counter) = size(elements,1);
    nV(counter) = size(triangles,1);
    time(counter) = time_;
    memory(counter) = memory_;
    eta(counter) = sqrt(eta_);
    eta_tilde(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_));
    mu_tilde(counter) = sqrt(sum(mu_tilde_));
    oscGamma(counter) = sqrt(sum(oscD));
    oscOmega(counter) = sqrt(sum(oscV));
       
    %*** plot results
    if figure_number
        figure(figure_number)
        clf
        
        %*** plot estimators over degrees of freedom
        subplot(2,2,1)
        loglog(nE,eta,'ro-')
        hold on
        loglog(nE,eta_tilde,'rx-')
        loglog(nE,mu,'bo-')
        loglog(nE,mu_tilde,'bx-')
        loglog(nE,oscGamma,'<k-')
        loglog(nE,oscOmega,'sk-')
        loglog(nE,err,'pm-')
        
        title('estimator vs number of coarse-mesh boundary elements')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'oscD (unif)','oscV (unif)','error (unif)',3)
        xlabel('number of coarse-mesh elements')
        ylabel('estimators')
        drawnow
        
        %*** plot estimators over computational time
        subplot(2,2,2)
        loglog(time,eta,'ro-')
        hold on
        loglog(time,eta_tilde,'rx-')
        loglog(time,mu,'bo-')
        loglog(time,mu_tilde,'bx-')
        loglog(time,oscGamma,'<k-')
        loglog(time,oscOmega,'sk-')
        loglog(time,err,'pm-')
        
        title('estimator vs computational time')
        legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
               'oscD (unif)','oscV (unif)','error (unif)',3)
        xlabel('computational time [s]')
        ylabel('estimators')
        drawnow

        subplot(2,2,3)
        plotArclengthP0(coordinates,elements,x,phi_N,-1);
        drawnow
        
        subplot(2,2,4)
        trisurf(triangles,vertices(:,1),vertices(:,2),f(vertices),'facecolor','interp')
        axis equal
        view(2)
        drawnow
    end  
   
    %*** increase counter and iterate
    counter = counter + 1;
end

