function exampleHypsingVol(nVmax)

% exampleHypsingVol provides an experimental comparison between uniform and
% adaptive mesh-refinement for the hypersingular integral equation with 
% non-vanishing volume force.
%
% Usage: exampleHypsingVol(NVMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              We consider the hypersingular integral equation with volume
%              force
%
%               W*uD = (1/2 - K')*phi - N_1f
%
%              on the boundary Gamma of the turned L-shape, which is equivalent
%              to the Laplace problem
% 
%                 -Laplace(u) = f in Omega with du/dn = phi on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                 u(x) = ( (x-a)^2 + (y-b)^2 )^beta + r^{2/3} * cos(2theta/3)
%
%              for fixed parameters (a,b) = (.14 .14) and beta = 0.9.
%              Then, uD is the trace of u on Gamma. It is easily
%              seen that uD is smooth, whereas the volume data f and the 
%              Neumann date phi are singular.
%              Moreover, N_1f is singular at the origin.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../general/');
addpath('../../../lib');

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** extract boundary mesh
[vertices,triangles,coordinates,elements] = buildSortedMesh(vertices,triangles);
nE = size(elements,1);

%*** visualize Gamma and initial triangulation
figure(1)
clf
for j = 1:nE
    a = coordinates(elements(j,1),:);
    b = coordinates(elements(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-k','MarkerFaceColor','r')
    hold on
end
axis equal
title('initial mesh with N=8 elements')

%*** compute results for adaptive mesh-refinement
[nE_adap,nV_adap,time_adap,memory_adap,eta_adap,eta_tilde_adap,mu_adap, ...
    mu_tilde_adap,mu_V_tilde_adap,oscN_adap,oscV_adap,error_adap] ...
    = exampleAdapHypsingVol(vertices,triangles,@phi,@f,nVmax,0.25,@g,2);

%*** compute results for uniform mesh-refinement
[nE_unif,nV_unif,time_unif,memory_unif,eta_unif,eta_tilde_unif,mu_unif, ...
    mu_tilde_unif,mu_V_tilde_unif,oscN_unif,oscV_unif,error_unif] ...
    = exampleUnifHypsingVol(vertices,triangles,@phi,@f,nVmax,@g,2);

%*** plot results
figure(2)
clf

%*** plot estimators over coarse-mesh elements
subplot(1,3,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,mu_V_tilde_unif,'r^-')
loglog(nE_unif,oscN_unif,'m*-')
loglog(nE_unif,oscV_unif,'ms-')
loglog(nE_unif,error_unif,'m^-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bs-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,mu_V_tilde_adap,'b^-')
loglog(nE_adap,oscN_adap,'c*-')
loglog(nE_adap,oscV_adap,'cs-')
loglog(nE_adap,error_adap,'c^-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,N.^(-3/2),'k--')
N = [nE_unif,2*max(nE_unif)];
loglog(N,N.^(-2/3),'k--')

title('Neumann problem on symmetrically rotated L-shape (exampleHypsingVol)')
xlabel('number of coarse-mesh elements')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
    'mu-V-tilde (unif)','oscN (unif)','oscV (unif)','error (unif)',...
    'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
    'mu-V-tilde (adap)','oscN (adap)','oscV (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,3,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,mu_V_tilde_unif,'r^-')
loglog(time_unif,oscN_unif,'m*-')
loglog(time_unif,oscV_unif,'ms-')
loglog(time_unif,error_unif,'m^-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,mu_V_tilde_adap,'b^-')
loglog(time_adap,oscN_adap,'c*-')
loglog(time_adap,oscV_adap,'cs-')
loglog(time_adap,error_adap,'c^-')

xlabel('computational time [s]')
ylabel('estimators')      

%*** plot estimators over computational time
subplot(1,3,3)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,mu_V_tilde_unif,'r^-')
loglog(time_unif,oscN_unif,'m*-')
loglog(time_unif,oscV_unif,'ms-')
loglog(time_unif,error_unif,'m^-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,mu_V_tilde_adap,'b^-')
loglog(time_adap,oscN_adap,'c*-')
loglog(time_adap,oscV_adap,'cs-')
loglog(time_adap,error_adap,'c^-')

xlabel('memory consumption in [MB]')
ylabel('estimators')      
