function exampleMixedVol(nVmax)

% exampleMixedVol provides an experimental comparison between uniform and
% adaptive mesh-refinement for the integral formulation of a mixed boundary
% value problem with non-vanishing volume force.
%
% Usage: exampleMixedVol(NVMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
% 
%              We consider the mixed problem with volume force
%
%                -K*uN + V*phiD  = 1/2 + K*g - V*phi - N0*f
%                 W*uN + K'*phiD = 1/2 - W*g - K'*phi - N1*f
%
%              on the boundary Gamma of the turned L-shape, which is equivalent
%              to the Laplace problem
% 
%                 -Laplace(u) = f    in Omega
%                           u = g    on GammaD
%                       du/dn = phi  on GammaN
%
%              We prescribe the exact solution in polar coordinates
%
%                 u(x) = ( (x-a)^2 + (y-b)^2 )^beta + r^{2/3} * cos(2theta/3)
%
%              for fixed parameters (a,b) = (.14 .14) and alpha = 0.9.
%              Then, phi is the normal derivative of u on Gamma and 
%              g is its trace. It is easily
%              seen that phi and g are smooth, whereas the volume data f are
%              singular at the point (a,b). Moreover, uN is the trace of
%              u on GammaN and phiD is the normal derivative of u on
%              GammaD. uN is smooth whereas phiD is non-smooth.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath('../general')
addpath('../../../lib')

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** split Gamma into Dirichlet and Neumann boundary
dirichlet = [6 5; 5 8];
neumann = [1 2;2 3;3 6;8 7;7 4;4 1];

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** visualize Gamma and initial partition
figure(1)
clf
for j = 1:size(dirichlet,1)
    a = vertices(dirichlet(j,1),:);
    b = vertices(dirichlet(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-b','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
for j = 1:size(neumann,1)
    a = vertices(neumann(j,1),:);
    b = vertices(neumann(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-r','MarkerFaceColor','k', ...
      'MarkerEdgeColor','k')
    hold on
end
axis equal
title('initial mesh with N=8 elements (GammaD = blue, GammaN = red)')

[nE_adap,nV_adap,time_adap,memory_adap,eta_adap,eta_tilde_adap,mu_adap, ...
    mu_tilde_adap,mu1_tilde_adap,oscG_adap,oscV_adap,err_adap] = ... 
    exampleAdapMixedVol(vertices,triangles,dirichlet,neumann,@g,@phi, ...
                        @f,nVmax,0.25,2);

[nE_unif,nV_unif,time_unif,memory_unif,eta_unif,eta_tilde_unif,mu_unif, ...
    mu_tilde_unif,mu1_tilde_unif,oscG_unif,oscV_unif,err_unif] = ... 
    exampleUnifMixedVol(vertices,triangles,dirichlet,neumann,@g,@phi, ...
                        @f,nVmax,2);

%*** plot estimators over coarse-mesh elements
subplot(1,3,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,mu1_tilde_unif,'rd-')
loglog(nE_unif,oscG_unif,'m*-')
loglog(nE_unif,oscV_unif,'ms-')
loglog(nE_unif,err_unif,'m^-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bx-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,mu1_tilde_adap,'bd-')
loglog(nE_adap,oscG_adap,'c*-')
loglog(nE_adap,oscV_adap,'cs-')
loglog(nE_adap,err_adap,'c^-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,N.^(-3/2),'k--')

title('Mixed BVP on symmetrically rotated L-shape (exampleMixedVol)')
xlabel('number of coarse-mesh elements (boundary)')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
    'tilde-mu-V (unif)','oscD (unif)','oscV (unif)','error (unif)',...
    'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
    'tilde-mu-V (adap)','oscD (adap)','oscV (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,3,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,mu1_tilde_unif,'rd-')
loglog(time_unif,oscG_unif,'m*-')
loglog(time_unif,oscV_unif,'ms-')
loglog(time_unif,err_unif,'m^-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,mu1_tilde_adap,'bd-')
loglog(time_adap,oscG_adap,'c*-')
loglog(time_adap,oscV_adap,'cs-')
loglog(time_adap,err_adap,'c^-')

xlabel('computational time [s]')
ylabel('estimators')      

% plot estimators over memory consumption
subplot(1,3,3)
loglog(memory_unif/1048576,eta_unif,'rx-'); hold on
loglog(memory_unif/1048576,eta_tilde_unif,'ro-')
loglog(memory_unif/1048576,mu_unif,'rs-')
loglog(memory_unif/1048576,mu_tilde_unif,'r.-')
loglog(memory_unif/1048576,mu1_tilde_unif,'rd-')
loglog(memory_unif/1048576,oscG_unif,'m*-')
loglog(memory_unif/1048576,oscV_unif,'ms-')
loglog(memory_unif/1048576,err_unif,'m^-')

loglog(memory_adap/1048576,eta_adap,'bx-')
loglog(memory_adap/1048576,eta_tilde_adap,'bo-')
loglog(memory_adap/1048576,mu_adap,'bs-')
loglog(memory_adap/1048576,mu_tilde_adap,'b.-')
loglog(memory_adap/1048576,mu1_tilde_adap,'bd-')
loglog(memory_adap/1048576,oscG_adap,'c*-')
loglog(memory_adap/1048576,oscV_adap,'cs-')
loglog(memory_adap/1048576,err_adap,'c^-')

xlabel('memory consumption [MB]')
ylabel('estimators')      
