function exampleSymmVol(nVmax)

% exampleSymmVol provides an experimental comparison between uniform and 
% adaptive mesh-refinement for Symm's integral equation on non-vanishing 
% volume force.
%
% Usage: exampleSymmVol(NVMAX)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              We consider Symm's integral equation with volume force
%
%                 V*phi = (K+1/2)*g - Nf
%
%              on the boundary Gamma of the turned L-shape, which is equivalent
%              to the Laplace problem
% 
%                 -Laplace(u) = f in Omega with u = g on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = -1/4(x^2 + y^2) + r^{2/3} * cos(2theta/3)
%
%              where (r,theta) are polar coordinates and (x,y) are
%              cartesian coordinates.
%              By choice of u it follows that f is a polynomial
%              of degree 0.
%              Then, phi is the normal derivative of u on Gamma,
%              and has a singularity at the origin.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

% add working library
addpath('../general/')
addpath('../../../lib');

vertices = load('coordinates.dat')/4;
triangles = load('triangles.dat');

%*** rotate domain Omega so that exact solution is symmetric
alpha = 3*pi/4;
vertices = vertices*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]';

%*** extract boundary mesh
[vertices,triangles,coordinates,elements] = buildSortedMesh(vertices,triangles);

nE = size(elements,1);

%*** visualize Gamma and initial triangulation
figure(1)
clf
for j = 1:nE
    a = coordinates(elements(j,1),:);
    b = coordinates(elements(j,2),:);
    plot([a(1) b(1)],[a(2),b(2)],'o-k','MarkerFaceColor','r')
    hold on
end
axis equal
title('initial mesh with N=8 elements')

%*** compute results for adaptive mesh-refinement
[nE_adap,nV_adap,time_adap,memory_adap,eta_adap,eta_tilde_adap,mu_adap,mu_tilde_adap, ...
    oscD_adap,oscV_adap,error_adap] ...
    = exampleAdapSymmVol(vertices,triangles,@g,@f,nVmax,0.25, @phi,2);

%*** compute results for uniform mesh-refinement
[nE_unif,nV_unif,time_unif,memory_unif,eta_unif,eta_tilde_unif,mu_unif,mu_tilde_unif, ...
    oscD_unif,oscV_unif,error_unif] ...
    = exampleUnifSymmVol(vertices,triangles,@g,@f,nVmax, @phi,2);

%*** plot results
figure(2)
clf

%*** plot estimators over coarse-mesh elements
subplot(1,3,1)
loglog(nE_unif,eta_unif,'rx-'); hold on
loglog(nE_unif,eta_tilde_unif,'ro-')
loglog(nE_unif,mu_unif,'rs-')
loglog(nE_unif,mu_tilde_unif,'r.-')
loglog(nE_unif,oscD_unif,'m*-')
loglog(nE_unif,oscV_unif,'ms-')
loglog(nE_unif,error_unif,'m^-')
 
loglog(nE_adap,eta_adap,'bx-')
loglog(nE_adap,eta_tilde_adap,'bo-')
loglog(nE_adap,mu_adap,'bx-')
loglog(nE_adap,mu_tilde_adap,'b.-')
loglog(nE_adap,oscD_adap,'c*-')
loglog(nE_adap,oscV_adap,'cs-')
loglog(nE_adap,error_adap,'c^-')

N = [nE_adap,2*max(nE_adap)];
loglog(N,N.^(-3/2),'k--')

title('Dirichlet problem on symmetrically rotated L-shape (exampleSymmVol)')
xlabel('number of coarse-mesh elements (boundary)')
ylabel('estimators')
legend('eta (unif)','tilde-eta (unif)','mu (unif)','tilde-mu (unif)',...
    'oscD (unif)','oscV (unif)','error (unif)',...
    'eta (adap)','tilde-eta (adap)','mu (adap)','tilde-mu (adap)',...
    'oscD (adap)','oscV (adap)','error (adap)')

%*** plot estimators over computational time
subplot(1,3,2)
loglog(time_unif,eta_unif,'rx-'); hold on
loglog(time_unif,eta_tilde_unif,'ro-')
loglog(time_unif,mu_unif,'rs-')
loglog(time_unif,mu_tilde_unif,'r.-')
loglog(time_unif,oscD_unif,'m*-')
loglog(time_unif,oscV_unif,'ms-')
loglog(time_unif,error_unif,'m^-')

loglog(time_adap,eta_adap,'bx-')
loglog(time_adap,eta_tilde_adap,'bo-')
loglog(time_adap,mu_adap,'bs-')
loglog(time_adap,mu_tilde_adap,'b.-')
loglog(time_adap,oscD_adap,'c*-')
loglog(time_adap,oscV_adap,'cs-')
loglog(time_adap,error_adap,'c^-')

xlabel('computational time [s]')
ylabel('estimators')      

% plot estimators over memory consumption
subplot(1,3,3)
loglog(memory_unif/1048576,eta_unif,'rx-'); hold on
loglog(memory_unif/1048576,eta_tilde_unif,'ro-')
loglog(memory_unif/1048576,mu_unif,'rs-')
loglog(memory_unif/1048576,mu_tilde_unif,'r.-')
loglog(memory_unif/1048576,oscD_unif,'m*-')
loglog(memory_unif/1048576,oscV_unif,'ms-')
loglog(memory_unif/1048576,error_unif,'m^-')

loglog(memory_adap/1048576,eta_adap,'bx-')
loglog(memory_adap/1048576,eta_tilde_adap,'bo-')
loglog(memory_adap/1048576,mu_adap,'bs-')
loglog(memory_adap/1048576,mu_tilde_adap,'b.-')
loglog(memory_adap/1048576,oscD_adap,'c*-')
loglog(memory_adap/1048576,oscV_adap,'cs-')
loglog(memory_adap/1048576,error_adap,'c^-')

xlabel('memory consumption [MB]')
ylabel('estimators')
