function phix = phi(x,a,b)

% phi provides the Neumann data for some Poisson problem realized 
% in examples/poisson/zshape1/
%
% Usage: Y = phi(X,A,B)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/zshape1, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega with du/dn = phi on Gamma (or GammaN)
%
%              We prescribe the exact solution in polar coordinates
%
%                u(x) = |x-z|^(2*alpha).
%
%              where alpha = 0.51 and z = [0.1 0.1].
%
%              The evaluation points are given by the (N x 2)-matrix X. The
%              (N x 2)-matrices A and B give the nodes of the corresponding
%              boundary elements, i.e., X(j,:) belongs to the line segment 
%              Ej = conv{ A(j,:) , B(j,:) }. This information is used to 
%              compute the outer unit normal vector on Ej. The 
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = phi(X(j,:)). 
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution is u(x) = |x-z|^(2*alpha) = [ (x1-z1)^2 + (x2-z2)^2 ]^alpha
[alpha,z] = parameters();

%*** first-order derivatives du/dx1 and du/dx2
x = x - repmat(z,length(x),1);
d1u = 2*alpha*x(:,1).*sum(x.^2,2).^(alpha-1);
d2u = 2*alpha*x(:,2).*sum(x.^2,2).^(alpha-1);

%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** evaluate normal derivative phi
phix = sum([d1u d2u].*n,2);
