function make(varargin)

%make builds HILBERT's operators.
%
% Available targets and their description:
%     help:            shows this list of available targets.
%     all:             builds all MEX functions that belong to HILBERT.
%     clean:           deletes all files that are created during the building
%                      of HILBERT's MEX functions.
%     realclean:       like clean, but also deletes the configuration file.
%     configure:       interactivly creates a configuration file.
%     configure-gui:   runs the graphical dialogs to interactivly create a
%                      configuration file.
%     configure-text:  runs the text-based configuration assistant to
%                      interactivly create a configuration file.
%
% Furthermore there is a target named function_name for each of HILBERT's
% MEX functions.
%
% make is a general purpose make tool. The actual implementation of the
% targets is located in the directory source/make/Makefile.m. Further helper
% functions are also located in this directory and its sub directories.
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%    addpath source/make;
%    addpath source/make/lib;

    if (nargin == 0)
        make all;
        return;
    elseif (nargin > 1)
        for argin=varargin
            make(argin);
        end
        return;
    end
    
    target_name = varargin{1};

    %*** Load an alternative Makefile, if a function handle is provided.
    if (strcmp(class(target_name), 'function_handle') == true)
      getTargetByName(target_name);
      return;
    end
    
    %*** This code recursivly builds the target named target_name.
    t = getTargetByName(target_name);
    if (isstruct(t) == false) % t is a file-target
        tf = fileAvailable(target_name);
        if (tf == false)
            error(['Target ' target_name ...
                                ' does not exist and is not a file.']);
        end
    elseif (t.available_handler(t) == false)
        for depend=t.dependencies
            dependstr = char(depend);
            depend_target = getTargetByName(dependstr);
            if (isstruct(depend_target) == false)
                if (fileAvailable(dependstr) ~= false)
                    continue;
                end
                error(['Target ' dependstr ...
                                ' does not exist and is not a file.']);
            end

            if (depend_target.available_handler(depend_target) == false)
                make(dependstr);
            end
        end

        try
            build_status = t.build_handler(t);
        catch e
            disp('Uncaught build handler exception:');
            disp(e.message);
            disp('This is probably a severe and unintended error.');
            disp('Please run "make realclean" to clean up');
            error('Uncaught build handler exception!');
        end

 
        if (build_status == false)
            error(['Failed when building ' t.name]);
        end
    else
        disp(['The target ' target_name ' is already up-to-date.']);
    end
    
    disp(['Built target ' target_name ' successfully.']);

    return;
end

