function targets = Makefile()

%Makefile returns a list of targets that is processed by make.m and its
%helpers.
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed boundary
%              conditions by use of BEM in 2D.
%
%             This file is part of the building system for HILBERT. HILBERT's
%             building system is a recursive, dependency-based system similar
%             to "make".
%
%             This file contains a list of targets and their dependencies
%             between each other. It also contains some functions that
%             contain the code that is responsible for building certain
%             targets.
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

addpath source/make/helper;
addpath source/make/dialogs;
addpath source/make/config;

createEmptyConfigure();

targets = [ ...
  ... Essential files:
  addObjectFiles('buildN', 'buildK', 'buildV', 'buildW', 'constants', ...
      'evaluateK', 'evaluateKadj', 'evaluateV', 'evaluateW', 'evaluateN', ...
      'doubleLayerPotential', 'gaussQuadrature', 'geometry', ...
      'newtonPotential', 'singleLayerPotential', ...
      'threadedK', 'threadedN', 'threadedV', 'threadedW', 'threads'), ...
  ... Target for the operators:
  addThrMexTarget('buildN', {'buildN.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'singleLayerPotential.o', 'newtonPotential.o', 'threads.o', ...
      'threadedN.o'}), ...
  addThrMexTarget('buildK', {'buildK.o', 'doubleLayerPotential.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'threads.o', 'threadedK.o'}), ...
  addThrMexTarget('buildV', {'buildV.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'singleLayerPotential.o', 'threads.o', 'threadedV.o'}), ...
  addThrMexTarget('buildW', {'buildW.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'singleLayerPotential.o', 'threads.o', 'threadedW.o'}), ...
  addMexTarget('evaluateK', {'evaluateK.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'doubleLayerPotential.o'}), ...
  addMexTarget('evaluateKadj', {'evaluateKadj.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'doubleLayerPotential.o'}), ...
  addMexTarget('evaluateV', {'evaluateV.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'singleLayerPotential.o'}), ...
  addMexTarget('evaluateW', {'evaluateW.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'doubleLayerPotential.o'}), ...
  addMexTarget('evaluateN', {'evaluateN.o', ...
      'constants.o', 'gaussQuadrature.o', 'geometry.o', ...
      'newtonPotential.o', 'singleLayerPotential.o'}), ...
  ... Meta targets:
  createTarget('all', {'buildN','buildK','buildV','buildW', ...
        'evaluateK', 'evaluateKadj', 'evaluateV', 'evaluateW', 'evaluateN'},...
      @installWindowsDlls, @neverAvailable), ...
  ... Cleaning up
  createTarget('clean', {}, @clean, @neverAvailable), ...
  createTarget('clear', {'clean'}, @nullAction, @neverAvailable), ...
  createTarget('realclean', {'clean'}, @realclean, @neverAvailable), ...
  ... Configuration
  createTarget('configure-gui',{},@configureGui,@neverAvailable),...
  createTarget('configure-text',{},@configureText,@neverAvailable),...
  createTarget('configure',{},@configure,@neverAvailable), ...
  createTarget('clean-build',{'clean', 'all'},@nullAction,@neverAvailable),...
  ... Windows installation target
  createTarget('win-install',{},@installPrecompiled,@neverAvailable),...
  ... Help:
  createTarget('help', {}, @help, @neverAvailable) ...
];

%*** Deletes the configuration file.
function status = realclean(target)
status = true;

try
  delete source/make/Configure.m
  createEmptyConfigure();
catch
  status = false;
end

%*** Deletes all object-files and MEX-files.
function status = clean(target)
try
  clear all;
  status = true;
  delete source/*.o
  delete source/*.obj
  delete(['lib/*.' mexext()]);
  delete lib/*.dll
catch
  status = false;
end

%*** Displays the help message.
function status = help(target)
disp(sprintf([ ...
'*** List of available targets:\n' ...
'\thelp:    shows this list of available targets.\n' ...
'\tall:     compiles all MEX functions that belong to HILBERT.\n' ...
'\tclean:   deletes all files that are created during the building of\n'... 
'\t         the HILBERT MEX files.\n' ...
'\trealclean: like clean, but also deletes the configuration file.\n' ...
'\tconfigure: interactivly creates a Configure.m file.\n' ...
'\tconfigure-gui: runs the graphical configuration dialog.\n' ...
'\tconfigure-text: runs the text-based configuration dialog.\n' ...
'\n' ...
'Furthermore for each function in HILBERT there is a target\n' ...
'function_name that builds this function.\n' ...
]));
status = true;

function status = configure(target)
  try
    status = configureGui(target);
  catch
    status = configureText(target);
  end

%*** Provides an interactive text-based configuration dialog.
function status = configureText(target)
  try
    generateConfigure(textConfig());
    status = 1;
  catch
    status = 0;
  end

function status = configureGui(target)
  try
    config = DialogConfig();
    generateConfigure(config);
    clear Configure;
    tf = DialogMakeCleanBuild();
    if (tf)
      make clean-build;
    end
    status = 1;
  catch
    disp lasterr();
    status = 0;
  end

function status = installWindowsDlls(target)
  try
    if (ispc())
      copyfile(['source\make\ext\lib\' mexext '\pthreadVC2.dll'], ...
               'lib\pthreadVC2.dll');
    end
    status = 1;
  catch
    disp lasterr();
    status = 0;
  end
  
function status = installPrecompiled(target)
  config = Configure();
  try
    installWindowsBinaries(config);
    status = 1;
  catch
    e = lasterror();
    disp(e.message);
    status = 0;
  end

