function target = createTarget(name, dependencies, build_handler, varargin)
%createTarget is a helper function for building a target structure.
%
% Usage: createTarget(name, dependencies, build_handler [, available_handler])
%
% Description: This file is part of the HILBERT program package for the
%            numerical solution of the Laplace equation with mixed boundary
%            conditions by use of BEM in 2D.
%
%            This file is part of the building system for HILBERT. HILBERT's
%            building system is a recursive, dependency-based system similar
%            to "make".
%
%            createTarget is a helper function for building a target
%            structure. Each target is represented by a structure that has
%            the fields "name", "dependencies", "build_handler" and
%            "available_handler". "name" is the target's name, "dependencies"
%            is a (cell) array of strings containing the targets' names, this
%            target depends on. "build_handler" is a function handle for the
%            function that contains the code for building the target.
%            "available_handler" is a function handle to the function that
%            checks whether that target is available or not. Both, the build
%            and the availability handler get a copy of the target structure
%            when called.
%
%            The build handler must return true if it built the target
%            successfully and false in case it did not. It may use the
%            availability handler as an indicator of success.
%
%            The availability handler must return either true, false or a
%            number representing the last modification time of that target.
%            true is returned, in case the target is available and it cannot
%            become obsolete. This might be useful, in case a target provides
%            a certain resource, like a network connection. The handler must
%            return false, in case a target is unavailable and the last
%            modification time (in MATLAB's date/time format) in case the
%            target is available and may become obsolete.
%
%            One may omit the availability handler when calling createTarget.
%            In this case, the default availability handler "isAvailable" is
%            chosen.
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at

  if nargin == 4
      available_handler = varargin(1);
  else
      available_handler = @isAvailable;
  end

  target = struct('name', name, 'dependencies', ...
                  {dependencies} ,...
                  'build_handler', build_handler, ...
                  'available_handler', available_handler);
