function [vertices,new_volumes,varargout] = ...
             refineVolumeMesh(vertices,volumes,varargin)

% refineVolumeMesh refines a given volume mesh via Newest Vertex Bisection (NVB)
% of certain marked volumes and guarantees that the resulting mesh is regular
%
% Usage: [VERTICES,VOLUMES [,FATHER2SON]] = ...
%            refineVolumeMesh(VERTICES,VOLUMES [,MARKED_VOLUMES])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed 
%              boundary conditions by use of BEM in 2D.
%
%              Let {T1,...,TN} be a mesh described by matrices VERTICES and
%              VOLUMES. The optional (m x 1)-vector MARKED_VOLUMES contains
%              the indices of all volumes Tj which will be refined by
%              newest vertex bisection.
%              The refined triangulation is returned in terms of
%              extended matrices VERTICES and VOLUMES. Moreover, the
%              optional (N x 4)-matrix FATHER2SON links the initial mesh
%              with the refined mesh in the sense that FATHER2SON(j,:)
%              contains the indices of the fine volumes which are the sons of
%              the coarse-mesh volume Tj. In case of a coarse mesh
%              volume has less than four sons the respective row 
%              FATHER2SON(j,:) is filled up with trailing zeros.
%              If Tj has not been refined, FATHER2SON(j,1) contains 
%              the index of Tj with respect to the refined mesh.
%
%              If the optional vector MARKED_VOLUMES is not given, the function
%              performs a uniform refinement, i.e., all volumes Tj are refined
%              into four sons.
%
% Author: Markus Aurada
%
% (C) M. Aurada, M. Ebner, S. Ferraz-Leite,
%     P. Goldenits, M. Karkulik, M. Mayr,
%     D. Praetorius
%
% Version: 3.0

%*** check the correct number of input parameters
if ~( (nargin == 2) || (nargin == 3) )
    error('refineVolumeMesh: Wrong number of input arguments!');
end

%*** check the correct number of output parameters
if ~( (nargout == 2) || (nargout == 3) )
    error('refineVolumeMesh: Wrong number of output arguments!');
end

%*** number of vertices and volumes 
nVerts = size(vertices,1);
nVols = size(volumes,1);

%*** if uniform refinement is requested, mark all volumes
if nargin == 3
   marked_vols = varargin{nargin-2};
else
   marked_vols = [1:nVols]';
end

%*** Node vectors of all edges (interior edges appear twice) 
I = volumes(:);
J = reshape(volumes(:,[2,3,1]),3*nVols,1);

%*** Symmetrize I and J (so far boundary edges appear only once)
symm_IJ = sparse([I;J],[J;I],[ones(3*nVols,1);-ones(3*nVols,1)]);
[II,JJ] = ind2sub(size(symm_IJ),find(symm_IJ<0));
I = [I;II];
J = [J;JJ];

%*** Obtain geometric information on edges
idx_IJ = find(I<J);
edge_number = zeros(length(I),1);
edge_number(idx_IJ) = 1:length(idx_IJ);
idx_JI = find(I>J);
number2edges = sparse(I(idx_IJ),J(idx_IJ),1:length(idx_IJ));
[foo{1:2},numbering_IJ] = find(number2edges);
[foo{1:2},idx_JI2IJ] = find( sparse(J(idx_JI),I(idx_JI),idx_JI) );
edge_number(idx_JI2IJ) = numbering_IJ;

%*** Provide volumes2edges and edge2nodes
volume2edges = reshape(edge_number(1:3*nVols),nVols,3);
edge2nodes = [I(idx_IJ),J(idx_IJ)];

%*** for marked elements, mark its reference edge for bisection
edge2new_node = zeros(length(edge2nodes),1);
edge2new_node(volume2edges(marked_vols,:)) = 1;

%*** recursively mark further edges to avoid hanging nodes in the refined mesh
swap = 1;
while ~isempty(swap)
    marked_edge = edge2new_node(volume2edges);
    swap = find( ~marked_edge(:,1) & (marked_edge(:,2) | marked_edge(:,3)) );
    edge2new_node(volume2edges(swap,1)) = 1;
end

%*** Generate new nodes
edge2new_node(edge2new_node~=0) = nVerts + (1:nnz(edge2new_node));
idx = find(edge2new_node);
vertices(edge2new_node(idx),:) = ...
   0.5*(vertices(edge2nodes(idx,1),:) + vertices(edge2nodes(idx,2),:));

%*** Provide new nodes for refinement of volumes
new_nodes = edge2new_node(volume2edges);

%*** Determine type of refinement for each volume
marked_edges = (new_nodes~=0);
none = ~marked_edges(:,1);
bisec1   = ( marked_edges(:,1) & ~marked_edges(:,2) & ~marked_edges(:,3) );
bisec12  = ( marked_edges(:,1) &  marked_edges(:,2) & ~marked_edges(:,3) );
bisec13  = ( marked_edges(:,1) & ~marked_edges(:,2) &  marked_edges(:,3) );
bisec123 = ( marked_edges(:,1) &  marked_edges(:,2) &  marked_edges(:,3) );

%*** Generate volume numbering for refined mesh
vol2num_sons = ones(nVols,1);
vol2num_sons(bisec1)   = 2; % bisec(1): newest vertex bisection of 1st edge
vol2num_sons(bisec12)  = 3; % bisec(2): newest vertex bisection of 
                            % 1st and 2nd edge
vol2num_sons(bisec13)  = 3; % bisec(2): newest vertex bisection of
                            % 1st and 3rd edge
vol2num_sons(bisec123) = 4; % bisec(3): newest vertex bisection of all edges
idx = [1;1+cumsum(vol2num_sons)];

%*** Generate new volumes
new_volumes = zeros(idx(end)-1,3);
new_volumes(idx(none),:) = volumes(none,:);
new_volumes([idx(bisec1),1+idx(bisec1)],:) = ...
   [volumes(bisec1,3),volumes(bisec1,1),new_nodes(bisec1,1); ...
    volumes(bisec1,2),volumes(bisec1,3),new_nodes(bisec1,1)];
new_volumes([idx(bisec12),1+idx(bisec12),2+idx(bisec12)],:) = ...
   [volumes(bisec12,3),volumes(bisec12,1),new_nodes(bisec12,1); ...
    new_nodes(bisec12,1),volumes(bisec12,2),new_nodes(bisec12,2); ...
    volumes(bisec12,3),new_nodes(bisec12,1),new_nodes(bisec12,2)]; 
new_volumes([idx(bisec13),1+idx(bisec13),2+idx(bisec13)],:) = ...
   [new_nodes(bisec13,1),volumes(bisec13,3),new_nodes(bisec13,3); ...
    volumes(bisec13,1),new_nodes(bisec13,1),new_nodes(bisec13,3); ...
    volumes(bisec13,2),volumes(bisec13,3),new_nodes(bisec13,1)];
new_volumes([idx(bisec123),1+idx(bisec123),2+idx(bisec123), ...
             3+idx(bisec123)],:) = ...
   [new_nodes(bisec123,1),volumes(bisec123,3),new_nodes(bisec123,3); ...
    volumes(bisec123,1),new_nodes(bisec123,1),new_nodes(bisec123,3); ...
    new_nodes(bisec123,1),volumes(bisec123,2),new_nodes(bisec123,2); ...
    volumes(bisec123,3),new_nodes(bisec123,1),new_nodes(bisec123,2)];

%*** build father2son
if nargout == 3
   tmp_2nd_col = bisec1 | bisec12 | bisec13 | bisec123;
   tmp_3nd_col = xor(tmp_2nd_col,bisec1);

   father2son = zeros(nVols,4);
   father2son(:,1) = idx(1:end-1);
   father2son(tmp_2nd_col,2) = idx(tmp_2nd_col) + 1;
   father2son(tmp_3nd_col,3) = idx(tmp_3nd_col) + 2;
   father2son(bisec123,4) = idx(bisec123) + 3;

   varargout{nargout-2} = father2son;
end

% History: MA  (06.10.2009 --> v0.6)
%          DPR (06.10.2009 --> v0.6.1) -> bisec1 statt bisec3 fuer marked_vols
%	   PG  (06.10.2009 -> History verschoben, Version 1.0)
%          MK  (04.05.2010 -> bisec3 statt bisec 1 für marked_vols
%
% DAS IST DIE AKTUELLE VERSION VON REFINEVOLUMEMESH!!!
%
% 
% WAS NOCH FEHLT:
% i)   DESCRIPTION -->> DONE  
% ii)  SOLL DER UNIFORME FALL GETRENNT BEHANDELT WERDEN?
%      ... der Gewinn ist ein Wimpernschlag, erzeugt aber mehr Code
% iii) TESTEN DER EINGABEPARAMETER -->> DONE
% iv)  CODE SCHOENERN -->> DONE
% v)   FRAGE AN REVIEWER: FAELLT DIR NOCH WAS AUF WAS FEHLEN KOENNTE?
