function ind = computeEstSlpResidual(coordinates,elements,phih,gh)

% computeEstSlpResidual returns the local contributions of the weighted-residual
% error estimator for Symm's integral equation V*phi = (K+1/2)*g for a direct
% boundary integral formulation
%
% Usage: IND = computeEstSlpResidual(COORDINATES,ELEMENTS,PHIH,GH)
%
% Let {T1,...,TN} be a triangulation of a boundary piece Gamma with nodes 
% z1,...,zn which is described in terms of the (n x 2)-matrix COORDINATES and
% the (N x 2)-matrix ELEMENTS. The (N x 1)-vector PHIH provides the elementwise
% values of the Galerkin solution phih, i.e. phih|_{Tj} = PHIH(j). The
% (n x 1)-vector GH provides the nodal value of an S1-approximation gh of the
% given data g, i.e. gh(zj) = GH(j).
%
% The function returns the elementwise contributions of the weighted-residual
% error estimator 
%
%   IND(j) = diam(Tj) * || [ (K+1/2)*gh - V*phih ]' ||_{L2(Tj)}^2
%
% where [.]' denotes the arclength-derivative.

%*** Gaussian quadrature on [-1,1] with 2 nodes and exactness 3
quad_nodes = [-1 1]/sqrt(3);
quad_weights = [1;1];

%*** elementwise interpolation is done in (gauss_left,gauss_right,midpoint)
quad_nodes(3) = 0;

%*** the remaining code is independent of the chosen quadrature rule
nE = size(elements,1);
nQ = length(quad_nodes);

%*** build vector of evaluations points as (nQ*nE x 2)-matrix
a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);
sx = reshape(a,2*nE,1)*(1-quad_nodes) + reshape(b,2*nE,1)*(1+quad_nodes);
sx = 0.5*reshape(sx',nQ*nE,2);

%*** evaluate gh elementwise at (left, right, midpoint)
gh_left = gh(elements(:,1));
gh_right = gh(elements(:,2));
gh_sx = gh_left*(1-quad_nodes) + gh_right*(1+quad_nodes);
gh_sx = 0.5*reshape(gh_sx',nQ*nE,1);

%*** evaluate p = V*phih - (K+1/2)*gh elementwise at (left, right, midpoint)
p = evaluateV(coordinates,elements,phih,sx) ...
  - evaluateK(coordinates,elements,gh,sx) ...
  - 0.5*gh_sx;

%*** evaluate arclength-derivative p' elementwise at (left,right)
p_prime = reshape(p,nQ,nE)' * [-3 1 ; -1 3 ; 4 -4]*sqrt(3)/2;

%*** return ind(j) = diam(Ej) * || [ V*phi - (K+1/2)*gh ]' ||_{L2(Ej)}^2
ind = 2*p_prime.^2*quad_weights;

% Author: Dirk Praetorius
%
% History: DPR 17.11.2009