function ind = computeEstSlpResidualIndirect(coordinates,elements,phih,g)

% computeEstSlpResidual returns the local contributions of the weighted-residual
% error estimator for Symm's integral equation V*phi = g for an indirect
% boundary integral formulation
%
% Usage: IND = computeEstSlpResidualIndirect(COORDINATES,ELEMENTS,PHIH,G)
%
% Let {T1,...,TN} be a triangulation of a boundary piece Gamma with nodes 
% z1,...,zn which is described in terms of the (n x 2)-matrix COORDINATES and
% the (N x 2)-matrix ELEMENTS. The (N x 1)-vector PHIH provides the elementwise
% values of the Galerkin solution phih, i.e. phih|_{Tj} = PHIH(j). Let g be the
% right-hand side for Symm's integral equation. The input parameter G is either
% the nodal basis vector of an S1-function gh which approximatess g, i.e.
% gh(zj) = G(j), or a function handle to evaluate g(x). In the latter case, we
% assume that the corresponding function takes an (M x 2)-matrix X of evaluation
% points and returns an (M x 1)-vector Y of functions values, i.e.
% Y(j) = g(X(j,:))

%*** Gaussian quadrature on [-1,1] with 2 nodes and exactness 3
quad_nodes = [-1 1]/sqrt(3);
quad_weights = [1;1];

%*** elementwise interpolation is done in (gauss_left,gauss_right,midpoint)
quad_nodes(3) = 0;

%*** the remaining code is independent of the chosen quadrature rule
nE = size(elements,1);
nQ = length(quad_nodes);

%*** build vector of evaluations points as (nQ*nE x 2)-matrix
a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);
sx = reshape(a,2*nE,1)*(1-quad_nodes) + reshape(b,2*nE,1)*(1+quad_nodes);
sx = 0.5*reshape(sx',nQ*nE,2);

%*** evaluate gh elementwise at (left, right, midpoint)
if isa(g,'double')
    g_left = g(elements(:,1));
    g_right = g(elements(:,2));
    g_sx = g_left*(1-quad_nodes) + g_right*(1+quad_nodes);
    g_sx = 0.5*reshape(g_sx',nQ*nE,1);
else isa(g,'function_handle')
    g_sx = g(sx);
end

%*** evaluate p = V*phih - g elementwise at (left, right, midpoint)
p = evaluateV(coordinates,elements,phih,sx) - g_sx;

%*** evaluate arclength-derivative p' elementwise at (left,right)
p_prime = reshape(p,nQ,nE)' * [-3 1 ; -1 3 ; 4 -4]*sqrt(3)/2;

%*** return ind(j) = diam(Ej) * || [ V*phi - g ]' ||_{L2(Ej)}^2
ind = 2*p_prime.^2*quad_weights;

% Author: Dirk Praetorius
%
% History: DPR 17.11.2009 (Formulierung fuer direkte BEM, d.h. inkl. DLP)
%          DPR 01.12.2009 (indireke BEM)