function [fails,passes,relative_errors]=testComputeVij()

%testSLP: Performs several tests to verify the C-Function computeVij
%         contained in singleLayerPotential.c
%
%Usage: [fails,passes,relative_errors]=testComputeVij()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D. The
%             function computeVij that is tested is supposed to compute
%             the entry into the stiffness matrix V
%             for given boundary elements [a,b] and [c,d].
%             For a given far field random cases are tested for analytical
%             and semianalytic computation. Reference soultion is generated
%             using dblquad.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... errors compaired to the reference solution.
%
%Authors: Samuel Ferraz-Leite
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: SFL (20.07.2009 -> creation and implementation -> v0.9)
%         MM  (21.07.2009 -> rules for compiling removed. -> v0.9.1)
%Version: 3.0

%Initialize variables
passes = 0;
fails = 0;
relative_errors = zeros(120,2); %first col.: analytic - second: semidiscr.

%Test 48 cases where segments lie on the same line with all kinds of
%orientations
for i=1:3
    value=zeros(8,2);
    a = random('unif',-1,1,2,1);
    vector = random('unif',-2,2,2,1);
    try
        vector = vector/norm(vector,2);
    catch
        vector = [1;1];
    end
    b = a+random('unif',0.2,2,1,1)*vector;
    c = b+random('unif',0.5,3,1,1)*vector;
    d = c+random('unif',0.2,2,1,1)*vector;
    value(1,1) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),0);
    value(2,1) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),0);
    value(3,1) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),0);
    value(4,1) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),0);
    value(5,1) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),0);
    value(6,1) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),0);
    value(7,1) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),0);
    value(8,1) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),0);
    value(1,2) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),eps);
    value(2,2) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),eps);
    value(3,2) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),eps);
    value(4,2) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),eps);
    value(5,2) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),eps);
    value(6,2) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),eps);
    value(7,2) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),eps);
    value(8,2) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),eps);
    f=@(s,t)(f0(s,t,a,b,c,d));
    reference = ...
              -1/(2*pi)*norm(b-a,2)*norm(d-c,2)*dblquad(f,0,1,0,1,1e4*eps);
    relative_errors((i-1)*8+1:i*8,:) = ...
                                      abs(reference-value)./abs(reference);
    for j=1:8
        if (relative_errors((i-1)*8+j,1)>=1e6*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d analytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
        if (relative_errors((i-1)*8+j,2)>=1e8*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d semianalytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
    end
end

%Test 48 cases where segments are parallel with all kinds of orientations
for i=4:6
    value=zeros(8,2);
    a = random('unif',-1,1,2,1);
    vector = random('unif',-2,2,2,1);
    try
        vector = vector/norm(vector,2);
    catch
        vector = [1;1];
    end
    b = a+random('unif',0.2,2,1,1)*vector;
    c = a+random('unif',1,5,1,1)*[vector(2);-vector(1)]+...
                             random('unif',-2,2,1,1)*vector;
    d = c+random('unif',0.2,2,1,1)*vector;
    value(1,1) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),0);
    value(2,1) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),0);
    value(3,1) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),0);
    value(4,1) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),0);
    value(5,1) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),0);
    value(6,1) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),0);
    value(7,1) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),0);
    value(8,1) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),0);
    value(1,2) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),eps);
    value(2,2) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),eps);
    value(3,2) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),eps);
    value(4,2) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),eps);
    value(5,2) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),eps);
    value(6,2) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),eps);
    value(7,2) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),eps);
    value(8,2) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),eps);
    f=@(s,t)(f0(s,t,a,b,c,d));
    reference = ...
              -1/(2*pi)*norm(b-a,2)*norm(d-c,2)*dblquad(f,0,1,0,1,1e4*eps);
    relative_errors((i-1)*8+1:i*8,:) = ...
                                      abs(reference-value)./abs(reference);
    for j=1:8
        if (relative_errors((i-1)*8+j,1)>=1e6*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d analytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
        if (relative_errors((i-1)*8+j,2)>=1e8*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d semianalytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
    end
end

%24 completely random cases!
for i = 7:9
    a = random('unif',-1,1,2,1);
    vector = random('unif',-2,2,2,1);
    try
        vector = vector/norm(vector,2);
    catch
        vector = [1;1];
    end
    b = a+random('unif',0.2,2,1,1)*vector;
    c = random('unif',5,8,2,1);
    vector = random('unif',-2,2,2,1);
    try
        vector = vector/norm(vector,2);
    catch
        vector = [1;1];
    end    
    d = c+random('unif',0.2,2,1,1)*vector;
    value(1,1) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),0);
    value(2,1) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),0);
    value(3,1) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),0);
    value(4,1) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),0);
    value(5,1) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),0);
    value(6,1) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),0);
    value(7,1) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),0);
    value(8,1) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),0);
    value(1,2) = mxComputeVij(a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2),eps);
    value(2,2) = mxComputeVij(a(1),a(2),b(1),b(2),d(1),d(2),c(1),c(2),eps);
    value(3,2) = mxComputeVij(b(1),b(2),a(1),a(2),c(1),c(2),d(1),d(2),eps);
    value(4,2) = mxComputeVij(b(1),b(2),a(1),a(2),d(1),d(2),c(1),c(2),eps);
    value(5,2) = mxComputeVij(c(1),c(2),d(1),d(2),a(1),a(2),b(1),b(2),eps);
    value(6,2) = mxComputeVij(d(1),d(2),c(1),c(2),a(1),a(2),b(1),b(2),eps);
    value(7,2) = mxComputeVij(c(1),c(2),d(1),d(2),b(1),b(2),a(1),a(2),eps);
    value(8,2) = mxComputeVij(d(1),d(2),c(1),c(2),b(1),b(2),a(1),a(2),eps);
    f=@(s,t)(f0(s,t,a,b,c,d));
    reference = ...
              -1/(2*pi)*norm(b-a,2)*norm(d-c,2)*dblquad(f,0,1,0,1,1e4*eps);
    relative_errors((i-1)*8+1:i*8,:) = ...
                                      abs(reference-value)./abs(reference);
    for j=1:8
        if (relative_errors((i-1)*8+j,1)>=1e6*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d analytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
        if (relative_errors((i-1)*8+j,2)>=1e8*eps)
            fails=fails+1;
            fprintf('Testcase failed at permutation %d semianalytic!\n',j);
            fprintf('-------------------\n');
            fprintf('a=[%g,%g], b=[%g,%g] c=[%g,%g] d=[%g,%g]\n',...
                             a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
            fprintf('C-Function returned %g\n',value(j));
            fprintf('quad returned %g\n',reference);
            fprintf('relative error %g\n',relative_errors((i-1)*8+j));
            fprintf('absolute error %g\n',abs(reference-value(j)));
            fprintf('-------------------\n');
            fprintf('-------------------\n\n');
        else
            passes=passes+1;
        end
    end
end


function val = f0(s,t,a,b,c,d)
val = zeros(length(s),length(t));
for i = 1:length(s)
    for j = 1:length(t)
        val(i,j) = log(norm(a+s(i)*(b-a)-c-t(j)*(d-c),2));
    end
end
