function [fails,passes,relative_errors]=testDistancePointToSegment()

%testDistancePointToSegment: Performs several tests to verify the
%                            C-Function distancePointToSegment implemented
%                            in geometry.c
%
%Usage: [fails,passes,relative_errors]=testDistancePointToSegment()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D. The
%             function distancePointToSegment is supposed to compute
%             the euclidian distance between a given point p and a line
%             segment [a,b]. First, 25 cases are tested where the distance
%             of p to [a,b] is the distance of p to a. Then 25 cases are
%             tested, where the distance of p to [a,b] is the distance of p
%             to b. The next 25 tests consider p lying besides the segment
%             [a,b] at a distance of sqrt(2)/2. Finally 25 cases are
%             considered, where p is element of [a,b], hence the distance
%             is 0. All cases are generated somewhat randomly.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... comparison to reference solution.
%
%Authors: Samuel Ferraz-Leite
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: SFL (14.07.2009 -> creation and implementation -> 0.4.1)
%         SFL (17.07.2009 -> now works with new version -> 0.9.1)
%         MM  (21.07.2009 -> rules for compiling removed -> 0.9.2)
%Version: 3.0

%Initialize data
fprintf('100 cases are being tested.\n\n');

fails = 0;
passes = 0;
relative_errors = zeros(100,1);

%Test case of point being on the same line as [a,b] but is outside
for i = 1:25
    %Create random configuration with known distance of p from a
    a = random('unif',0,10,1,2);
    vector = random('unif',1,2,1,2);
    vector = vector/norm(vector,2);
    b = a+random('unif',0.1,5,1,1)*vector;
    distance = random('unif',0.1,5,1,1);
    p = a-distance*vector;
    %Compare known distance with computation of distancePointToSegment
    value = mxDistancePointToSegment(p(1),p(2),a(1),a(2),b(1),b(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], p=[%g,%g]\n',a(1),a(2),...
                                                    b(1),b(2),p(1),p(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

for i = 26:50
    %Create random configuration with known distance of p from b
    b = random('unif',0,10,1,2);
    vector = random('unif',1,2,1,2);
    vector = vector/norm(vector,2);
    a = b+random('unif',0.1,5,1,1)*vector;
    distance = random('unif',0.1,5,1,1);
    p = b-distance*vector;
    %Compare known distance with computation of distancePointToSegment
    value = mxDistancePointToSegment(p(1),p(2),a(1),a(2),b(1),b(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], p=[%g,%g]\n',a(1),a(2),...
                                                    b(1),b(2),p(1),p(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%Now test some cases where p is not on the same line as [a,b] and the
%distance of p to [a,b] is not the distance to one of its endpoints.
for i = 51:75
    a = [0,0];
    b = [1,1];
    distance = sqrt(2)/2;
    t = random('unif',0,1,1,1);
    p = [-0.5,0.5]+t*[1,1];
    value = mxDistancePointToSegment(p(1),p(2),a(1),a(2),b(1),b(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], p=[%g,%g]\n',a(1),a(2),...
                                                    b(1),b(2),p(1),p(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%Finally consider p\in [a,b] --> distance=0
for i = 76:100
    a = [0,0];
    b = [1,0];
    t = random('unif',0,1,1,1);
    p = [t,0];
    distance = 0;
    value = mxDistancePointToSegment(p(1),p(2),a(1),a(2),b(1),b(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], p=[%g,%g]\n',a(1),a(2),...
                                                    b(1),b(2),p(1),p(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end
