function [fails,passes,relative_errors]=testDistanceSegmentToSegment()

%testDistanceSegmentToSegment: Performs several tests to verify the
%                            C-Function distanceSegmentToSegment contained
%                            in geometry.c
%
%Usage: [fails,passes,relative_errors]=testDistanceSegmentToSegment()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D. The
%             function distanceSegmentToSegment that is tested is supposed
%             to compute the euclidian distance between two segments [a,b]
%             and[c,d]. First, 25 cases are tested where the segments
%             intersect. Then 25 cases are tested, where the segments are
%             parallel at random distance. Finally 40 cases are tested
%             where the segments lie on the same line and have random
%             distance to each other.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... comparison to the reference solution.
%
%Authors: Samuel Ferraz-Leite
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: SFL (15.07.2009 -> creation and partial implementation -> 0.2.1)
%         SFL (17.07.2009 -> modify function calls -> 0.4.1)
%         MM  (21.07.2009 -> rules for compiling removed -> 0.4.2)
%Version: 3.0

%Initialize data
fprintf('90 cases are being tested.\n\n');

fails = 0;
passes = 0;
relative_errors=zeros(90,1);

%25 cases: crossing segments with angle >= 0
for i = 1:25
    intersection_point = random('unif',-10,10,1,2);
    
    %construct first segment with random length and orientation
    direction = random('unif',-1,1,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    a = intersection_point+random('unif',0,5,1,1)*direction;
    b = intersection_point-random('unif',0,5,1,1)*direction;
    
    %construct second segment with random length and orientation
    direction = random('unif',-1,1,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    c = intersection_point+random('unif',0,5,1,1)*direction;
    d = intersection_point-random('unif',0,5,1,1)*direction;
    
    %call distanceSegmentToSegment and compare results
    distance = 0;
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%25 cases: chose a random distance and construct parallel segments
for i = 26:50
    %construct random segment with random direction
    a = random('unif',-10,10,1,2);
    direction = random('unif',-2,2,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    b = a+random('unif',0.2,1,1,1)*direction;
    
    %choose random distance, construct parallel but a but shifted segment
    distance=random('unif',0,10,1,1);
    c=a+distance*[direction(2),-direction(1)]+...
                 random('unif',0.01,0.2,1,1)*direction;
    d=c+random('unif',3,10,1,1)*direction;
    
    %compare result from distanceSegmentToSegment with known solution
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end
    
%10 cases: both segment are on line with known distance euqual to the
%distance between a and c
for i = 51:60
    a = random('unif',-10,10,1,2);
    direction = random('unif',-2,2,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    b = a+random('unif',0.2,1,1,1)*direction;
    distance=random('unif',0.1,1,1,1);
    c=a-distance*direction;
    d=c-random('unif',0.2,1,1,1)*direction;
    %compare result from distanceSegmentToSegment with known solution
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%10 cases: both segment are on line with known distance euqual to the
%distance between a and d
for i = 61:70
    a = random('unif',-10,10,1,2);
    direction = random('unif',-2,2,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    b = a+random('unif',0.2,1,1,1)*direction;
    distance=random('unif',0.1,1,1,1);
    d=a-distance*direction;
    c=d-random('unif',0.2,1,1,1)*direction;
    %compare result from distanceSegmentToSegment with known solution
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%10 cases: both segment are on line with known distance euqual to the
%distance between b and c
for i = 71:80
    a = random('unif',-10,10,1,2);
    direction = random('unif',-2,2,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    b = a+random('unif',0.2,1,1,1)*direction;
    distance=random('unif',0.1,1,1,1);
    c=b+distance*direction;
    d=c+random('unif',0.2,1,1,1)*direction;
    %compare result from distanceSegmentToSegment with known solution
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end

%10 cases: both segment are on line with known distance euqual to the
%distance between b and c
for i = 81:90
    a = random('unif',-10,10,1,2);
    direction = random('unif',-2,2,1,2);
    try
        direction = direction/norm(direction,2);
    catch
        direction = [1,1];
    end
    b = a+random('unif',0.2,1,1,1)*direction;
    distance=random('unif',0.1,1,1,1);
    d=b+distance*direction;
    c=d+random('unif',0.2,1,1,1)*direction;
    %compare result from distanceSegmentToSegment with known solution
    value = mxDistanceSegmentToSegment(a(1),a(2),b(1),b(2),...
                                       c(1),c(2),d(1),d(2));
    relative_errors(i) = abs(distance-value)/abs(distance);
    if (relative_errors(i)>1e4*eps)
        fails = fails+1;
        fprintf('Testcase failed!\n');
        fprintf('-------------------\n');
        fprintf('a=[%g,%g], b=[%g,%g], c=[%g,%g], d=[%g,%g]\n',...
                 a(1),a(2),b(1),b(2),c(1),c(2),d(1),d(2));
        fprintf('C-Function returned %g\n',value);
        fprintf('distance is %g\n',distance);
        fprintf('relative error %g\n',relative_errors(i));
        fprintf('absolute error %g\n',abs(distance-value));
        fprintf('-------------------\n');
        fprintf('-------------------\n\n');
    else
        passes = passes+1;
    end
end
