% testEvaluateN.m
%
% DESCRIPTION: Tests evaluation of the Newton potential:
%              Integrate
%                  -1/(2*pi)*\int_Ej \int_\Omega log|x-y|f(y) dy ds_x
%              where Ej is a boundary element. Outer integration is done
%              by Matlab routines, where evaluation of inner integral is
%              replaced by HILBERT's evaluateN function.
%              As a reference solution we will use the vector (N*fh)
%              where fh contains the values of the elementwise constant f
%              and N is the matrix that comes from the buildN function.
%
% AUTHOR: Thomas Fuehrer, 03.11.2011

function [ fails, passes, rel_err, abs_err ] = testEvaluateN( varargin )
  
  addpath('../lib')
  
  % A test is considered as failed if relative error between reference
  % solution and approximation is larger than relTol
  relTol = 1e-8;
  
  % Number of uniform refinements
  nRef = 4;
  
  if( nargin >= 2)
      error(sprintf(['Too many arguments in testEvaluateN.\n' ...
                       '  Usage: testEvaluateN(nRef);\n']));
  elseif (nargin==1)
          nRef = varargin{1};
  end
  
  % test on \Omega = [0,1]^2, f=1
  vertices = [0 0; 1 0; 1 1; 0 1; 0.5 0.5];
  volumes = [1 2 5; 2 3 5; 3 4 5; 4 1 5];
  boundary = [1 2; 2 3; 3 4; 4 1];
  
  for i = 1:nRef
      [vertices volumes boundary] = refineMesh(vertices,volumes,boundary);
  end
  nE = size(volumes,1);
  nB = size(boundary,1);
  
  % f = 1
  f = ones(nE,1);
  
  % build Matrix N (P0-ansatz, P0 testfunctions)
  N = buildN(vertices,boundary,vertices,volumes);
  
  % compute (Nf) in P0
  ref = N*f;
  
  h = sqrt( sum((vertices(boundary(:,1),:)-vertices(boundary(:,2),:)).^2,2) );
  x = zeros(nB,1);
  for i=1:nB
      a = vertices(boundary(i,1),:);
      b = vertices(boundary(i,2),:);
      u = (b-a)*0.5;
      v = (b+a)*0.5;
      %x(i) = 0.5 * quad(@(y) funeval(vertices,volumes,y,u,v,f), -1,1);
      x(i) = 0.5 * h(i)* quadgk(@(y) funeval(vertices,volumes,y,u,v,f), -1,1,'RelTol',1e-13);
      %x(i) = 0.5 * h(i)* quadl(@(y) funeval(vertices,volumes,y,u,v,f), -1,1,'RelTol',1e-13);
  end
  
  abs_err = abs(ref-x);
  rel_err = abs_err./abs(ref);
  
  fails = sum(rel_err>relTol);
  passes = nB - fails;
end

function out = funeval(vertices,volumes,y,u,v,f)

  n = length(y);
  out = zeros(size(y));
  
  for i=1:n
       out(i) = evaluateN(vertices,volumes,f,u*y(i)+v);
  end
end

% HISTORY: MK 23.11.11 -> code review, leichte änderugen
