function [ fails, passes, rel_err, abs_err ] = testEvaluateN2( varargin )

% testEvaluateN2.m
% 
% DESCRIPTION: Test of the evaluation of the Newton potential:
%                 -) creates some random points x_i in- and outside the 
%                    domain \Omega = [0,1]^2. 
%                 -) evaluates the Newton potential at points x_i with help
%                    of the HILBERT function evaluateN
%                        s_i = -1/(2*\pi) * \int_\Omega log|x_i-y| dy
%                 -) these results will then be compared with standard 
%                    integration routines from Matlab 
%
% AUTHOR: Thomas Fuehrer, 03.11.2011

addpath('../lib')

% A test is considered as failed if relative error between reference
% solution and approximation is larger than relTol
relTol = 1e-8;

% Number of uniform refinements
nRef = 4;

if( nargin >= 2)
    error(sprintf(['Too many arguments in testEvaluateN.\n' ...
                     '  Usage: testEvaluateN(nRef);\n']));
elseif (nargin==1)
        nRef = varargin{1};
end

%% test on \Omega = [0,1]^2, f=1
vertices = [0 0; 1 0; 1 1; 0 1; 0.5 0.5];
volumes = [1 2 5; 2 3 5; 3 4 5; 4 1 5];

for i = 1:nRef
    [vertices volumes] = refineVolumeMesh(vertices,volumes);
end
nVol = size(volumes,1);
nVert = size(vertices,1);


n1 = 10;
n2 = 10;
nxy = n1+n2;
%nxy = nxy+nVert;
xy = zeros(nxy,2);

% create random numbers within \Omega
xy(1:n1,:) = rand(n1,2);

% add some more points (may lay outside of \Omega)
xy(n1+1:n1+n2,:) = 1000*rand(n2,2);

% add nodes of the mesh
%xy(n1+n2+1:end,:) = vertices;

% f = 1
f = ones(nVol,1);

% evaluate Newton pot. at points xy with HILBERT function 
x = evaluateN(vertices,volumes,f,xy); 

% Compute solution with Matlab integration routines
ref = zeros(nxy,1);
for i=1:nxy
    %ref(i) = -1/(2*pi) * dblquad( @(x,y) log(sqrt((xy(i,1)-x).^2+(xy(i,2)-y).^2)), 0,1,0,1,1e-8,@quadl);
    ref(i) = -1/(4*pi) * quad2d( @(x,y) log( ((xy(i,1)-x).^2+(xy(i,2)-y).^2)), 0,1,0,1,...
        'AbsTol',1e-20);
end

% compare solutions
abs_err = abs(ref-x);
rel_err = abs(abs_err./ref);

fails = sum(rel_err>relTol);
passes = nxy - fails;

% History: MK (23.11.11 -> code review, leichte änderungen)
