function [fails,passes,absolute_errors] = testEvaluateWKPrime()

% testEvaluateWAndKadj: Performs several tests to verify the MEX-Functions
%                         evaluateW and evaluateKadj
%                         contained in evaluateW.c and evaluateKadj.c
%
% Usage: [fails,passes,relative_errors]=testEvaluateWAndKadj()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D.
%	      The functions buildW and buildK that are tested are supposed
%	      to compute the entries of the discrete hypersingular integral
%	      operator W and the discrete adjoint double-layer potential
%	      operator KPrime.
%	      buildK computes the discrete double-layer potential K,
%	      but KPrime = transpose(K).
%	      Tests are performed for several geometries and known solutions.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... errors compaired to the reference_evalW solution.
%
%Authors: Michael Karkulik
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

addpath ../lib
tol=1e-7;
fails=0;
passes=0;
absolute_errors=0;

%==============================================================================
% Test W(1) = 0 for random meshes on several geometries
%==============================================================================
%
% Square [0, 0.5]^2
%
% We evaluate W(1) on the midpoint of every element and check if the result
% is zero
fprintf('\n=================================================');
fprintf('\n# Starting Test 1: W1 = 0, mesh-size~800 elements');
fprintf('\n=================================================');
fprintf('\n\n# Test 1.A, 3 cases: Omega = [0,0.5]^2\n');
for i=1:3
  % generate a random mesh on the domain
  points_so_far = 0; %#ok<*NASGU>
  x_coordinates = [0;random('unif',0,0.5,50,1);0.5];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  coordinates = [x_coordinates,zeros(size(x_coordinates))];
  [~,I] = sort(x_coordinates);
  elements = [I(1:end-1),I(2:end)];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.0001,0.5,50,1);0.5];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates]; %#ok<*AGROW>
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0,0.499,50,1);0];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = random('unif',0.001,0.499,50,1);
  [~,I] = unique(y_coordinates,'first');
  y_coordinates = y_coordinates(I);
  [~,I] = sort(y_coordinates,'descend');    
  coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  elements = [elements;I(end)+points_so_far 1];

  midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );
  reference_evalW = zeros(size(midpoints,1),1);
  uD = ones(size(coordinates,1),1);

  % create outer normal vector to every element
  n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
  n_x = [ n_x(:,2) -n_x(:,1) ];
  norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
  n_x = n_x ./ [norm_n_x norm_n_x];

  EW = evaluateW(coordinates,elements,uD,midpoints,n_x);

  for j=1:size(midpoints,1)
    absolute_error = abs(EW(j)-reference_evalW(j));
    rel_error = absolute_error./abs(reference_evalW);
    if ( rel_error > tol )
      fprintf('\n    WARNING: ref = %e, evalW = %e, abs_error = %e',reference_evalW(j), ...
              EW(j),absolute_error);
      fprintf('\n       %e,%e',midpoints(j,1),midpoints(j,2));
      fails=fails+1;
    else
      passes=passes+1;
    end
  end
end

%
% Square [0, 0.5]^2 rotated by angle alpha=pi/4
%
% We evaluate W(1) on the midpoint of every element and check if the result
% is zero
fprintf('\n\n# Test 1.B, 3 cases: Omega=[0,0.5]^2 rotated by pi/4\n');
for i = 4:6
  alpha = pi/4;
  rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
  
  points_so_far = 0;
  x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  coordinates = [x_coordinates,zeros(size(x_coordinates))];
  [~,I] = sort(x_coordinates);
  elements = [I(1:end-1),I(2:end)];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0,0.499,200,1);0];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = random('unif',0.001,0.499,200,1);
  [~,I] = unique(y_coordinates,'first');
  y_coordinates = y_coordinates(I);
  [~,I] = sort(y_coordinates,'descend');    
  coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  elements = [elements;I(end)+points_so_far 1];

  coordinates = (rotation_matrix*coordinates')';
  midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );
  reference_evalW = zeros(size(midpoints,1),1);
  uD = ones(size(coordinates,1),1);

  % create outer normal vector to every element
  n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
  n_x = [ n_x(:,2) -n_x(:,1) ];
  norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
  n_x = n_x ./ [norm_n_x norm_n_x];

  EW = evaluateW(coordinates,elements,uD,midpoints,n_x);

  for j=1:size(midpoints,1)
    absolute_error = abs(EW(j)-reference_evalW(j));
    if ( absolute_error > tol )
      fprintf('\n    WARNING: ref = %e, evalW = %e, abs_error = %e for midpoint(%d,:) = [%e,%e]',reference_evalW(j), ...
              EW(j),absolute_error, j, midpoints(j,1), midpoints(j,2));
      
      fails=fails+1;
    else
      passes=passes+1;
    end
  end
end
%
% Square [0, 0.5]^2 rotated by angle alpha=pi/3
%
% We evaluate W(1) on the midpoint of every element and check if the result
% is zero
fprintf('\n\n# Test 1.C, 3 cases: Omega=[0,0.5]^2 rotated by pi/3\n');
for i = 7:9
  alpha = pi/3;
  rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
  
  points_so_far = 0;
  x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  coordinates = [x_coordinates,zeros(size(x_coordinates))];
  [~,I] = sort(x_coordinates);
  elements = [I(1:end-1),I(2:end)];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0,0.499,200,1);0];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = random('unif',0.001,0.499,200,1);
  [~,I] = unique(y_coordinates,'first');
  y_coordinates = y_coordinates(I);
  [~,I] = sort(y_coordinates,'descend');    
  coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  elements = [elements;I(end)+points_so_far 1];

  coordinates = (rotation_matrix*coordinates')';

  midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );
  reference_evalW = zeros(size(midpoints,1),1);
  uD = ones(size(coordinates,1),1);

  % create outer normal vector to every element
  n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
  n_x = [ n_x(:,2) -n_x(:,1) ];
  norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
  n_x = n_x ./ [norm_n_x norm_n_x];

  EW = evaluateW(coordinates,elements,uD,midpoints,n_x);

  for j=1:size(midpoints,1)
    absolute_error = abs(EW(j)-reference_evalW(j));
    if ( absolute_error > tol )
      fprintf('\n    WARNING: ref = %e, evalW = %e, abs_error = %e',reference_evalW(j), ...
              EW(j),absolute_error); 
      fails=fails+1;
    else
      passes=passes+1;
    end
  end
end
%
% L-shaped domain, straight and rotated
%
% We evaluate W(1) on the midpoint of every element and check if the result
% is zero
fprintf('\n\n# Test 1.D, 3 cases: Omega=[0,0.5]^2\\[0.25,0.5]^2');
fprintf(', straight and rotated\n');
for i = 10:12
  alpha = (i-10)*pi/5;
  rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
  
  points_so_far = 0;
  x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  coordinates = [x_coordinates,zeros(size(x_coordinates))];
  [~,I] = sort(x_coordinates);
  elements = [I(1:end-1),I(2:end)];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.0001,0.25,100,1);0.25];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0.25,0.499,100,1);0.25];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0.25];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.25*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.251,0.5,100,1);0.5];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.25*ones(size(y_coordinates)),y_coordinates];
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0,0.249,100,1);0];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = random('unif',0.001,0.499,200,1);
  [~,I] = unique(y_coordinates,'first');
  y_coordinates = y_coordinates(I);
  [~,I] = sort(y_coordinates,'descend');    
  coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  elements = [elements;I(end)+points_so_far 1];

  if (alpha>0)
      coordinates = (rotation_matrix*coordinates')';
  end

  midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );
  reference_evalW = zeros(size(midpoints,1),1);
  uD = ones(size(coordinates,1),1);

  % create outer normal vector to every element
  n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
  n_x = [ n_x(:,2) -n_x(:,1) ];
  norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
  n_x = n_x ./ [norm_n_x norm_n_x];

  EW = evaluateW(coordinates,elements,uD,midpoints,n_x);

  for j=1:size(midpoints,1)
    absolute_error = abs(EW(j)-reference_evalW(j));
    if ( absolute_error > tol )
      fprintf('\n    WARNING: ref = %e, evalW = %e, abs_error = %e',reference_evalW(j), ...
              EW(j),absolute_error); 
      fails=fails+1;
    else
      passes=passes+1;
    end
  end
end
%
% Z-Shaped domain, straight and rotated
%
% We evaluate W(1) on the midpoint of every element and check if the result
% is zero
fprintf('\n\n# Test 1.E, 3 cases: Omega=[0,0.5]^2\\conv{(0,0)');
fprintf(',(0.25,0.25),(0,0.25)},');
fprintf('\n                     straight and rotated\n');
for i = 13:15
  alpha = (i-13)*pi/5;
  rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
  
  points_so_far = 0;
  x_coordinates = [0;random('unif',0,0.5,1,1);0.5];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  coordinates = [x_coordinates,zeros(size(x_coordinates))];
  [~,I] = sort(x_coordinates);
  elements = [I(1:end-1),I(2:end)];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.0001,0.5,1,1);0.5];
  [~,I] = unique(y_coordinates,'first');
  I = sort(I);
  y_coordinates = y_coordinates(I);
  coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
  [~,I] = sort(y_coordinates);
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0,0.499,1,1);0];
  [~,I] = unique(x_coordinates,'first');
  x_coordinates = x_coordinates(I);
  x_coordinates = [x_coordinates(2:end);0];
  [~,I] = sort(x_coordinates,'descend');
  coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  y_coordinates = [random('unif',0.25,0.499,1,1);0.25];
  [~,I] = unique(y_coordinates,'first');
  y_coordinates = y_coordinates(I);
  y_coordinates = [y_coordinates(2:end);0.25];
  [~,I] = sort(y_coordinates,'descend');    
  coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);

  x_coordinates = [random('unif',0.01,0.25,1,1);0.25];
  [~,I] = unique(x_coordinates,'first');
  I=sort(I);
  x_coordinates = x_coordinates(I);
  [~,I] = sort(x_coordinates);
  coordinates = [coordinates;x_coordinates,0.25*ones(size(x_coordinates))];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  points_so_far = length(coordinates);
  
  direction = [-1;-1];
  direction = direction/norm(direction,2);
  t = random('unif',0.01,sqrt(2)/4-0.0001,1,1);
  [~,I] = sort(t);
  for j = 1:length(t)
      new_coordinates(j,:) = ([0.25;0.25]+t(j)*direction)';
  end
  coordinates = [coordinates;new_coordinates];
  elements = [elements;points_so_far I(1)+points_so_far];
  elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
  elements = [elements;I(end)+points_so_far 1];
  
  if (alpha>0)
      coordinates = (rotation_matrix*coordinates')';
  end

  midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );
  reference_evalW = zeros(size(midpoints,1),1);
  uD = ones(size(coordinates,1),1);

  % create outer normal vector to every element
  n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
  n_x = [ n_x(:,2) -n_x(:,1) ];
  norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
  n_x = n_x ./ [norm_n_x norm_n_x];

  EW = evaluateW(coordinates,elements,uD,midpoints,n_x);

  for j=1:size(midpoints,1)
    absolute_error = abs(EW(j)-reference_evalW(j));
    if ( absolute_error > tol )
      fprintf('\n    WARNING: ref = %e, evalW = %e, abs_error = %e',reference_evalW(j), ...
              EW(j),absolute_error); 
      fails=fails+1;
    else
      passes=passes+1;
    end
  end
end

%==============================================================================
% Test for evalW and evalKadj:
% 
% When u(x,y) = x, then -Laplace(u)=0. The Representation formula predicts
%
%   0.5 * \partial_n u = Kadj(\partial_n u) + W(u)
%==============================================================================
fprintf('\n\n# Test 2.A, 1 case: Omega = [0,0.5]^2, Testing for representation formula');
addpath evaluateVK;
nEmax = 500;
theta = 0.5;
coordinates = load('coordinates.dat');
elements = load('elements.dat');

while (size(elements,1) < nEmax)
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    [~, uDh_fine] ...
        = computeOscDirichlet(coordinates_fine,elements_fine,@g1);
    V_fine = buildV(coordinates_fine,elements_fine);
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
    x_fine = V_fine\b_fine;

    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);

    %*** compute data oscillations
    osc_fine = computeOscDirichlet(coordinates_fine,elements_fine,@g1);
    osc_ = osc_fine(father2son(:,1))+osc_fine(father2son(:,2));

    %*** mark elements for refinement
    marked = markElements(theta,1/4,mu_tilde_+osc_);

    %*** generate new mesh
    [coordinates,elements] ...
        = refineBoundaryMesh(coordinates,elements,marked);

    %*** free obsolete memory
    clear V b
    clear coordinates_fine elements_fine father2son
    clear V_fine b_fine x_fine x
end

fprintf('\n  Solving on a mesh with %d elements',size(elements,1));
[~, uDh] = computeOscDirichlet(coordinates, elements, @g1);
V = buildV(coordinates, elements);
b = buildSymmRHS(coordinates, elements, uDh);
x = V \ b;

midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );

% create outer normal vector to every element
n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
n_x = [ n_x(:,2) -n_x(:,1) ];
norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
n_x = n_x ./ [norm_n_x norm_n_x];

EW = evaluateW(coordinates,elements,uDh,midpoints,n_x);
EKadj = evaluateKadj(coordinates,elements,x,midpoints,n_x);
reference = 0.5*phi1(midpoints, ...
            coordinates(elements(:,1),:),coordinates(elements(:,2),:));

for j=1:size(midpoints,1)
  absolute_error = abs(EW(j)+EKadj(j)-reference(j));
  if ( absolute_error > tol )
    fprintf('\n    WARNING: ref = %e, evalW+evalKadj = %e, abs_error = %e',reference(j), ...
            EW(j)+EKadj(j),absolute_error); 
    fails=fails+1;
  else
    passes=passes+1;
  end
end
%==============================================================================
% Test for evalW and evalKadj:
% 
% When u(x,y) = sin(2*pi*x)*cos(2*pi*y), then -Laplace(u)=0.
% The Representation formula predicts
%
%   0.5 * \partial_n u = Kadj(\partial_n u) + W(u)
%==============================================================================
fprintf('\n\n# Test 2.B, 1 case: Omega = [0,0.5]^2, Testing for representation formula');
addpath evaluateVK;
nEmax = 300;
theta = 0.5;
coordinates = load('coordinates.dat');
elements = load('elements.dat');

while (size(elements,1) < nEmax)
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    [~, uDh_fine] ...
        = computeOscDirichlet(coordinates_fine,elements_fine,@g);
    V_fine = buildV(coordinates_fine,elements_fine);
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
    x_fine = V_fine\b_fine;

    %*** compute (h-h/2)-error estimator tilde-mu
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);

    %*** compute data oscillations
    osc_fine = computeOscDirichlet(coordinates_fine,elements_fine,@g);
    osc_ = osc_fine(father2son(:,1))+osc_fine(father2son(:,2));

    %*** mark elements for refinement
    marked = markElements(theta,1/4,mu_tilde_+osc_);

    %*** generate new mesh
    [coordinates,elements] ...
        = refineBoundaryMesh(coordinates,elements,marked);

    %*** free obsolete memory
    clear V b
    clear coordinates_fine elements_fine father2son
    clear V_fine b_fine x_fine x
end

fprintf('\n  Solving on a mesh with %d elements',size(elements,1));
[~, uDh] = computeOscDirichlet(coordinates, elements, @g);
V = buildV(coordinates, elements);
b = buildSymmRHS(coordinates, elements, uDh);
x = V \ b;

midpoints = 0.5*( coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:) );

% create outer normal vector to every element
n_x = coordinates(elements(:,2),:) - coordinates(elements(:,1),:);
n_x = [ n_x(:,2) -n_x(:,1) ];
norm_n_x = sqrt( n_x(:,1).*n_x(:,1) + n_x(:,2).*n_x(:,2) );
n_x = n_x ./ [norm_n_x norm_n_x];

EW = evaluateW(coordinates,elements,uDh,midpoints,n_x);
EKadj = evaluateKadj(coordinates,elements,x,midpoints,n_x);
%reference = 0.5*phi(midpoints, ...
%            coordinates(elements(:,1),:),coordinates(elements(:,2),:));
reference = 0.5*x;

for j=1:size(midpoints,1)
  absolute_error = abs(EW(j)+EKadj(j)-reference(j));
    fprintf('\n    WARNING: ref = %e, evalW+evalKadj = %e, abs_error = %e',reference(j), ...
            EW(j)+EKadj(j),absolute_error); 
  if ( absolute_error > tol )
    fails=fails+1;
  else
    passes=passes+1;
  end
end

