%testNewtonPotential tests the function newtonPotential in
%newtonPotentialP0_new using MATLAB's quad2d as a reference.
%
%Usage: [fails, passes] = testNewtonPotential();
%
%Description: testNewtonPotential creates one random, fixed triangle Tj
%             where all corners are contained in [0,1]^2. Then it creates
%             40 random points xi in different areas around the triangle.
%             It computes the integral
%
%               int_Tj zeta(y) log |xi-y| dy
%
%             using an affine coordinate transformation and MATLAB's quad2d
%             routine on the one hand and newtonPotential on the onther
%             hand. Then it compares both values. A test is considerd as
%             failed when the relative error between the MATLAB reference
%             solution and the test solution is greater then 1e-10.
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%
%Authors: Markus Mayr
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius
%
%History: MM (19.05.2010 -> 1.1.0, Initial version)
%
%Version: 3.0

function [fails, passes] = testNewtonPotential
  addpath ../work/lib
  e = zeros(2,1);

  n1 = zeros(2,1); n2 = zeros(2,1); n3 = zeros(2,1);
  while (det([n2-n1,n3-n1]) < 1e-2)
    n1 = rand(2,1); n2 = rand(2,1); n3 = rand(2,1);
  end
  integrator = getIntegrator(n1,n2,n3);

  for i = 1:10
    for offset = [1.5 1.5 -1.5 -1.5; 1.5 -1.5 1.5 -1.5]
      x = rand(2,1) + offset;
      e = e + theTest(integrator,n1,n2,n3,x);
    end
  end
  
  n1 = [0;0]; n2 = [1;0]; n3 = [0;1];
  integrator = getIntegrator(n1,n2,n3);
  xs = [1 1.5 2.5 4.5 8.5 100 1000 10000; 1 1.5 2.5 4.5 8.5 100 1000 10000];

  for i = 1:size(xs,2)
      e = e+theTest(integrator,n1,n2,n3,xs(:,i));
  end
  
  fails = e(1);
  passes = e(2);
end

function e = theTest(integrator,n1,n2,n3,x)
      e = [0;0];
      ref = integrator(x); % MATLAB-Quadratur
      ftest = formulaTest(n1,n2,n3,x); % MATLAB-Implementierung
      test = mxNewtonPotential(n1,n2,n3,x); % C-Implementierung
      
      fprintf(['Reference value against formula test value: %g\n' ...
               'Formula test value against test value: %g\n' ...
               'Reference value against test value: %g\n'], ...
               abs(ref-ftest), abs(ftest-test), abs(ref-test));

      abs_err = abs(ref-test);
      rel_err = abs_err / abs(ref);
      if (rel_err > 1e-10)
        fprintf(['Failed: n1 = [%g,%g], n2 = [%g,%g], n3 = [%g,%g]\n'...
                 '        x = [%g,%g], abs_err = %g, rel_err = %g\n', ...
                 '        reference value = %g, test value = %g\n',...
                 '        test-ref quotient = %g\n'], ...
                  n1(1),n1(2),n2(1),n2(2),n3(1),n3(2),...
                  x(1),x(2),abs_err,rel_err, ref, test, test/ref);
%                 '        dist_n1n2_x = %g, dist_n2n3_x = %g\n', ...
%                 '        dist_n1n3_x = %g, diam = %g\n', ...
%                 '        dist_n1_n2 = %g, dist_n1_n3 = %g, ' ...
%                 'dist_n2_n3 = %g, |x| = %g\n'],...
%                  mxDistancePointToSegment(x(1),x(2),n1(1),n1(2),n2(1),n2(2)), ...
%                  mxDistancePointToSegment(x(1),x(2),n2(1),n2(2),n3(1),n3(2)), ...
%                  mxDistancePointToSegment(x(1),x(2),n1(1),n1(2),n3(1),n3(2)), ...
%                  max([norm(n1-n2), norm(n1-n3), norm(n2-n3)]), ...
%                  norm(n1-n2), norm(n1-n3), norm(n2-n3), norm(x) ...
%                  );
%        atanintRef = evalAtanInt(n1,n2,n3,x);
%        atanintTest = mxEvalAtanInt(n1,n2,n3,x);
%        disp(abs(atanintRef-atanintTest))
        e(1) = 1;
      else
        e(2) = 1;
      end
end

function val = formulaTest(n1,n2,n3,x)
    n1 = reshape(n1,2,1); n2 = reshape(n2,2,1); n3 = reshape(n3,2,1);
    u = reshape(n1-n2,2,1);
    v = reshape(n1-n3,2,1);
    w = reshape(x-n1,2,1);
    a = u'*u;
    
    volK = abs(det([n2-n1, n3-n1]))/2;
    
    I   = 0.25 * (mxSlp(0, (v-u)./2, w+u+(v-u)./2) ...
          - mxSlp(1, (v-u)./2, w+u+(v-u)./2));

    II  = (u'*w)*mxSlp(0,(v-u)./2,w+u+(v-u)./2)+...
          1/2*(u'*v)*mxSlp(0,(v-u)./2,w+u+(v-u)./2)+...
          1/2*(u'*v)*mxSlp(1,(v-u)./2,w+u+(v-u)./2);

    III = u'*(w+v./2)*mxSlp(0, v./2, w+v./2) ...
          + 0.5 *u'*v*mxSlp(1, v./2, w+v./2);

    val = volK * I + volK/(2 * a) * II - volK/(2 * a) * III ...
                  - volK + volK/(2*a) * evalAtanInt(n1,n2,n3,x);
              
%    testref = mxNewtonPotential(n1,n2,n3,x)
%    if(abs(val-testref)>1e-8)
%        error('FORMULAS DIFFER!');
%    end
end

function integrator = getIntegrator(n1,n2,n3)
  triangledesc = @(z) 1-z;
  integrator = @(x) quad2d(@(y1,y2) integrand(n1,n2,n3,x,y1,y2), ...
                            0, 1, 0, triangledesc, ...
                            'RelTol', 1e-12);
end

function values = integrand(n1,n2,n3,x,y1,y2)
    A = [n2-n1, n3-n1];
    T = @(y) n1 + A*y;
    values = zeros(size(y1));
    for i=1:size(y1,1)
        for j=1:size(y1,2)
            values(i,j) = ...%y2(i,j) * ...
                log(norm(x-T([y1(i,j);y2(i,j)]))) * abs(det(A));
        end
    end
end

function y = evalAtanInt(n1,n2,n3,x)
  u = reshape(n1-n2, 2, 1);
  v = reshape(n1-n3, 2, 1);
  w = reshape(x-n1, 2, 1);

  a = norm(u)^2;
  b = @(zeta) 2*u'*(w+zeta*v);
  c = @(zeta) norm(w+zeta*v)^2;
  Delta = @(zeta) 4*a*c(zeta)-b(zeta)^2;

  y = quad2d(@(eta,zeta) atanIntegrand(eta,zeta, a, b, c, Delta), ...
               0, 1, 0, @(z) 1-z, 'RelTol', 1e-12);
end

function r = atanIntegrand(eta,zeta, a, b, c, Delta)
    r = zeros(size(zeta));
    for i=1:size(eta,1)
        for j=1:size(eta,2)
            r(i,j) = Delta(zeta(i,j)) / (c(zeta(i,j)) ...
                       + eta(i,j)*b(zeta(i,j)) + eta(i,j)^2*a);
        end
    end
end

