function [fails,passes,relative_errors]=testSlp()

%testSLP: Performs several tests to verify the C-Function slp contained in
%         singleLayerPotential.c
%
%Usage: [fails,passes,relative_errors]=testSlp()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D. The
%             function slp that is tested is supposed to compute
%             the the value of the integral
%             \int_{-1}^{-1}t^{k}log(t*u+v)dt with u,v\in\R^2.
%             100 random cases are tested, quad gives the reference
%             solution.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... errors compaired to the reference solution.
%
%Authors: Samuel Ferraz-Leite
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: SFL (15.07.2009 -> creation and implementation -> 0.4.1)
%         SFL (16.07.2009 -> Updated comments and help -> 0.4.2)
%         SFL (20.07.2009 -> updated to work with new slp version -> 0.4.3)
%         MM  (21.07.2009 -> rules for compiling removed. -> 0.4.4)
%         MM  (24.09.2009 -> Added tests for k=2. -> 0.4.5)
%         MM  (08.10.2009 -> Added tests for k=3, some restructuring, tests for
%                            special cases u=0 or v=0. -> 0.4.6)
%Version: 3.0

  %Initialize variables
  passes = 0;
  fails = 0;
  relative_errors = zeros(432,1);

  %produce 100 random values for u and generate v beiong suffeiciently
  %linearly independant to avoid singularities in the integral
  u = random('unif',-2,2,2,100);
  angle = random('unif',pi/8,7*pi/8,50,1);
  angle(51:100) = random('unif',9*pi/8,15*pi/8,50,1);
  v = zeros(2,100);
  for i=1:100
    v(:,i)=random('unif',0.2,2,1,1)*...
       ([cos(angle(i)),-sin(angle(i));sin(angle(i)),cos(angle(i))]*u(:,i));
  end

  %*** Run tests for k=0,1,2,3.
  for i=1:100
    theTest(4*i-3, 0, u(:,i), v(:,i));
    theTest(4*i-2, 1, u(:,i), v(:,i));
    theTest(4*i-1, 2, u(:,i), v(:,i));
    theTest(4*i-0, 3, u(:,i), v(:,i));
  end

  disp('We are now running some tests for an integrand with a singularity.');
  disp('For testing purposes, we use the quad function which may fails in');
  disp('some cases. If some test cases fail, it might be due to the fact');
  disp('that quad is too inaccurate, but most of the time it works.');
  disp(['The warning "MATLAB:quad:ImproperFcnValue" has been disabled ' ...
          'temporarily.']);
  warning('off', 'MATLAB:quad:ImproperFcnValue');

  %*** Run tests for k=0,1,2,3 and u = [0;0] or v = [0;0].
  for k=0:3
    for i=0:3
      v1 = rand(2,1);
      if (norm(v1) < 1e4*eps)
        v1 = 1e5 .* v1;
      end

      theTest(400+8*k+2*i+1, k, [0;0], v1, 10*eps);

      u2 = rand(2,1);
      if (norm(u2) < 1e4*eps)
        u2 = 1e5 .* u2;
      end
      theTest(400+8*k+2*i+2, k, u2, [0;0], 10*eps);
    end
  end

  warning('on', 'MATLAB:quad:ImproperFcnValue');
  disp('Warning "MATLAB:quad:ImproperFcnValue" enabled.');

  %*** Computes values from slp and reference solutions using quad and
  %    compares them.
  function theTest(i, k, u, v, varargin)
    abs_tol = 0;
    if (nargin == 5)
      abs_tol = varargin{1};
    end

    value = mxSlp(k, u, v);
    f = @(s) f0(s,k, u, v);
    reference = quad(f, -1,1, 1e2*eps);
    absolute_error = abs(reference-value);
    relative_errors(i) = absolute_error/abs(reference);
    if (relative_errors(i) > 1e4*eps && absolute_error > abs_tol)
      fprintf('*** Testcase %d failed!\n', i);
      fprintf('    u=[%g,%g], v=[%g,%g], k=%d\n', u(1), u(2), v(1), v(2), k);
      fprintf('    C-function returned %g   |   quad returned %g\n', ...
        value, reference);
      fprintf('    relative error %g   | absolute error %g', ...
        relative_errors(i), absolute_error);
      fails = fails + 1;
    else     
      passes = passes + 1;
    end
  end
end

function val=f0(s,k,u,v)
  val=zeros(size(s));
  for i=1:length(s)
    val(i)=s(i)^k*log(norm(u.*s(i)+v,2)^2);
  end
end
