function testVUniformConvergence()

%testVUniformConvergence: Computes the discrete solution for V\phi=1 on the
%                         line segment [-1,1] and plots the error in the
%                         energy norm in a double logarithmic scale
%
%Usage: testVUniformConvergence()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D. This is a
%             simple test using that the analytical solution to Symm's
%             integral equation V\phi=1 on the crack [-1,1] is known to
%             have energy norm \pi. The energy of the sequence of solutions
%             computed on uniform meshes is supposed to converge at a rate
%             of approximately 1/2. The test is considered passed if the
%             plotted convergence-graph looks nice. We admit that this is
%             only a unsatisfactory criterion. However V is furthermore
%             validated through tests involving also the discrete Operator
%             matrix K.
%
%Authors: Samuel Ferraz-Leite
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: SFL (15.07.2009 -> creation and implementation -> 0.9.1)
%         MM  (21.07.2009 -> rules for compiling removed -> 0.9.2)
%Version: 3.0

build_time = 0;

coordinates = [-1,0;0,0;1,0];
elements = [1,2;2,3];
enorm_square = zeros(13,1);
dof = [];
for i = 1:13
    tic;
    V = buildV(coordinates,elements);
    build_time = build_time + toc;
    b = rhs(coordinates,elements);
    x = V\b;
    enorm_square(i) = x'*V*x;
    dof = [dof,size(elements,1)];
    [coordinates,elements] = refine(coordinates,elements);
end
disp(['Total matrix build time is ', build_time, '.']);
error=sqrt(pi-enorm_square);
loglog(dof,error,'-ro');
hold on;
loglog([1 10^4],[1 10^(-2)],'--k');

function [coordinates,elements] = refine(coordinates,elements)
K = size(coordinates,1);
N = size(elements,1);
M = N-1;

%*** Allocate memory
elements(M+N,1) = 0;
coordinates(K+M,1) = 0;

%*** Refinement
for j = 1:N
  ab = elements(j,:);
  % midpoint
  coordinates(K+j,:) = 1/2*[1 1]*coordinates(ab,:);
  % new elements
  elements(N+j,:) = [K+j,ab(2)];
  elements(j,:) = [ab(1),K+j];
end

function val = rhs(coordinates,elements)

N = size(elements,1);
val = zeros(N,1);

for j = 1:N
  a=coordinates(elements(j,1),:);
  b=coordinates(elements(j,2),:);
  val(j) = -(b(1)^2-a(1)^2) / 2;
end
