function [fails,passes,absolute_errors] = testWAndKPrime()
%testWAndKPrime: Performs several tests to verify the MEX-Functions buildW
%		 and buildK (KPrime = transpose(K)) contained in
%		 buildW.c and buildK.c
%
%Usage: [fails,passes,relative_errors]=testWAndKPrime()
%
%Description: This file is part of (internal) ABEM2D-M programm packages
%             for the numerical solution of Laplace equation with mixed
%             boundary conditions by use of BEM in 2D.
%	      The functions buildW and buildK that are tested are supposed
%	      to compute the entries of the discrete hypersingular integral
%	      operator W and the discrete adjoint double-layer potential
%	      operator KPrime.
%	      buildK computes the discrete double-layer potential K,
%	      but KPrime = transpose(K).
%	      Tests are performed for several geometries and known solutions.
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... errors compaired to the reference solution.
%
%Authors: Michael Karkulik
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

%History: MK (04.08.2009 - creation and implementation of some tests -> 0.1.1)
%Version: 3.0

addpath ../

fails = 0;
passes = 0;
absolute_errors = zeros(24,1);

%First W1 = 0 is tested for random meshes on several geometries
fprintf('\nStarting Test 1: W1 = 0, mesh-size~800 elements\n\n');
fprintf('Test 1.A, 3 cases: Omega = [0,0.5]^2\n');

%3 cases for the square [0, 0.5]^2
for i = 1:3
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,200,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    W = buildW(coordinates,elements);
    reference = 0;
    value = sum(sum(W));
    absolute_errors(i) = abs(value-reference);
    if (absolute_errors(i)>1e2*eps)
	fails = fails+1;
	fprintf('ARRRGH!\n');
    else
	passes = passes+1;
    end
end

fprintf('Test 1.B, 3 cases: Omega=[0,0.5]^2 rotated by pi/4\n');
%3 cases: square (0,1/2)^2 rotated by angle alpha=pi/4
for i = 4:6
    
    alpha = pi/4;
    rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
    
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,200,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    coordinates = (rotation_matrix*coordinates')';
    
    W = buildW(coordinates,elements);
    reference = 0;
    value = sum(sum(W));
    absolute_errors(i) = abs(reference-value);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

fprintf('Test 1.C, 3 cases: Omega=[0,0.5]^2 rotated by pi/3\n');
%3 cases: square (0,1/2)^2 rotated by angle alpha=pi/3
for i = 7:9
    
    alpha = pi/3;
    rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
    
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,200,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    coordinates = (rotation_matrix*coordinates')';
    
    W = buildW(coordinates,elements);
    reference = 0;
    value = sum(sum(W));
    absolute_errors(i) = abs(reference-value);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

fprintf('Test 1.D, 3 cases: Omega=[0,0.5]^2\\[0.25,0.5]^2');
fprintf(', straight and rotated\n');
%3 cases: L-shaped domain, straight and rotated
for i = 10:12
    
    alpha = (i-10)*pi/5;
    rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
    
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.25,100,1);0.25];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0.25,0.499,100,1);0.25];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0.25];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.25*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.251,0.5,100,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.25*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.249,100,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    if (alpha>0)
        coordinates = (rotation_matrix*coordinates')';
    end
    
    W = buildW(coordinates,elements);
    reference = 0;
    value = sum(sum(W));
    absolute_errors(i) = abs(reference-value);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

fprintf('Test 1.E, 3 cases: Omega=[0,0.5]^2\\conv{(0,0)');
fprintf(',(0.25,0.25),(0,0.25)}, straight and rotated\n');
%3 cases: Z-Shaped domain, straight and rotated
for i = 13:15
    
    alpha = (i-13)*pi/5;
    rotation_matrix = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
    
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,1,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,1,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,1,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.25,0.499,1,1);0.25];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    y_coordinates = [y_coordinates(2:end);0.25];
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0.01,0.25,1,1);0.25];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    [dummy,I] = sort(x_coordinates);
    coordinates = [coordinates;x_coordinates,0.25*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);
    
    direction = [-1;-1];
    direction = direction/norm(direction,2);
    t = random('unif',0.01,sqrt(2)/4-0.0001,1,1);
    [dummy,I] = sort(t);
    for j = 1:length(t)
        new_coordinates(j,:) = ([0.25;0.25]+t(j)*direction)';
    end
    coordinates = [coordinates;new_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];
    
    if (alpha>0)
        coordinates = (rotation_matrix*coordinates')';
    end
    
    W = buildW(coordinates,elements);
    reference = 0;
    value = sum(sum(W));
    absolute_errors(i) = abs(reference-value);
    if (absolute_errors(i)>1e6*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

%Second, test function is u(x,y)=x on the domain (0,0.5)^2,

fprintf('\nStarting Test 2: u (x,y)=x, W[\\gamma_0 u] = 0 on square [0,0.5]^2');
fprintf(', mesh-size~800 elements\n\n');
for i = 16:18
    points_so_far = 0;
    howmany = 10;
    x_coordinates = [0;random('unif',0,0.5,howmany,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,howmany,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,howmany,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,howmany,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    % determine dirichlet data
    uD = coordinates(:,1); % u(x,y) = x
    
    % determine neumann data
    uN = zeros(size(coordinates,1),1);
    midpoints = 0.5*(coordinates(elements(:,1),:)...
                + coordinates(elements(:,2),:));
    uD_midpoints = midpoints(:,1);
    uN(find(uD_midpoints==0)) = -1;
    uN(find(uD_midpoints==0.5)) = 1;

    K = buildK(coordinates,elements);
    W = buildW(coordinates,elements);

    % the test is sum(KPrime*uN - W*uD) = 0
    % we use KPrime*uN = (uN'*K)'
    reference = 0;
    value = sum((uN'*K)' + W*uD);
    absolute_errors(i) = abs(value-reference);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

%Third, test function is u(x,y)=y on the domain (0,0.5)^2,

fprintf('Starting Test 3: u (x,y)=y, W[\\gamma_0 u] = 0 on square [0,0.5]^2');
fprintf(', mesh-size~800 elements\n\n');
for i = 19:21
    points_so_far = 0;
    x_coordinates = [0;random('unif',0,0.5,200,1);0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I=sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,200,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    % determine dirichlet data
    uD = coordinates(:,2);
    
    % determine neumann data
    uN = zeros(size(coordinates,1),1);
    midpoints = 0.5*(coordinates(elements(:,1),:)...
    		     + coordinates(elements(:,2),:));
    uD_midpoints = midpoints(:,2);
    uN(find(uD_midpoints==0)) = -1;
    uN(find(uD_midpoints==0.5)) = 1;

    K = buildK(coordinates,elements);
    W = buildW(coordinates,elements);

    % the test is sum(KPrime*uN + W*uD) = 0
    % we use KPrime*uN = (uN'*K)'
    reference = 0;
    value = sum((uN'*K)' + W*uD);
    absolute_errors(i) = abs(value-reference);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

% Finally, a test with u(x,y) = xy on the square

fprintf('Starting Test 4: u(x,y)=xy, on square [0,0.5]^2');
fprintf(' with a random mesh of size~800\n\n');
for i = 22:24

    points_so_far = 0;
    x_coordinates = [0; random('unif',0,0.5,200,1); 0.5];
    [dummy,I] = unique(x_coordinates,'first');
    I = sort(I);
    x_coordinates = x_coordinates(I);
    coordinates = [x_coordinates,zeros(size(x_coordinates))];
    [dummy,I] = sort(x_coordinates);
    elements = [I(1:end-1),I(2:end)];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.0001,0.5,200,1);0.5];
    [dummy,I] = unique(y_coordinates,'first');
    I = sort(I);
    y_coordinates = y_coordinates(I);
    coordinates = [coordinates;0.5*ones(size(y_coordinates)),y_coordinates];
    [dummy,I] = sort(y_coordinates);
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    x_coordinates = [random('unif',0,0.499,200,1);0];
    [dummy,I] = unique(x_coordinates,'first');
    x_coordinates = x_coordinates(I);
    x_coordinates = [x_coordinates(2:end);0];
    [dummy,I] = sort(x_coordinates,'descend');
    coordinates = [coordinates;x_coordinates,0.5*ones(size(x_coordinates))];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    points_so_far = length(coordinates);

    y_coordinates = [random('unif',0.001,0.499,200,1)];
    [dummy,I] = unique(y_coordinates,'first');
    y_coordinates = y_coordinates(I);
    [dummy,I] = sort(y_coordinates,'descend');    
    coordinates = [coordinates;zeros(size(y_coordinates)),y_coordinates];
    elements = [elements;points_so_far I(1)+points_so_far];
    elements = [elements;I(1:end-1)+points_so_far,I(2:end)+points_so_far];
    elements = [elements;I(end)+points_so_far 1];

    reference = 0;
    uD = coordinates(:,1).*coordinates(:,2);
    uN = zeros(size(elements,1),1);
    midpoints = 0.5*(coordinates(elements(:,1),:)...
    		     + coordinates(elements(:,2),:));
    I = find(midpoints(:,2)==0);
    uN(I) = -midpoints(I,1);
    I = find(midpoints(:,1)==0);
    uN(I) = -midpoints(I,2);
    I = find(midpoints(:,1)==0.5);
    uN(I) = midpoints(I,2);
    I = find(midpoints(:,2)==0.5);
    uN(I) = midpoints(I,1);
    K = buildK(coordinates,elements);
    W = buildW(coordinates,elements);

    value = sum((uN'*K)' + W*uD);
    absolute_errors(i) = abs(value-reference);
    if (absolute_errors(i)>1e2*eps)
        fails = fails+1;
        fprintf('ARRGH!\n');
    else
        passes = passes+1;
    end
end

