#!/usr/bin/perl

use warnings;
use strict;

=pod

=head1 NAME

find_and_replace.pl - Substitutes a perl regular expression in a list of files.

=head1 SYNOPSIS

  find_and_replace.pl <pattern> <replacement> <files...>

C<pattern> is a perl regular expression, C<replacement> is a perl
expression and C<files...> is a list of files. In each file in
C<files...> each occurence of C<pattern> is replaced by
C<replacement>.

=head1 DESCRIPTION

In contrast to C<perl -npe 's/pattern/replacement/g'> this script
provides a diff-like output whenever it finds an occurence of
C<pattern> and asks you if you want to replace it. There is also
the option to answer "always no" (in case you detected an error in
your regular expression) or "always yes" (in case you want the
behaviour of C<perl -npe ...>.

=head1 EXAMPLES

The following example detects version numbers in MATLAB files and
replaces them by "% Version: 3.0" for all "*.m" files in the C<lib>
directory, c.f. man find.

  find_and_replace.pl '\% (?:V|v)ersion: (\d+\.\d+(?:\.\d+)?)' \
    '\% Version: 3.0' $(find .. -name "*.m")

The next example replaces all occurences of "BEM or FEM-BEM" by
"BEM" (in all *.c, *.h and *.m files):

  find_and_replace.pl 'BEM or FEM-BEM' 'BEM' \
    $(find .. -name "*.[chm]")

This example adjusts copyright information:

  find_and_replace.pl '\% \(C\) (\d+)(?:\-\d+)?' \
    '\% (C) ${1}-2012' $(find .. -name "*.m")

=cut

our $pattern      = shift @ARGV;
our $replacement  = shift @ARGV;
my  @files        = @ARGV;

unless (@files)
{
  print <<EOF;
Usage: find_and_replace.pl <pattern> <replacement> <files...>

More information is available by using perldoc \$PATH_TO/find_and_replace.pl.
EOF
  exit 0;
}

my $always_answer_by = undef;

for my $file (@files)
{
  open IN, '<', $file or die "Could not open $file: $!\n";
  my @lines = <IN>;
  close IN;

  my @output;

  my $line_counter = 1;
  for my $line (@lines)
  {
    if ($line =~ m/$pattern/)
    {
      my $newline = $line;
      $newline =~ s/$pattern/'"'.$replacement.'"'/gee;

      print "\# $file:$line_counter:\n",
        "replace> $line",
        "by     < $newline";

      my $answer;

      if (defined $always_answer_by)
      {
        $answer = $always_answer_by;
      }
      else
      {
        my $answer_valid;
        print "Do you want to replace? ",
              "[n=no | ay=always yes | an=always no | *=yes] ";
        $answer = <STDIN>;
        chomp $answer;
        $answer = lc($answer);

        if ($answer eq 'ay')
        {
          $answer = 'y';
          $always_answer_by = 'y';
        }
        elsif ($answer eq 'an')
        {
          $answer = 'n';
          $always_answer_by = 'n';
        }
      }

      if ($answer eq 'n')
      {
        push @output, $line;
      }
      else
      {
        push @output, $newline;
      }
    }
    else
    {
      push @output, $line;
    }
  }

  open OUT, '>', $file or die "Could not open file: $!\n";
  print OUT for (@output);
  close OUT;
}

=head1 AUTHORS

Markus Mayr - L<markus.mayr@tuwien.ac.at>

=cut

